<?php
/**
* NOTICE OF LICENSE.
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software
* and associated documentation files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge, publish, distribute,
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial
* portions of the Software.
*
* The software is provided "as is", without warranty of any kind, express or implied including 
* but not limited to the warranties of merchantability, fitness for a particular purpose and
* noninfrigement. In no event shall the authors or copyright holdes be liable for any claim, 
* damages or the liability, whether in an action of contract, tort or otherwise, arising from, 
* out of or in connection with the software or the use or other dealings in the software.
*
* @author    Diabolo SARL <ecommerce@diazol.com>
* @copyright 2011 - 2016 Diabolo SARL
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

require_once 'import/classes/publication.class.php';
require_once 'import/classes/import.class.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class ArtifactCaisseSync extends Module
{
    private $_dirs;
    private $_conversions;

    public function __construct()
    {
        $this->name = 'artifactcaissesync';
        $this->tab = 'administration';
        $this->version = '1.2.1';
        $this->author = 'Diabolo SARL';
        $this->need_instance = 0;
        $this->bootstrap = true;
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => '1.7');
        $this->secure_key = Tools::encrypt($this->name);
        $this->module_key = '49aa270778f82a97c74c1cd2ab75cad9';
        $this->confirmUninstall = $this->l('Uninstalling Artifact Caisse Sync will completely remove sync from Artifact to Prestashop ! Are you sure you want to do it ?');

        if (version_compare(_PS_VERSION_, '1.5.0.0 ', '>=')) {
            $this->multishop_context = Shop::CONTEXT_ALL;
        }

        parent::__construct();

        // list of dirs which must be created
        $this->_dirs = array();
        $this->_dirs[0] = _PS_ROOT_DIR_.'/arte-fact/';
        $this->_dirs[1] = _PS_ROOT_DIR_.'/arte-fact/outputs/';
        $this->_dirs[2] = _PS_ROOT_DIR_.'/arte-fact/inputs/';
        $this->_dirs[3] = _PS_ROOT_DIR_.'/arte-fact/inputs/files_xml/';
        $this->_dirs[4] = _PS_ROOT_DIR_.'/arte-fact/inputs/images/';
        $this->_dirs[5] = _PS_ROOT_DIR_.'/arte-fact/inputs/images/arbre/';
        $this->_dirs[6] = _PS_ROOT_DIR_.'/arte-fact/inputs/images/marques/';
        $this->_dirs[7] = _PS_ROOT_DIR_.'/arte-fact/inputs/images/produits/';
        $this->_dirs[8] = _PS_ROOT_DIR_.'/arte-fact/logs/';
        $this->_cache_dir = _PS_ROOT_DIR_.'/tools/smarty/cache/';

        $this->displayName = $this->l('Artifact Caisse Sync');
        $this->description = $this->l('Synchronisation des données Artifact / Prestashop');
    }

    public function getVersion()
    {
        return $this->version;
    }

    public function install()
    {
        //compatibilité
        $testVersion = (int)(str_replace('.', '', _PS_VERSION_));
        //if($testVersion<1410 || $testVersion>1560){
        if ($testVersion < 1410) {
            $this->_errors[] = $this->l('The version of your ArtEFact Module is not compliant with your PrestaShop version.');
            return false;
        }

        // make folder for storing files
        if (!empty($this->_dirs)) {
            $htmlError = '<ul>';
            $error = '';
            foreach ($this->_dirs as $dir) {
                if (!file_exists($dir)) {
                    if (!mkdir($dir, 0777)) {
                        $error .= '<li>'.$dir.'</li>';
                        $this->_errors[] = $this->l('directory not create');
                    }
                }
            }
            if (!empty($error)) {
                $htmlError .= $error.'</ul>';
                echo parent::displayError('Permissions denied: can\'t create folder'.$htmlError);
                return false;
            }
        }

        // Clear cache
        if (is_dir($this->_cache_dir)) {
            $files_dir = scandir($this->_cache_dir);
            foreach ($files_dir as $k => $file) {
                $pos = strpos($file, '.tpl');
                if ($pos !== false) {
                    @unlink($this->_cache_dir.$file);
                }
            }
        }

        // update tables
        Db::getInstance()->Execute(
            'ALTER TABLE `'._DB_PREFIX_.'product_attribute` 
			CHANGE `reference` `reference` VARCHAR(64) CHARACTER SET utf8 COLLATE utf8_general_ci NULL'
        );

        Db::getInstance()->Execute(
            'ALTER TABLE `'._DB_PREFIX_.'product_attribute` 
			CHANGE `default_on` `default_on` TINYINT(1) UNSIGNED NULL DEFAULT NULL'
        );

        Db::getInstance()->Execute(
            'ALTER TABLE `'._DB_PREFIX_.'feature_value_lang` 
			CHANGE `value` `value` VARCHAR( 325 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL '
        );

        Db::getInstance()->Execute(
            'UPDATE IGNORE `'._DB_PREFIX_.'product_attribute` SET default_on = NULL'
        );

        Db::getInstance()->Execute(
            'UPDATE IGNORE `'._DB_PREFIX_.'product_attribute` SET default_on = 1'
        );

        Db::getInstance()->Execute(
            'ALTER TABLE `'._DB_PREFIX_.'product_attribute_shop` ADD UNIQUE `id_product_attribute` (`id_product_attribute`,`id_shop`,`default_on`)'
        );

        Db::getInstance()->Execute(
            'ALTER TABLE `'._DB_PREFIX_.'product` ADD `artifact_reference` VARCHAR(64) NULL DEFAULT NULL AFTER `supplier_reference`'
        );
        Db::getInstance()->Execute(
            'ALTER TABLE `'._DB_PREFIX_.'product_attribute` ADD `artifact_reference` VARCHAR(96) NULL DEFAULT NULL AFTER `supplier_reference`'
        );

        // insert conversion rules
        Db::getInstance()->Execute(
            'CREATE TABLE IF NOT EXISTS `'._DB_PREFIX_.'artefact_conversions_rules` (
				`imports_conversions_rules_id` INT(11) NOT NULL AUTO_INCREMENT,
				`imports_conversions_type` ENUM("brand","category","taxes","taxes_group","attribute_group","attribute","feature","feature_value", "product", "declinaison", "product_origine", "attribute_to_product") NOT NULL DEFAULT "category",
				`imports_conversions_rules_symbol` VARCHAR(325) DEFAULT NULL,
				`imports_conversions_rules_original` INT(11) DEFAULT NULL,
				`updated` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
				PRIMARY KEY  (`imports_conversions_rules_id`),
				UNIQUE KEY `imports_conversions_id` (`imports_conversions_type`,`imports_conversions_rules_symbol`),
				KEY `imports_conversions_rules_original` (`imports_conversions_rules_original`)
			) ENGINE=MYISAM DEFAULT CHARSET=utf8'
        );

        Db::getInstance()->Execute(
            'CREATE TABLE IF NOT EXISTS '._DB_PREFIX_.'artefact_orders_export (
				`id_artifact_export` int(10) unsigned NOT NULL AUTO_INCREMENT,
				`id_order`           int(10) unsigned NOT NULL,
				`order_is_exported`  int(1) unsigned NOT NULL DEFAULT 0,
				`updated`           TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
				PRIMARY KEY (`id_artifact_export`),
				KEY `id_order` (`id_order`),
				KEY `order_is_exported` (`order_is_exported`),
				KEY `updated` (`updated`)
			) ENGINE=InnoDB DEFAULT CHARSET=utf8'
        );

        return parent::install() &&

            Configuration::updateValue('PS_ARTIFACT_DEBUG', '1')
            && Configuration::updateValue('PS_ARTIFACT_ACTIVE_SYNC', '1')

            && Configuration::updateValue('PS_PRODUCT_ACTIVATION', '0')
            && Configuration::updateValue('PS_PRODUCT_STOCK_ACTIVATION', '0')
            && Configuration::updateValue('PS_COMBINATION_STOCK_ACTIVATION', '0')
            && Configuration::updateValue('PS_CATEGORIE_ACTIVATION', '0')
            && Configuration::updateValue('PS_BRAND_ACTIVATION', '0')
            && Configuration::updateValue('PS_SEARCH_INDEXATION', '1')
            && Configuration::updateValue('PS_ARF_SPLIT_ARF_DESC', '1')
            && Configuration::updateValue('PS_ARF_ALLOW_PROD_UPD_CAT', '0')
            && Configuration::updateValue('PS_ARF_ALLOW_PROD_UPD_NAME', '0')
            && Configuration::updateValue('PS_ARF_ALLOW_PROD_UPD_DESC', '0')
            && Configuration::updateValue('PS_ARF_ALLOW_PROD_UPD_IMG', '0')
            && Configuration::updateValue('PS_ARF_ALLOW_CAT_UPD', '0')
            && Configuration::updateValue('PS_ARF_ALLOW_BRAND_UPD', '0')
            && Configuration::updateValue('PS_ARF_ALLOW_PROD_SPECS_UPD', '0')
            && Configuration::updateValue('PS_ARF_FIRST_RUN', '0')
			&& Configuration::updateValue('PS_ARF_MAX_ID_ORDER', Db::getInstance()->getValue('SELECT MAX(id_order) FROM '._DB_PREFIX_.'orders ')) ;

    }

    public function uninstall()
    {
        // drop tables
        Db::getInstance()->Execute(
            'DROP TABLE `'._DB_PREFIX_.'artefact_conversions_rules`'
        );
        Db::getInstance()->Execute(
            'DROP TABLE `'._DB_PREFIX_.'artefact_orders_export`'
        );

        // update tables back to their initial structures
        Db::getInstance()->Execute(
            'ALTER TABLE `'._DB_PREFIX_.'product_attribute` 
			CHANGE `reference` `reference` VARCHAR(32) CHARACTER SET utf8 COLLATE utf8_general_ci NULL'
        );

        Db::getInstance()->Execute(
            'ALTER TABLE `'._DB_PREFIX_.'product_attribute` 
			CHANGE `default_on` `default_on` TINYINT(1) UNSIGNED NULL DEFAULT \'0\''
        );

        Db::getInstance()->Execute(
            'ALTER TABLE `'._DB_PREFIX_.'feature_value_lang` 
			CHANGE `value` `value` VARCHAR( 255 ) CHARACTER SET utf8 COLLATE utf8_general_ci NULL DEFAULT NULL '
        );

        Db::getInstance()->Execute(
            'ALTER TABLE `'._DB_PREFIX_.'product_attribute_shop`
			 DROP INDEX `id_product_attribute` '
        );

        // delete directories
        foreach ($this->_dirs as $dir) {
            $this->rmArtEfact($dir);
        }

        Configuration::deleteByName('PS_ARTIFACT_DEBUG');
        Configuration::deleteByName('PS_ARTIFACT_ACTIVE_SYNC');
        Configuration::deleteByName('PS_PRODUCT_ACTIVATION');
        Configuration::deleteByName('PS_PRODUCT_STOCK_ACTIVATION');
        Configuration::deleteByName('PS_COMBINATION_STOCK_ACTIVATION');
        Configuration::deleteByName('PS_CATEGORIE_ACTIVATION');
        Configuration::deleteByName('PS_BRAND_ACTIVATION');
        Configuration::deleteByName('PS_ARF_SPLIT_ARF_DESC');
        Configuration::deleteByName('PS_ARF_ALLOW_PROD_UPD_CAT');
        Configuration::deleteByName('PS_ARF_ALLOW_PROD_UPD_NAME');
        Configuration::deleteByName('PS_ARF_ALLOW_PROD_UPD_DESC');
        Configuration::deleteByName('PS_ARF_ALLOW_PROD_UPD_IMG');
        Configuration::deleteByName('PS_ARF_ALLOW_CAT_UPD');
        Configuration::deleteByName('PS_ARF_ALLOW_BRAND_UPD');
        Configuration::deleteByName('PS_ARF_ALLOW_PROD_SPECS_UPD');

		Configuration::deleteByName('PS_ARF_FIRST_RUN');
		Configuration::deleteByName('PS_ARF_MAX_ID_ORDER');

        return parent::uninstall();
    }

    public function getContent()
    {
        $output = '<h2>'.$this->displayName.'</h2>';
        $output .= '<h2>'.$this->l('For further information about Artifact for Prestashop, please check our website').' :  <a href="http://www.diazol.com/logiciels/arte-fact/didacticiels/" target="_blank">diazol.com</a></h2>';
        $output .= '<div>
			'.$this->l('Setup and install').' :
			<ul>
				<li>'.$this->l('Check that the directories').' : <br />
					<ul>
						<li><strong>/modules/artifactcaissesync</strong> '.$this->l('and subdirectories').'</li>
						<li><strong>/arte-fact</strong> '.$this->l('and subdirectories').'</li>
					</ul>
					'.$this->l('have sufficient permissions (read / write)').'.
				</li>
				<li>'.$this->l('Once your integration tests done, Set up your Cron task table (crontab) for execution of the following scripts').' :
					<ul>
						<li><strong>/modules/artifactcaissesync/export/export.php</strong> '.$this->l('(every 5 min to every minute) as required').' ;</li>
						<li><strong>/modules/artifactcaissesync/import/import.php</strong> '.$this->l('(every 5 min to every minute) as required').' ;</li>
					</ul>
				</li>
			</ul>
		</div>';

        if (Tools::isSubmit('submitArtifactSpecials')) {
            Configuration::updateValue('PS_ARTIFACT_VERSION', Tools::getValue('artifact_version'));
			Configuration::updateValue('PS_ARF_MAX_ID_ORDER', Tools::getValue('artifact_max_id_order'));
            Configuration::updateValue('PS_ARTIFACT_DEBUG', Tools::getValue('artifact_debug'));
            Configuration::updateValue('PS_ARTIFACT_ACTIVE_SYNC', Tools::getValue('artifact_active'));
            Configuration::updateValue('PS_ARF_FIRST_RUN', Tools::getValue('artifact_firstrun'));

            $output .= $this->displayConfirmation($this->l('Settings updated'));
        } elseif (Tools::isSubmit('submitArtifactPublication')) {
            Configuration::updateValue('PS_PRODUCT_CATEGORIES_PARENT', Tools::getValue('productCategoriesParent'));
            Configuration::updateValue('PS_PRODUCT_ACTIVATION', Tools::getValue('product_activation'));
            Configuration::updateValue('PS_PRODUCT_STOCK_ACTIVATION', Tools::getValue('product_stock_activation'));
            Configuration::updateValue('PS_COMBINATION_STOCK_ACTIVATION', Tools::getValue('combination_stock_activation'));
            Configuration::updateValue('PS_CATEGORIE_ACTIVATION', Tools::getValue('categorie_activation'));
            Configuration::updateValue('PS_BRAND_ACTIVATION', Tools::getValue('brand_activation'));

            $publication = new ArtifactPublication();

            if (Tools::getValue('product_activation') == 1) {
                $publication->publicationActiveProducts();
            } else {
                $publication->publicationNonActiveProducts();
            }

            if (Tools::getValue('product_stock_activation') == 1) {
                $publication->publicationActiveStockProducts();
            } else {
                $publication->publicationNonActiveStockProducts();
            }

            if (Tools::getValue('categorie_activation') == 1) {
                $publication->publicationActiveCategories();
            } else {
                $publication->publicationNonActiveCategories();
            }

            if (Tools::getValue('brand_activation') == 1) {
                $publication->publicationActiveBrands();
            }


            $output .= $this->displayConfirmation($this->l('Publication control updated'));
        } elseif (Tools::isSubmit('submitArtifactAdvanced')) {
            Configuration::updateValue('PS_ARF_SPLIT_ARF_DESC', Tools::getValue('splitArtifactDescription'));
            Configuration::updateValue('PS_ARF_ALLOW_PROD_UPD_CAT', Tools::getValue('allowPrestashopProdUpdCategory'));
            Configuration::updateValue('PS_ARF_ALLOW_PROD_UPD_NAME', Tools::getValue('allowPrestaProdUpdName'));
            Configuration::updateValue('PS_ARF_ALLOW_PROD_UPD_DESC', Tools::getValue('allowPrestaProdUpdDesc'));
            Configuration::updateValue('PS_ARF_ALLOW_PROD_UPD_IMG', Tools::getValue('allowPrestaProdUpdImg'));
            Configuration::updateValue('PS_ARF_ALLOW_CAT_UPD', Tools::getValue('allowPrestashopCategorieUpdate'));
            Configuration::updateValue('PS_ARF_ALLOW_BRAND_UPD', Tools::getValue('allowPrestashopBrandUpdate'));
            Configuration::updateValue('PS_ARF_ALLOW_PROD_SPECS_UPD', Tools::getValue('allowPrestashopProductSpecs'));

            $output .= $this->displayConfirmation($this->l('Advanced Settings updated'));
        } elseif (Tools::getValue('submitArtifactTruncateCatalog') && Tools::getValue('checkArtifactTruncateCatalog')) {
            self::truncate('catalog');
            $output .= $this->displayConfirmation($this->l('Catalog truncated'));
        }

        $output .= '
		<script type="text/javascript">
			$(document).ready(function(){
				$("button[name=submitArtifactTruncateCatalog]").click(function(){
					if ($(\'#checkArtifactTruncateCatalog_on\').attr(\'checked\') != "checked") {
						alert(\''.addslashes(html_entity_decode($this->l('Please read the disclaimer and click "Yes" above'))).'\');
						return false;
					}
					if (confirm(\''.addslashes(html_entity_decode($this->l('Are you sure that you want to delete all catalog data?'))).'\'))
						return true;
					return false;
				});
			});
		</script>';

        return $output .= $this->renderForm();
    }

    private function rmArtEfact($dir)
    {
        if (is_dir($dir)) {
            $objects = scandir($dir);
            foreach ($objects as $object) {
                if ($object != '.' && $object != '..') {
                    if (filetype($dir.'/'.$object) == 'dir') {
                        $this->rmArtEfact($dir.'/'.$object);
                    } else {
                        unlink($dir.'/'.$object);
                    }
                }
            }
            reset($objects);
            rmdir($dir);
        }
    }

    public function renderForm()
    {
        $fields_form_1 = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'text',
                        'is_bool' => false,
                        'label' => $this->l('Artifact Version (API)'),
                        'name' => 'artifact_version',
                        'desc' => $this->l('Enter above the 3-digit Artifact E-commerce API version has found in Artifact Software ( Aide > A Propos'),
                    ),
					array(
						'type' => 'text',
						'is_bool' => false,
						'label' => $this->l('Export Valid orders following this Order ID'),
						'name' => 'artifact_max_id_order',
						'desc' => $this->l('Artifact will sync valid orders after the OrderID filled in the above field. Enter 0, if you want all valid orders to be sync.')
					),
                    array(
                        'type' => 'radio',
                        'is_bool' => true,
                        'label' => $this->l('Artifact Debug'),
                        'name' => 'artifact_debug',
                        'desc' => $this->l('Debug mode will log import information in the following directory : /arte-fact/logs/'),
                        'values' => array(
                            array(
                                'id' => 'artifact_debug_on',
                                'value' => 1,
                                'label' => $this->l('Debug'),
                            ),
                            array(
                                'id' => 'artifact_debug_off',
                                'value' => 0,
                                'label' => $this->l('Production'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Save incoming Artifact Data'),
                        'name' => 'artifact_active',
                        'desc' => $this->l('Activating this option will create a zip archive of the input folder in the artefact bkp folder before processing the import function !'),
                    'values' => array(
                            array(
                                'id' => 'artifact_active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'artifact_active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    )
                    /*,
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Artifact Synchronisation Initiale'),
                        'name' => 'artifact_firstrun',
                        'desc' => $this->l('Activating this option allow the first sync to search for existing brands, taxes and products instead of automatically creating them.'),
                    'values' => array(
                            array(
                                'id' => 'artifact_firstrun_on',
                                'value' => 1,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id' => 'artifact_firstrun_off',
                                'value' => 0,
                                'label' => $this->l('Disabled')
                            )
                        )
                    )
                    */
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                    'class' => 'btn btn-default pull-right',
                    'name' => 'submitArtifactSpecials',
                    'id' => 'submitArtifactSpecials',
                ),
            ),
        );

        $fields_form_2 = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Publication Control'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Products Categories Association'),
                        'name' => 'productCategoriesParent',
                        'desc' => $this->l('Associate all products with each parents categories instead of the single child Artifact category'),
                        'values' => array(
                            array(
                                'id' => 'productCategoriesParent_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'productCategoriesParent_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Product content publication control'),
                        'name' => 'product_activation',
                        'desc' => $this->l('Unpublish product without any image and description'),
                        'values' => array(
                            array(
                                'id' => 'product_activation_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'product_activation_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Product stock publication control'),
                        'name' => 'product_stock_activation',
                        'desc' => $this->l('Unpublish product without stock'),
                        'values' => array(
                            array(
                                'id' => 'product_stock_activation_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'product_stock_activation_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Combination stock publication control'),
                        'name' => 'combination_stock_activation',
                        'desc' => $this->l('not create / delete combination without stock'),
                        'values' => array(
                            array(
                                'id' => 'combination_stock_activation_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'combination_stock_activation_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),

                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Categories publication control'),
                        'name' => 'categorie_activation',
                        'desc' => $this->l('Unpublish categories without any activated product'),
                        'values' => array(
                            array(
                                'id' => 'categorie_activation_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'categorie_activation_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Brands publication control'),
                        'name' => 'brand_activation',
                        'desc' => $this->l('Unpublish brands/manufacturers with any activated product'),
                        'values' => array(
                            array(
                                'id' => 'brand_activation_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'brand_activation_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                    'class' => 'btn btn-default pull-right',
                    'name' => 'submitArtifactPublication',
                    'id' => 'submitArtifactPublication',
                ),
            ),
        );

        $fields_form_3 = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Advanced Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Split Artifact product description'),
                        'name' => 'splitArtifactDescription',
                        'desc' => $this->l('In order to fill your catalog product short and long description, you can use the following escape chars "...[linebreak]"in Artifact at the end of your short description.').'<br /> example : <p>ceci est la description courte de mon produit ...<br/><br/>ici commence la description longue de mon produit !</p>',
                        'values' => array(
                            array(
                                'id' => 'splitArtifactDescription_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'splitArtifactDescription_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Allow the use of prestashop back-office to update product categories association'),
                        'name' => 'allowPrestashopProdUpdCategory',
                        'values' => array(
                            array(
                                'id' => 'allowPrestashopProdUpdCategory_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'allowPrestashopProdUpdCategory_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Allow the use of prestashop back-office to update product name'),
                        'name' => 'allowPrestaProdUpdName',
                        'values' => array(
                            array(
                                'id' => 'allowPrestaProdUpdName_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'allowPrestaProdUpdName_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Allow the use of prestashop back-office to update product descriptions'),
                        'name' => 'allowPrestaProdUpdDesc',
                        'values' => array(
                            array(
                                'id' => 'allowPrestaProdUpdDesc_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'allowPrestaProdUpdDesc_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Allow the use of prestashop back-office to manage product images'),
                        'desc' => $this->l('Use Artifact E-commerce params accordingly in order to fasten transfers (tout sauf les images)'),
                        'name' => 'allowPrestaProdUpdImg',
                        'values' => array(
                            array(
                                'id' => 'allowPrestaProdUpdImg_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'allowPrestaProdUpdImg_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Allow the use of prestashop back-office to update products technical data'),
                        'desc' => $this->l('You will be able to edit/add technical data (caractéristiques) on product detail'),
                        'name' => 'allowPrestashopProductSpecs',
                        'values' => array(
                            array(
                                'id' => 'allowPrestashopProductSpecs_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'allowPrestashopProductSpecs_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Allow the use of prestashop back-office to update categories information'),
                        'desc' => $this->l('You will be able to edit categories name, translations, order'),
                        'name' => 'allowPrestashopCategorieUpdate',
                        'values' => array(
                            array(
                                'id' => 'allowPrestashopCategorieUpdate_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'allowPrestashopCategorieUpdate_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Allow the use of prestashop back-office to update brands information'),
                        'desc' => $this->l('You will be able to edit brands name, translations, order'),
                        'name' => 'allowPrestashopBrandUpdate',
                        'values' => array(
                            array(
                                'id' => 'allowPrestashopBrandUpdate_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'allowPrestashopBrandUpdate_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                    'class' => 'btn btn-default pull-right',
                    'name' => 'submitArtifactAdvanced',
                    'id' => 'submitArtifactAdvanced',
                ),
            ),
        );

        $fields_form_4 = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Catalog Reset'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'is_bool' => true,
                        'label' => $this->l('Activate Prestashop Catalog Reset'),
                        'name' => 'checkArtifactTruncateCatalog',
                        'desc' => $this->l('I understand that all the catalog data will be removed without possible rollback: products, features, categories, tags, images, prices, attachments, scenes, stocks, attribute groups and values, manufacturers, suppliers as the extra Artifact information...'),
                        'values' => array(
                            array(
                                'id' => 'checkArtifactTruncateCatalog_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'checkArtifactTruncateCatalog_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            ),
                        ),
                    ),
                ),
                'submit' => array(
                    'title' => $this->l('Delete catalog'),
                    'class' => 'btn btn-default pull-right',
                    'name' => 'submitArtifactTruncateCatalog',
                    'id' => 'submitArtifactTruncateCatalog',
                ),
            ),
        );

        $helper = new HelperForm();
        $helper->module = $this;
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $this->fields_form = array();
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        );

        return $helper->generateForm(array($fields_form_1, $fields_form_2, $fields_form_3, $fields_form_4));
    }

    public function getConfigFieldsValues()
    {
        return array(

            'artifact_version' => Tools::getValue('artifact_version', Configuration::get('PS_ARTIFACT_VERSION')),
			'artifact_max_id_order' => Tools::getValue('artifact_max_id_order', Configuration::get('PS_ARF_MAX_ID_ORDER')),
            'artifact_debug' => Tools::getValue('artifact_debug', Configuration::get('PS_ARTIFACT_DEBUG')),
            'artifact_active' => Tools::getValue('artifact_active', Configuration::get('PS_ARTIFACT_ACTIVE_SYNC')),
            'artifact_firstrun' => Tools::getValue('artifact_firstrun', Configuration::get('PS_ARF_FIRST_RUN')),

            'productCategoriesParent' => Tools::getValue('productCategoriesParent', Configuration::get('PS_PRODUCT_CATEGORIES_PARENT')),
            'product_activation' => Tools::getValue('product_activation', Configuration::get('PS_PRODUCT_ACTIVATION')),
            'product_stock_activation' => Tools::getValue('product_stock_activation', Configuration::get('PS_PRODUCT_STOCK_ACTIVATION')),
            'combination_stock_activation' => Tools::getValue('combination_stock_activation', Configuration::get('PS_COMBINATION_STOCK_ACTIVATION')),
            'categorie_activation' => Tools::getValue('categorie_activation', Configuration::get('PS_CATEGORIE_ACTIVATION')),
            'brand_activation' => Tools::getValue('brand_activation', Configuration::get('PS_BRAND_ACTIVATION')),

            'splitArtifactDescription' => Tools::getValue('splitArtifactDescription', Configuration::get('PS_ARF_SPLIT_ARF_DESC')),

            'allowPrestashopProdUpdCategory' => Tools::getValue('allowPrestashopProdUpdCategory', Configuration::get('PS_ARF_ALLOW_PROD_UPD_CAT')),
            'allowPrestaProdUpdName' => Tools::getValue('allowPrestaProdUpdName', Configuration::get('PS_ARF_ALLOW_PROD_UPD_NAME')),
            'allowPrestaProdUpdDesc' => Tools::getValue('allowPrestaProdUpdDesc', Configuration::get('PS_ARF_ALLOW_PROD_UPD_DESC')),
            'allowPrestaProdUpdImg' => Tools::getValue('allowPrestaProdUpdImg', Configuration::get('PS_ARF_ALLOW_PROD_UPD_IMG')),
            'allowPrestashopCategorieUpdate' => Tools::getValue('allowPrestashopCategorieUpdate', Configuration::get('PS_ARF_ALLOW_CAT_UPD')),
            'allowPrestashopBrandUpdate' => Tools::getValue('allowPrestashopBrandUpdate', Configuration::get('PS_ARF_ALLOW_BRAND_UPD')),
			'allowPrestashopProductSpecs' => Tools::getValue('allowPrestashopProductSpecs', Configuration::get('PS_ARF_ALLOW_PROD_SPECS_UPD')),
            'checkArtifactTruncateCatalog' => 0,
        );
    }

    private function _getConversions()
    {
        $conversions = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'artefact_conversions_rules`', false, false);

        while ($row = Db::getInstance()->nextRow($conversions)) {
            $this->_conversions[$row['imports_conversions_type']][$row['imports_conversions_rules_symbol']] = $row['imports_conversions_rules_original'];
        }

        $languages = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'lang`', false, false);

        while ($row = Db::getInstance()->nextRow($languages)) {
            $this->_languages[Tools::strtolower($row['iso_code'])] = $row['id_lang'];
        }
    }

    public function truncate()
    {
        $db = Db::getInstance();

        // let's first clean only taxes resulting from artifact sync.
        $this->_getConversions();
        if (is_array($this->_conversions['taxes']) && count($this->_conversions['taxes']) > 0) {
            foreach ($this->_conversions['taxes'] as $key => $value) {
                $db->execute('DELETE FROM `'._DB_PREFIX_.'tax` WHERE id_tax = '.$value);
                $db->execute('DELETE FROM `'._DB_PREFIX_.'tax_lang` WHERE id_tax = '.$value);
                $db->execute('DELETE FROM `'._DB_PREFIX_.'tax_rule` WHERE id_tax = '.$value);
            }
        }
        if (is_array($this->_conversions['taxes_group']) && count($this->_conversions['taxes_group']) > 0) {
            foreach ($this->_conversions['taxes_group'] as $key => $value) {
                $db->execute('DELETE FROM `'._DB_PREFIX_.'tax_rules_group` WHERE id_tax_rules_group = '.$value);
                $db->execute('DELETE FROM `'._DB_PREFIX_.'tax_rules_group_shop` WHERE id_tax_rules_group = '.$value);
            }
        }

        $id_home = $this->getMultiShopValues('PS_HOME_CATEGORY');
        $id_root = $this->getMultiShopValues('PS_ROOT_CATEGORY');
        $db->execute('DELETE FROM `'._DB_PREFIX_.'category` WHERE id_category NOT IN ('.implode(',', array_map('intval', $id_home)).', '.implode(',', array_map('intval', $id_root)).')');
        $db->execute('DELETE FROM `'._DB_PREFIX_.'category_lang` WHERE id_category NOT IN ('.implode(',', array_map('intval', $id_home)).', '.implode(',', array_map('intval', $id_root)).')');
        $db->execute('DELETE FROM `'._DB_PREFIX_.'category_shop` WHERE id_category NOT IN ('.implode(',', array_map('intval', $id_home)).', '.implode(',', array_map('intval', $id_root)).')');

        foreach (scandir(_PS_CAT_IMG_DIR_) as $dir) {
            if (preg_match('/^[0-9]+(\-(.*))?\.jpg$/', $dir)) {
                unlink(_PS_CAT_IMG_DIR_.$dir);
            }
        }

        Db::getInstance()->Execute(
            'ALTER TABLE `'._DB_PREFIX_.'product` ADD `artifact_reference` VARCHAR(64) NULL DEFAULT NULL AFTER `supplier_reference`'
        );
        Db::getInstance()->Execute(
            'ALTER TABLE `'._DB_PREFIX_.'product_attribute` ADD `artifact_reference` VARCHAR(96) NULL DEFAULT NULL AFTER `supplier_reference`'
        );


        $tables = array(
            'product',
            'product_shop',
            'feature_product',
            'product_lang',
            'category_product',
            'product_tag',
            'tag',
            'image',
            'image_lang',
            'image_shop',
            'specific_price',
            'specific_price_priority',
            'product_carrier',
            'cart_product',
            /*
            'compare_product',
            */
            'product_attachment',
            'product_country_tax',
            'product_download',
            'product_group_reduction_cache',
            'product_sale',
            'product_supplier',
            'warehouse_product_location',
            'stock',
            'stock_available',
            'stock_mvt',
            'customization',
            'customization_field',
            'supply_order_detail',
            'attribute_impact',
            'product_attribute',
            'product_attribute_shop',
            'product_attribute_combination',
            'product_attribute_image',
            'attribute',
            'attribute_impact',
            'attribute_lang',
            'attribute_group',
            'attribute_group_lang',
            'attribute_group_shop',
            'attribute_shop',
            'product_attribute',
            'product_attribute_shop',
            'product_attribute_combination',
            'product_attribute_image',
            'stock_available',
            'manufacturer',
            'manufacturer_lang',
            'manufacturer_shop',
            'supplier',
            'supplier_lang',
            'supplier_shop',
            'customization',
            'customization_field',
            'customization_field_lang',
            'customized_data',
            'feature',
            'feature_lang',
            'feature_product',
            'feature_shop',
            'feature_value',
            'feature_value_lang',
            'pack',
            /*
            'scene_products',
            'scene',
            'scene_category',
            'scene_lang',
            'scene_products',
            'scene_shop',
            */
            'search_index',
            'search_word',
            'specific_price',
            'specific_price_priority',
            'specific_price_rule',
            'specific_price_rule_condition',
            'specific_price_rule_condition_group',
            'stock',
            'stock_available',
            'stock_mvt',
            'warehouse',
            'artefact_conversions_rules',
            'artefact_orders_export',
        );

        foreach ($tables as $table) {
            $db->execute('TRUNCATE TABLE `'._DB_PREFIX_.bqSQL($table).'`');
        }

        $db->execute('DELETE FROM `'._DB_PREFIX_.'address` WHERE id_manufacturer > 0 OR id_supplier > 0 OR id_warehouse > 0');

        Image::deleteAllImages(_PS_PROD_IMG_DIR_);
        if (!file_exists(_PS_PROD_IMG_DIR_)) {
            mkdir(_PS_PROD_IMG_DIR_);
        }

        foreach (scandir(_PS_MANU_IMG_DIR_) as $dir) {
            if (preg_match('/^[0-9]+(\-(.*))?\.jpg$/', $dir)) {
                unlink(_PS_MANU_IMG_DIR_.$dir);
            }
        }
        foreach (scandir(_PS_SUPP_IMG_DIR_) as $dir) {
            if (preg_match('/^[0-9]+(\-(.*))?\.jpg$/', $dir)) {
                unlink(_PS_SUPP_IMG_DIR_.$dir);
            }
        }

        self::clearAllCaches();
    }

    protected static function clearAllCaches()
    {
        $index = file_exists(_PS_TMP_IMG_DIR_.'index.php') ? Tools::file_get_contents(_PS_TMP_IMG_DIR_.'index.php') : '';
        Tools::deleteDirectory(_PS_TMP_IMG_DIR_, false);
        file_put_contents(_PS_TMP_IMG_DIR_.'index.php', $index);
        Context::getContext()->smarty->clearAllCache();
    }

    protected function getMultiShopValues($key)
    {
        if (version_compare(_PS_VERSION_, '1.6.0.3', '>=') === true) {
            return Configuration::getMultiShopValues($key);
        } else {
            $shops = Shop::getShops(false, null, true);
            $id_lang = (int)$this->context->language->id;
            $results = array();
            array_push($results, Configuration::get($key));

            foreach ($shops as $id_shop) {
                array_push($results, Configuration::get($key, $id_lang, null, $id_shop));
            }

            return $results;
        }
    }
}
