<?php
/**
* NOTICE OF LICENSE.
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software
* and associated documentation files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge, publish, distribute,
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial
* portions of the Software.
*
* The software is provided "as is", without warranty of any kind, express or implied including 
* but not limited to the warranties of merchantability, fitness for a particular purpose and
* noninfrigement. In no event shall the authors or copyright holdes be liable for any claim, 
* damages or the liability, whether in an action of contract, tort or otherwise, arising from, 
* out of or in connection with the software or the use or other dealings in the software.
*
* @author    Diabolo SARL <ecommerce@diazol.com>
* @copyright 2011 - 2016 Diabolo SARL
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

class AddressXML
{
    private $invoice_address = array();
    private $delivery_address = array();

    public function __construct($customer_id, $lang_id, $_id_address_invoice, $id_address_delivery)
    {
        $objCustomer = new Customer($customer_id);
        $customer = $objCustomer->getFields();
        $addresses = $objCustomer->getAddresses($lang_id);

        if (count($addresses) == 1) {
            $this->delivery_address = $this->invoice_address = reset($addresses);
        } else {
            foreach ($addresses as $address) {
                if ($address['id_address'] == $_id_address_invoice) {
                    $this->invoice_address = $address;
                }

                if ($address['id_address'] == $id_address_delivery) {
                    $this->delivery_address = $address;
                }
            }
        }

        $this->delivery_address['id_gender'] = $this->invoice_address['id_gender'] = $customer['id_gender'];
    }

    private function getAddressXML(&$address_info, $tag_name)
    {
        $dom = new DOMDocument('1.0', 'utf-8');
        $dom->formatOutput = true;

        $address = $dom->createElement($tag_name);
        $address = $dom->appendChild($address);

        $address1 = $dom->createElement('Adresse1', (!empty($address_info['address1'])) ? trim($address_info['address1']) : '');
        $address->appendChild($address1);

        $address2 = $dom->createElement('Adresse2', (!empty($address_info['address2'])) ? trim($address_info['address2']) : '');
        $address->appendChild($address2);

        $postcode = $dom->createElement('CodePostal', (!empty($address_info['postcode'])) ? trim($address_info['postcode']) : '');
        $address->appendChild($postcode);

        $city = $dom->createElement('Ville', (!empty($address_info['city'])) ? trim($address_info['city']) : '');
        $address->appendChild($city);

        $country = $dom->createElement('Pays', (!empty($address_info['country'])) ? trim($address_info['country']) : '');
        $address->appendChild($country);

        return $dom->saveXML($address);
    }

    private function getContactAddressXML($contact_info, $tag_sufix = '')
    {
        $dom = new DOMDocument('1.0', 'utf-8');
        $dom->formatOutput = true;

        $gender = $dom->createElement('Civilite'.$tag_sufix, $this->getGenderName($contact_info['id_gender']));
        $dom->appendChild($gender);

        $last_name = $dom->createElement('Nom'.$tag_sufix, $contact_info['lastname']);
        $dom->appendChild($last_name);

        $first_name = $dom->createElement('Prenom'.$tag_sufix, $contact_info['firstname']);
        $dom->appendChild($first_name);

        return $dom;
    }

    private function getGenderName($gender_id)
    {
        $result = '';
        switch ($gender_id) {
            case '1':
                $result = 'Mr';
                break;
            case '2':
                $result = 'Ms';
                break;
            default:
                $result = '';
        }

        return $result;
    }

    protected function getContactMainAddressXML()
    {
        return $this->getContactAddressXML($this->invoice_address);
    }

    protected function getContactINvoiceAddressXML()
    {
        return $this->getContactAddressXML($this->invoice_address, 'Facturation');
    }

    protected function getContactDeliveryAddressXML()
    {
        return $this->getContactAddressXML($this->delivery_address, 'Livraison');
    }

    protected function getCustomerMainAddressXML()
    {
        return $this->getAddressXML($this->invoice_address, 'Adresse');
    }

    protected function getInvoiceAddressXML()
    {
        return $this->getAddressXML($this->invoice_address, 'AdresseFacturation');
    }
    protected function getDeliveryAddressXML()
    {
        return $this->getAddressXML($this->delivery_address, 'AdresseLivraison');
    }
}
