<?php
/**
* NOTICE OF LICENSE.
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software
* and associated documentation files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge, publish, distribute,
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial
* portions of the Software.
*
* The software is provided "as is", without warranty of any kind, express or implied including 
* but not limited to the warranties of merchantability, fitness for a particular purpose and
* noninfrigement. In no event shall the authors or copyright holdes be liable for any claim, 
* damages or the liability, whether in an action of contract, tort or otherwise, arising from, 
* out of or in connection with the software or the use or other dealings in the software.
*
* @author    Diabolo SARL <ecommerce@diazol.com>
* @copyright 2011 - 2016 Diabolo SARL
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

require_once 'address_xml.class.php';

class CustomerXML extends AddressXML
{
    private $_customer;

    public function __construct($customer_id, $lang_id, $_id_address_invoice, $id_address_delivery)
    {
        $objCustomer = new Customer((int)$customer_id);
        parent::__construct($customer_id, $lang_id, $_id_address_invoice, $id_address_delivery);
        $this->_customer = $objCustomer->getFields();
    }

    private function getPhoneXML()
    {
        $dom = new DOMDocument('1.0', 'utf-8');
        $dom->formatOutput = true;

        $phones = $dom->createElement('Telephones');
        $phones = $dom->appendChild($phones);

        if (!empty($this->_main_address['phone'])) {
            $phone = $dom->createElement('Telephone', $this->_main_address['phone']);
            $phone->setAttribute('type', 'fixe');
            $phone->setAttribute('nature', 'defaut');
            $phones->appendChild($phone);
        }

        if (!empty($this->_main_address['phone_mobile'])) {
            $phone = $dom->createElement('Telephone', $this->_main_address['phone_mobile']);
            $phone->setAttribute('type', 'portable');
            $phone->setAttribute('nature', 'defaut');
            $phones->appendChild($phone);
        }

        if (!empty($this->_main_address['other'])) {
            $phone = $dom->createElement('Telephone', $this->_main_address['other']);
            $phone->setAttribute('type', 'fax');
            $phone->setAttribute('nature', 'defaut');
            $phones->appendChild($phone);
        }

        return $dom->saveXML($phones);
    }

    private function getInvoiceXML()
    {
        $dom = new DOMDocument('1.0', 'utf-8');
        $dom->formatOutput = true;

        $invoice = $dom->createElement('Facturation');
        $invoice = $dom->appendChild($invoice);

        $contact_dom = $this->getContactINvoiceAddressXML();

        foreach ($contact_dom->childNodes as $node) {
            $node = $dom->importNode($node, true);
            $invoice->appendChild($node);
        }

        $phone = '';

        if (!empty($this->_invoice_address['phone'])) {
            $phone = $this->_invoice_address['phone'];
        } elseif (!empty($this->_invoice_address['phone_mobile'])) {
            $phone = $this->_invoice_address['phone_mobile'];
        } elseif (isset($this->_invoice_address['other'])) {
            $phone = $this->_invoice_address['other'];
        }

        $phone = $dom->createElement('TelephoneFacturation', $phone);
        $invoice->appendChild($phone);

        ////
        // Added Customer's Invoice Address
        ////
        $address_xml = new DOMDocument();
        $address_xml->loadXML($this->getInvoiceAddressXML());

        $address = $address_xml->documentElement;

        $address = $dom->importNode($address, true);

        $invoice->appendChild($address);

        return $dom->saveXML($invoice);
    }

    public function getCustomerXML()
    {
        $dom = new DOMDocument('1.0', 'utf-8');
        $dom->formatOutput = true;

        $customer = $dom->createElement('Client');
        $customer = $dom->appendChild($customer);

        $customer_id = $dom->createElement('ClientId', $this->_customer['id_customer']);
        $customer->appendChild($customer_id);

        $contact_dom = $this->getContactMainAddressXML();

        foreach ($contact_dom->childNodes as $node) {
            $node = $dom->importNode($node, true);
            $customer->appendChild($node);
        }

        $email = $dom->createElement('Email', $this->_customer['email']);
        $customer->appendChild($email);

        ////
        // Added Customer's Main Address
        ////
        $address_xml = new DOMDocument();
        $address_xml->loadXML($this->getCustomerMainAddressXML());

        $address = $address_xml->documentElement;
        $address = $dom->importNode($address, true);

        $customer->appendChild($address);

        ////
        // Added Customer's Phones
        ////
        $phone_xml = new DOMDocument();
        $phone_xml->loadXML($this->getPhoneXML());

        $phones = $phone_xml->documentElement;

        $phones = $dom->importNode($phones, true);

        $customer->appendChild($phones);

        ////
        // Added Customer's invoice info
        ////
        $invoice_info_xml = new DOMDocument();
        $invoice_info_xml->loadXML($this->getInvoiceXML());

        $invoice_info = $invoice_info_xml->documentElement;

        $invoice_info = $dom->importNode($invoice_info, true);
        $customer->appendChild($invoice_info);

        return $dom->saveXML($customer);
    }
}
