<?php
/**
* NOTICE OF LICENSE.
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software
* and associated documentation files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge, publish, distribute,
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial
* portions of the Software.
*
* The software is provided "as is", without warranty of any kind, express or implied including 
* but not limited to the warranties of merchantability, fitness for a particular purpose and
* noninfrigement. In no event shall the authors or copyright holdes be liable for any claim, 
* damages or the liability, whether in an action of contract, tort or otherwise, arising from, 
* out of or in connection with the software or the use or other dealings in the software.
*
* @author    Diabolo SARL <ecommerce@diazol.com>
* @copyright 2011 - 2016 Diabolo SARL
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

require_once 'customer_xml.class.php';
require_once 'order_xml.class.php';

class ExportXML
{
    private $export_xml_path;
    private $order_id;
    private $customer_id;
    private $lang_id;
    private $id_address_invoice;
    private $id_address_delivery;

    public function __construct()
    {
        $this->export_xml_path = dirname(__FILE__).'/../../../../arte-fact/outputs/';
    }

    public function setOrderID($order_id)
    {
        $this->order_id = $order_id;
    }

    public function setCustomerID($customer_id)
    {
        $this->customer_id = $customer_id;
    }

    public function setLangID($lang_id)
    {
        $this->lang_id = $lang_id;
    }

    public function setInvoiceAddressID($id_address_invoice)
    {
        $this->id_address_invoice = $id_address_invoice;
    }

    public function setDeliveryAddressID($id_address_delivery)
    {
        $this->id_address_delivery = $id_address_delivery;
    }

    public function saveXML()
    {
        $artifact_version = Configuration::get('PS_ARTIFACT_VERSION');

        if (empty($artifact_version)) {
            $artifact_version = '1.4.2';
        }

        // Creates an instance of the DOMImplementation class
        $imp = new DOMImplementation();

        // Creates a DOMDocumentType instance
        $dtd = $imp->createDocumentType('ArtE-fact', '', 'http://diazol.com/telechargement/artefact.dtd');

        // Creates a DOMDocument instance
        $dom = $imp->createDocument('', '', $dtd);

        $dom->encoding = 'UTF-8';

        $dom->formatOutput = true;

        $arte_fact = $dom->createElement('ArtE-fact');
        $arte_fact->setAttribute('version', $artifact_version);
        ////
        // Add Orders
        ////
        $orders = $dom->createElement('Ventes');

        $order_xml = new DOMDocument();
        $objOrderXML = new OrderXML($this->order_id, $this->lang_id);
        $order_xml->loadXML($objOrderXML->getOrderXML());

        $order = $order_xml->documentElement;

        $order = $dom->importNode($order, true);

        $orders->appendChild($order);
        $arte_fact->appendChild($orders);

        ////
        // Add Customer
        ////
        $customers = $dom->createElement('Clients');

        $customer_xml = new DOMDocument();
        $objCusttomerXML = new CustomerXML($this->customer_id, $this->lang_id, $this->id_address_invoice, $this->id_address_delivery);
        $customer_xml->loadXML($objCusttomerXML->getCustomerXML());

        $customer = $customer_xml->documentElement;

        $customer = $dom->importNode($customer, true);
        $customers->appendChild($customer);
        $arte_fact->appendChild($customers);

        $dom->appendChild($arte_fact);

        $dom->save($this->export_xml_path."order{$this->order_id}.xml");
    }
}
