<?php
/**
* NOTICE OF LICENSE.
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software
* and associated documentation files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge, publish, distribute,
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial
* portions of the Software.
*
* The software is provided "as is", without warranty of any kind, express or implied including 
* but not limited to the warranties of merchantability, fitness for a particular purpose and
* noninfrigement. In no event shall the authors or copyright holdes be liable for any claim, 
* damages or the liability, whether in an action of contract, tort or otherwise, arising from, 
* out of or in connection with the software or the use or other dealings in the software.
*
* @author    Diabolo SARL <ecommerce@diazol.com>
* @copyright 2011 - 2016 Diabolo SARL
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

class OrderProductsXML
{
	private $_order_id;

	private $_lang_id;
	private $_order_products;
	private $_conversions;
	private $_conversionsExport;

	public function __construct($order_id, $lang_id)
	{
		$objOrder = new Order($order_id);

		$this->_order_id = $order_id;
		$this->_lang_id = $lang_id;
		$this->_order_products = $objOrder->getProducts();
		$this->_getConversionsExport();
	}

	private function _getConversionsExport()
	{
		$conversions = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'artefact_conversions_rules`', false, false);
		// attention for export let's invert orignal & symbol

		while ($row = Db::getInstance()->nextRow($conversions)) {
			$this->_conversionsExport[$row['imports_conversions_type']][$row['imports_conversions_rules_original']] = $row['imports_conversions_rules_symbol'];
		}

		$languages = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'lang`', false, false);

		while ($row = Db::getInstance()->nextRow($languages)) {
			$this->_languages[Tools::strtolower($row['iso_code'])] = $row['id_lang'];
		}
	}

	private function _getConversions()
	{
		/*
		$conversions = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'artefact_conversions_rules` WHERE `imports_conversions_type`="product"', false, false);

		while ($row = Db::getInstance()->nextRow($conversions)) {
			$this->_conversions[$row['imports_conversions_type']][$row['imports_conversions_rules_symbol']] = $row['imports_conversions_rules_original'];
		}
		*/
		$conversions = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'artefact_conversions_rules`', false, false);

		while ($row = Db::getInstance()->nextRow($conversions)) {

			$symbol =($row['imports_conversions_type']=='product' || $row['imports_conversions_type']=='declinaison') ? $row['imports_conversions_rules_symbol'] : $symbol = Tools::link_rewrite(Tools::replaceAccentedChars((string)$row['imports_conversions_rules_symbol']));
			//$symbol = Tools::link_rewrite(Tools::replaceAccentedChars((string)$row['imports_conversions_rules_symbol']));
			$this->_conversions[$row['imports_conversions_type']][$symbol] = $row['imports_conversions_rules_original'];
		}

		$languages = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'lang` WHERE active=1', false, false);

		while ($row = Db::getInstance()->nextRow($languages)) {
			$this->_languages[mb_strtolower($row['iso_code'])] = (int)$row['id_lang'];
		}

		unset($conversions, $languages);

	}


	private function getProductAttribute($product_id, $attribute_id, $product_price, $price_tax = 'TTC')
	{
		$dom = new DOMDocument('1.0', 'utf-8');
		$dom->formatOutput = true;

		$prodocts_attributes = $dom->createElement('ProduitDeclinaison');
		$prodocts_attributes = $dom->appendChild($prodocts_attributes);

		$objProduct = new Product($product_id);
		$attributes = $objProduct->getAttributesGroups($this->_lang_id);
		$attribute_infos = array();

		foreach ($attributes as $attribute) {
			if ($attribute['id_product_attribute'] != $attribute_id) {
				continue;
			}

			$attribute_infos[] = $attribute;
		}
		if (count($attribute_infos) > 0) {
			// let's retrieve artifact reference
			$sql='SELECT artifact_reference FROM `' . _DB_PREFIX_ . 'product_attribute` WHERE  id_product_attribute= ' .  $attribute_id;
			$artifact_reference = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
			//$artifact_reference = $this->_conversionsExport['declinaison'][$attribute_id];
			$declinaison_reference = $dom->createElement('DeclinaisonReference', $artifact_reference);
			$prodocts_attributes->appendChild($declinaison_reference);
			$attributes = $dom->createElement('Attributs');

			foreach ($attribute_infos as $attribute_info) {
				if (isset($attribute_info['public_group_name']) && isset($attribute_info['attribute_name'])) {
					$attribute = $dom->createElement('Attribut');
					$attribute->setAttribute('id', $attribute_info['public_group_name']);
					$attribute->appendChild($dom->createCDATASection($attribute_info['attribute_name']));
					$attributes->appendChild($attribute);
				}
			}

			$prodocts_attributes->appendChild($attributes);
			$attribute_price = $dom->createElement('DeclinaisonPrix', sprintf('%01.2f', $product_price));
			$attribute_price->setAttribute('type', $price_tax);

			$prodocts_attributes->appendChild($attribute_price);
		}

		return $dom->saveXML($prodocts_attributes);
	}

	private function simulateAttribute($product_id, $declinaisonReference, $product_price, $price_tax = 'TTC')
	{
		$dom = new DOMDocument('1.0', 'utf-8');
		$dom->formatOutput = true;

		$prodocts_attributes = $dom->createElement('ProduitDeclinaison');
		$prodocts_attributes = $dom->appendChild($prodocts_attributes);

		$objProduct = new Product($product_id);
		$declinaison_reference = $dom->createElement('DeclinaisonReference', $declinaisonReference);
		$prodocts_attributes->appendChild($declinaison_reference);
		$attribute_price = $dom->createElement('DeclinaisonPrix', sprintf('%01.2f', $product_price));
		$attribute_price->setAttribute('type', $price_tax);

		$prodocts_attributes->appendChild($attribute_price);

		return $dom->saveXML($prodocts_attributes);
	}


	public function getProductsXML()
	{
		$tax_list = $tax_rules = array();

		$dom = new DOMDocument('1.0', 'utf-8');
		$dom->formatOutput = true;

		$products = $dom->createElement('Produits');
		$products = $dom->appendChild($products);

		$objTax = new Tax();

		foreach ($objTax->getTaxes($this->_lang_id) as $tax_rule) {
			$tax_list[$tax_rule['id_tax']] = $tax_rule['rate'];
		}

		$tax_rules_sql = Db::getInstance(_PS_USE_SQL_SLAVE_)->ExecuteS('
		SELECT DISTINCT id_tax_rules_group, id_tax
		FROM `'._DB_PREFIX_.'tax_rule`
		');

		foreach ($tax_rules_sql as $tax_rule) {
			$tax_rules[$tax_rule['id_tax_rules_group']] = $tax_rule['id_tax'];
		}
		foreach ($this->_order_products as $order_product) {

			$objProduct = new Product($order_product['product_id']);
			$product = $dom->createElement('Produit');
			$genericProduct=false;

			//some prestashop products are not used in Artifact
			if(isset($this->_conversionsExport['product'][$order_product['product_id']])) {
				$sql='SELECT artifact_reference FROM `' . _DB_PREFIX_ . 'product` WHERE  id_product= ' .  $order_product['product_id'];
				$artifact_reference = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
				if(empty($artifact_reference)) { 
					$genericProduct=true;
					$product_reference = $dom->createElement('ProduitReference', 'SEPR-1');
					$product->appendChild($product_reference);
				}
				else {
					$product_reference = $dom->createElement('ProduitReference', $this->_conversionsExport['product'][$order_product['product_id']]);
					$product->appendChild($product_reference);
					// let's check if product still exist...
				}
			}
			else {
				// let's check if we have an artifact reference ....
				$sql='SELECT artifact_reference FROM `' . _DB_PREFIX_ . 'product` WHERE  id_product= ' .  $order_product['product_id'];
				$artifact_reference = Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
				if(!empty($artifact_reference)) {
					$product_reference = $dom->createElement('ProduitReference', $artifact_reference);
					$product->appendChild($product_reference);
				}
				else {
				// let's use a generic product...
					$genericProduct=true;
					$product_reference = $dom->createElement('ProduitReference', 'SEPR-1');
					$product->appendChild($product_reference);
				}
			}
			$product_id = $order_product['product_id'];
			$product_details = $objProduct->getFields();

			//$product_reference_site = $dom->createElement('ProduitReferenceSite', $product_details['reference']);
			//$product->appendChild($product_reference_site);

			if ($order_product['reduction_percent'] != 0 || $order_product['reduction_amount'] != 0) {
				$product_priceTTC = number_format($order_product['original_product_price'] + ($order_product['original_product_price'] * ($order_product['tax_rate'] / 100)), 1, '.', '');
				$product_price_sale = number_format($order_product['unit_price_tax_incl'], 2);
				$product_unit_discount_amount = $product_priceTTC - $product_price_sale;
			} else {
				$product_unit_discount_amount = 0;
				$product_priceTTC=$order_product['product_price_wt'];
				$product_price_sale = $product_priceTTC;
			}

			$product_price_type = $dom->createElement('ProduitPrix', sprintf('%01.2f', $product_priceTTC));
			$product_price_type->setAttribute('type', 'TTC');
			$product->appendChild($product_price_type);

			if($genericProduct==false) {
				if(!empty($order_product['product_attribute_id'])) {
					if(isset($this->_conversionsExport['declinaison'][$order_product['product_attribute_id']])) {
						$product_attr_xml = new DOMDocument();
						$product_attr_xml->loadXML($this->getProductAttribute($order_product['product_id'], $order_product['product_attribute_id'], $product_priceTTC));
						$product_attr = $product_attr_xml->documentElement;
						$product_attr = $dom->importNode($product_attr, true);
						$product->appendChild($product_attr);
					}
					elseif(isset($this->_conversionsExport['attribute_to_product'][$order_product['product_id']])) {
						$product_attr_xml = new DOMDocument();
						$product_attr_xml->loadXML($this->simulateAttribute($order_product['product_id'], $this->_conversionsExport['attribute_to_product'][$order_product['product_id']], $product_priceTTC));
						$product_attr = $product_attr_xml->documentElement;
						$product_attr = $dom->importNode($product_attr, true);
						$product->appendChild($product_attr);
					}
					else {
						pp('else ' . $order_product['product_id'] . ' - ' . $order_product['product_attribute_id']) ;
					}
				}
				else {
						//pp('else else ' . $this->_conversionsExport['declinaison'][$order_product['product_id']]);
					if(isset($this->_conversionsExport['attribute_to_product'][$order_product['product_id']])) {
						$product_attr_xml = new DOMDocument();
						$product_attr_xml->loadXML($this->simulateAttribute($order_product['product_id'], $this->_conversionsExport['attribute_to_product'][$order_product['product_id']], $product_priceTTC));
						$product_attr = $product_attr_xml->documentElement;
						$product_attr = $dom->importNode($product_attr, true);
						$product->appendChild($product_attr);
					}
				}
			}
/*
			pp($order_product);
			exit;
*/
			$product_qry = $dom->createElement('ProduitQuantite', $order_product['product_quantity']);
			$product->appendChild($product_qry);

			// let's calculate rebate as pretashop send us everything but right information...
			// anyway we won't have valuable informatin due to round rules

			if ($order_product['reduction_percent'] != 0 || $order_product['reduction_amount'] != 0) {
				$product_discount_amount = $dom->createElement('ProduitMontantRemise', sprintf('%01.2f', (($product_unit_discount_amount))));
			} else {
				$product_discount_amount = $dom->createElement('ProduitMontantRemise', sprintf('%01.2f', 0));
			}

			$product_discount_amount->setAttribute('type', 'TTC');
			$product->appendChild($product_discount_amount);

			if($order_product['tax_rate']==0) {
				$tax_id='4';
			}
			else {
				$tax_id = !empty($this->_conversionsExport['taxes'][$tax_rules[$product_details['id_tax_rules_group']]]) ? $this->_conversionsExport['taxes'][$tax_rules[$product_details['id_tax_rules_group']]] : 0;
			}
			$product_vat = $dom->createElement('ProduitTVAId', $tax_id);
			$product->appendChild($product_vat);

			$product_price_with_vat = $dom->createElement('ProduitMontantTotalTTC', sprintf('%01.2f', $order_product['total_wt']));
			$product->appendChild($product_price_with_vat);

			$products->appendChild($product);

		}
		return $dom->saveXML($products);

	}
}
