<?php
/**
* NOTICE OF LICENSE.
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software
* and associated documentation files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge, publish, distribute,
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial
* portions of the Software.
*
* The software is provided "as is", without warranty of any kind, express or implied including 
* but not limited to the warranties of merchantability, fitness for a particular purpose and
* noninfrigement. In no event shall the authors or copyright holdes be liable for any claim, 
* damages or the liability, whether in an action of contract, tort or otherwise, arising from, 
* out of or in connection with the software or the use or other dealings in the software.
*
* @author    Diabolo SARL <ecommerce@diazol.com>
* @copyright 2011 - 2016 Diabolo SARL
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

require_once 'address_xml.class.php';
require_once 'order_products_xml.class.php';

class OrderXML extends AddressXML
{
    private $order;

    private $order_id;

    private $lang_id;

    public function __construct($order_id, $lang_id)
    {
        $obj_order = new Order($order_id);
        $this->order_id = $order_id;
        $this->lang_id = $lang_id;
        $this->order = $obj_order->getFields();

        parent::__construct($this->order['id_customer'], $lang_id, $this->order['id_address_invoice'], $this->order['id_address_delivery']);
    }

    private function getDeliveryInfoXML()
    {
        $dom = new DOMDocument('1.0', 'utf-8');
        $dom->formatOutput = true;

        $delievry = $dom->createElement('Livraison');
        $delievry = $dom->appendChild($delievry);

        $contact_dom = $this->getContactDeliveryAddressXML();

        foreach ($contact_dom->childNodes as $node) {
            $node = $dom->importNode($node, true);
            $delievry->appendChild($node);
        }

        // Add Delivery Address
        $address_xml = new DOMDocument();
        $address_xml->loadXML($this->getDeliveryAddressXML());
        $address = $address_xml->documentElement;
        $address = $dom->importNode($address, true);
        $delievry->appendChild($address);
        $comments = $dom->createElement('CommentaireLivraison', '');
        $delievry->appendChild($comments);

        return $dom->saveXML($delievry);
    }

    private function getPaymentMethodXML()
    {
        $dom = new DOMDocument('1.0', 'utf-8');
        $dom->formatOutput = true;

        $payment_method = $dom->createElement('VenteReglements');
        $payment_method = $dom->appendChild($payment_method);

        $method = $dom->createElement('VenteReglement');
        $method = $dom->appendChild($method);

        $method_name = $dom->createElement('TypeReglement', $this->order['module']);
        $method->appendChild($method_name);

        if ($this->order['module'] == 'cheque' || $this->order['module'] == 'bankwire') {
            $this->order['total_paid_real'] = $this->order['total_paid'];
        }

        $paid = $dom->createElement('MontantReglement', sprintf('%01.2f', $this->order['total_paid_real']));
        $method->appendChild($paid);

        $payment_method->appendChild($method);

        return $dom->saveXML($payment_method);
    }

    private function getShippingXML()
    {
        $dom = new DOMDocument('1.0', 'utf-8');
        $dom->formatOutput = true;

        $delivery_comments = $dom->createElement('VenteLivraison');
        $delivery_comments = $dom->appendChild($delivery_comments);

		if(!empty($this->order['id_carrier'])) {
			$objCarrier = new Carrier($this->order['id_carrier']);
			$shipping_method = $objCarrier->getFields();
			//$this->order['id_carrier']=18;
			$shipper = $dom->createElement('Transporteur', $shipping_method['name']);
		}
		else {
			$shipper = $dom->createElement('Transporteur', 'aucun');
		}

        $delivery_comments->appendChild($shipper);

        $delivery_date = 'NULL';

        if (!empty($this->order['delivery_date']) && $this->order['delivery_date'] != '0000-00-00 00:00:00') {
            $delivery_date = date('Y/m/d h:i:s', strtotime($this->order['delivery_date']));
        }


        $date_of_shipping = $dom->createElement('DateLivraison', $delivery_date);
        $delivery_comments->appendChild($date_of_shipping);

        //$obj_order = new Order($this->order_id);

        $comments = $dom->createElement('InfoLivraison');

        $last_message = $this->getLastMessage();

        $comments->appendChild($dom->createCDATASection($last_message));

        $delivery_comments->appendChild($comments);

        return $dom->saveXML($delivery_comments);
    }

    public function getLastMessage()
    {
        return Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue('
			 SELECT `message`
			 FROM `'._DB_PREFIX_.'message`
			 WHERE `id_order` = '.(int)$this->order_id.'
			 ORDER BY `id_message` DESC
			');
    }

    public function getOrderXML()
    {
        $dom = new DOMDocument('1.0', 'utf-8');
        $dom->formatOutput = true;

        $order = $dom->createElement('Vente');
        $order = $dom->appendChild($order);

        $order_id = $dom->createElement('VenteId', $this->order_id);
        $order_id->setAttribute('nature', 'vente');

        $order->appendChild($order_id);

        $date = $dom->createElement('Date', date('Y/m/d h:i:s', strtotime($this->order['date_add'])));
        $order->appendChild($date);

        $customer_id = $dom->createElement('ClientId', $this->order['id_customer']);
        $order->appendChild($customer_id);

        // Add Invoice Address
        $address_xml = new DOMDocument();
        $address_xml->loadXML($this->getInvoiceAddressXML());

        $address = $address_xml->documentElement;

        $address = $dom->importNode($address, true);

        $order->appendChild($address);

        // Add Delivery Info
        $delievry_xml = new DOMDocument();
        $delievry_xml->loadXML($this->getDeliveryInfoXML());

        $delievry = $delievry_xml->documentElement;

        $delievry = $dom->importNode($delievry, true);

        $order->appendChild($delievry);

        // Add Products Info
        $obj_orderProsuctsXML = new OrderProductsXML($this->order_id, $this->lang_id);
        $products_xml = new DOMDocument();
        $products_xml->loadXML($obj_orderProsuctsXML->getProductsXML());

        $products = $products_xml->documentElement;

        $products = $dom->importNode($products, true);

        $order->appendChild($products);

        // Add Total Info
        $order_total_vat = $dom->createElement('VenteTotalHT', sprintf('%01.2f', $this->order['total_products']));
        $order->appendChild($order_total_vat);

        $order_discount_vat = $dom->createElement('VenteMontantRemiseHT', sprintf('%01.2f', $this->order['total_discounts_tax_excl']));
        $order->appendChild($order_discount_vat);

        //$total_order_VAT = $dom->createElement('VenteMontantTVA', $this->order['total_products_wt'] - $this->order['total_products']);
        $total_order_VAT = $dom->createElement('VenteMontantTVA', '');
        $order->appendChild($total_order_VAT);

        $shiping_type_price = $dom->createElement('VentePort', sprintf('%01.2f', $this->order['total_shipping_tax_incl']));
        $shiping_type_price->setAttribute('type', 'TTC');
        $order->appendChild($shiping_type_price);

        $shipping_price = ($this->order['total_shipping_tax_excl'] != $this->order['total_shipping_tax_incl']);
        $shipping_price = $dom->createElement('VentePortMontantTVA', sprintf('%01.2f', ($this->order['total_shipping_tax_incl'] - $this->order['total_shipping_tax_excl'])));
        $order->appendChild($shipping_price);

        $total_order_with_vat = $dom->createElement('VenteMontantTotalTTC', sprintf('%01.2f', ($this->order['total_paid'])));
        $order->appendChild($total_order_with_vat);

        // Add Payment Method
        $payment_xml = new DOMDocument();
        $payment_xml->loadXML($this->getPaymentMethodXML());

        $payment = $payment_xml->documentElement;

        $payment = $dom->importNode($payment, true);

        $order->appendChild($payment);

        // Add Shipping Info
        $shipping_xml = new DOMDocument();
        $shipping_xml->loadXML($this->getShippingXML());

        $shipping = $shipping_xml->documentElement;

        $shipping = $dom->importNode($shipping, true);

        $order->appendChild($shipping);

        return $dom->saveXML($order);
    }
}
