<?php
/**
* NOTICE OF LICENSE.
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software
* and associated documentation files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge, publish, distribute,
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial
* portions of the Software.
*
* The software is provided "as is", without warranty of any kind, express or implied including 
* but not limited to the warranties of merchantability, fitness for a particular purpose and
* noninfrigement. In no event shall the authors or copyright holdes be liable for any claim, 
* damages or the liability, whether in an action of contract, tort or otherwise, arising from, 
* out of or in connection with the software or the use or other dealings in the software.
*
* @author    Diabolo SARL <ecommerce@diazol.com>
* @copyright 2011 - 2016 Diabolo SARL
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

class ArtifactImportBrands
{
	private $_conversions;
	private $_languages;
	private $_logger;

	public function import($obj)
	{
		if (empty($obj)) {
			return false;
		}

		$log = new ArtifactImport();

		if (isset($log->logger)) {
			$this->_logger = $log->logger;
		}

		$this->_getConversions();
		$this->_importBrands($obj);
	}

	private function _getConversions()
	{
		$conversions = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'artefact_conversions_rules`', false, false);

		while ($row = Db::getInstance()->nextRow($conversions)) {
			$this->_conversions[$row['imports_conversions_type']][$row['imports_conversions_rules_symbol']] = $row['imports_conversions_rules_original'];
		}

		$languages = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'lang`', false, false);

		while ($row = Db::getInstance()->nextRow($languages)) {
			$this->_languages[Tools::strtolower($row['iso_code'])] = $row['id_lang'];
		}
	}

	private function _importBrands($obj)
	{
		if (empty($obj)) {
			return false;
		}

		$brands = array();
		$data = array();

		foreach ($obj as $values) {
			$data['action'] = (string)$values->attributes()->action;
			$data['id'] = (int)$values->MarqueId;
			$data['MarqueImage'] = (string)$values->MarqueImage;
			$data['MarqueUrl'] = (string)$values->MarqueUrl;

			foreach ($values->MarqueInfos->MarqueInfo as $languages) {
				$language = Tools::strtolower((string)$languages->attributes()->lang);
				$data['MarqueInfos'][$language]['name'] = (string)$languages->MarqueNom;
				$data['MarqueInfos'][$language]['description'] = (string)$languages->MarqueDescription;

				if ($language == 'fr') {
					$data['MarqueName'] = (string)$languages->MarqueNom;
				}
			}
			$brands[] = $data;
		}

		foreach ($brands as $brand) {
			if ($brand['action'] == 'delete') {
				$this->_deleteBrand($brand['id']);
				continue;
			}

			$this->_importBrand($brand);
		}
	}

	private function _importBrand($brand)
	{
		$date = date('Y-m-d H:i:s');
		$brand_id = !empty($this->_conversions['brand'][$brand['id']]) ? $this->_conversions['brand'][$brand['id']] : 0;
		if(Configuration::get('PS_ARF_FIRST_RUN')==true  && (empty($brand_id))) {

			$manufacturer = new Manufacturer();
			if($brand_id = Manufacturer::getIdByName($brand['MarqueName'])) {

				$addTab = array(
					'imports_conversions_rules_symbol'    => $brand['id'],
					'imports_conversions_rules_original'  => $brand_id,
					'imports_conversions_type'            => 'brand'
				);

				Db::getInstance()->insert('artefact_conversions_rules', $addTab);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'artefact_conversions_rules', $addTab, 'INSERT');
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS BRANDS : INSERT FROM artefact_conversions_rules ",Db::getInstance()->getMsgError());
				}
				$this->_conversions['brand'][$brand['id']] = $brand_id;
			
			}
			else {
				// brand['action']==CREATE
				$addTab = array(
					'id_manufacturer' => 0,
					'active'          => 1,
					'name'            => pSQL($brand['MarqueName']),
					'date_add'        => pSQL($date),
					'date_upd'        => pSQL($date)
				);
				Db::getInstance()->insert('manufacturer', $addTab);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'manufacturer', $addTab, 'INSERT');
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS BRANDS : INSERT FROM MANUFACTURE ",Db::getInstance()->getMsgError());
				}
				$brand_id = Db::getInstance()->Insert_ID();
			
				$addTab = array(
					'imports_conversions_rules_symbol'    => $brand['id'],
					'imports_conversions_rules_original'  => $brand_id,
					'imports_conversions_type'            => 'brand'
				);
				Db::getInstance()->insert('artefact_conversions_rules', $addTab);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'artefact_conversions_rules', $addTab, 'INSERT');
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS BRANDS : INSERT FROM artefact_conversions_rules ",Db::getInstance()->getMsgError());
				}
				$this->_conversions['brand'][$brand['id']] = $brand_id;

			}
		}

		else {

			if (empty($brand_id)) {
				$addTab = array(
					'id_manufacturer' => 0,
					'active'          => 0,
					'name'            => pSQL($brand['MarqueName']),
					'date_add'        => pSQL($date),
					'date_upd'        => pSQL($date)
				);
				Db::getInstance()->insert('manufacturer', $addTab);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'manufacturer', $addTab, 'INSERT');
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS BRANDS : INSERT FROM MANUFACTURE ",Db::getInstance()->getMsgError());
				}
				$brand_id = Db::getInstance()->Insert_ID();
			
				$addTab = array(
					'imports_conversions_rules_symbol'    => $brand['id'],
					'imports_conversions_rules_original'  => $brand_id,
					'imports_conversions_type'            => 'brand'
				);
				Db::getInstance()->insert('artefact_conversions_rules', $addTab);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'artefact_conversions_rules', $addTab, 'INSERT');
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS BRANDS : INSERT FROM artefact_conversions_rules ",Db::getInstance()->getMsgError());
				}
				$this->_conversions['brand'][$brand['id']] = $brand_id;
			
			} 
			// we dont' update brands from Artifact...
			else {

				// back-office use on categories ?
				$updateBrands = true;
				if (Configuration::get('PS_ARF_ALLOW_BRAND_UPD') == true) {
					$updateBrands = false;

					if ($brand['action'] == 'create') {
						$updateBrands = true;
					}
				}
				if ($updateBrands == true) {
						$addTab = array(
							'active'          => 1,
							'name'            => pSQL($brand['MarqueName']),
							'date_add'        => pSQL($date),
							'date_upd'        => pSQL($date)
						);
						Db::getInstance()->update('manufacturer', $addTab,  "id_manufacturer = ".$brand_id);
						//Db::getInstance()->autoExecute(_DB_PREFIX_.'manufacturer', $addTab, 'UPDATE', "id_manufacturer = ".$brand_id);
						if (Db::getInstance()->getMsgError()) {
							if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS BRANDS : UPDATE FROM MANUFACTURE ",Db::getInstance()->getMsgError());
						}
				}
			}
		}

		if($brand['action']=='create') {

			Db::getInstance()->Execute('INSERT IGNORE INTO '._DB_PREFIX_.'manufacturer_shop (`id_manufacturer`, `id_shop`) 
										 SELECT "'.pSQL($brand_id).'", id_shop FROM '._DB_PREFIX_.'shop');
			if (Db::getInstance()->getMsgError()) {
				if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS BRANDS : INSERT FROM MANUFACTURE_SHOP ",Db::getInstance()->getMsgError());
			}
		
			foreach ($brand['MarqueInfos'] as $language => $values) {
				if (empty($this->_languages[$language])) {
					continue;
				}
				Db::getInstance()->Execute('INSERT INTO '._DB_PREFIX_.'manufacturer_lang (`id_manufacturer`, `id_lang`, `short_description`, `description`) 
											VALUES (
												"'.pSQL($brand_id).'", 
												"'.pSQL($this->_languages[$language]).'",
												"'.pSQL($values['name']).'", 
												"'.pSQL($values['description']).'"
											)
											ON DUPLICATE KEY UPDATE 
											short_description        = "'.pSQL($values['name']).'", 
											description = "'.pSQL($values['description']).'"');
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS BRANDS : INSERT FROM MANUFACTURE_LANG ",Db::getInstance()->getMsgError());
				}
			}
		
		}
		
		/* we don't sync brand image
		if (!empty($brand['MarqueImage'])) {
			$image_path     = dirname(__FILE__)."/../../../../arte-fact/inputs/images/marques/".$brand['MarqueImage'];
			$new_image_path = _PS_MANU_IMG_DIR_.$brand_id.'.jpg';
			if (file_exists($image_path)) {
				if (file_exists($new_image_path)) {
				  @unlink($new_image_path);
				}
				if (copy($image_path, $new_image_path)) {
					$imagesTypes = ImageType::getImagesTypes('manufacturers');
					foreach ($imagesTypes AS $k => $imageType) {
						imageResize($new_image_path, _PS_MANU_IMG_DIR_.$brand_id.'-'.stripslashes($imageType['name']).'.jpg', (int)($imageType['width']), (int)($imageType['height']));
					}
				}
			}
		}
		*/
	}


	private function _deleteBrand($brand_id)
	{
		$brand_id = !empty($this->_conversions['brand'][$brand_id]) ? $this->_conversions['brand'][$brand_id] : 0;
		if (empty($brand_id)) {
			return false;
		}

		$brand_obj = new Manufacturer($brand_id);
		$brand_obj->delete();

		/*
		Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'manufacturer_lang    WHERE id_manufacturer = '.$brand_id);
		Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'manufacturer         WHERE id_manufacturer = '.$brand_id);
		*/

		// Delete conversion
		Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'artefact_conversions_rules 
								  WHERE imports_conversions_type = "brand" AND imports_conversions_rules_original = '.$brand_id);
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS BRANDS : DELETE FROM artefact_conversions_rules ', Db::getInstance()->getMsgError());
			}
		}

		unset($this->_conversions['brand'][$brand_id]);
	}
}
