<?php
/**
* NOTICE OF LICENSE.
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software
* and associated documentation files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge, publish, distribute,
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial
* portions of the Software.
*
* The software is provided "as is", without warranty of any kind, express or implied including 
* but not limited to the warranties of merchantability, fitness for a particular purpose and
* noninfrigement. In no event shall the authors or copyright holdes be liable for any claim, 
* damages or the liability, whether in an action of contract, tort or otherwise, arising from, 
* out of or in connection with the software or the use or other dealings in the software.
*
* @author    Diabolo SARL <ecommerce@diazol.com>
* @copyright 2011 - 2016 Diabolo SARL
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

class ArtifactImportCategories
{
	private $_conversions;
	private $_languages;
	private $_default_category_id;
	private $_logger;

	public function import($obj)
	{
		if (empty($obj)) {
			return false;
		}

		$log = new ArtifactImport();

		if (isset($log->logger)) {
			$this->_logger = $log->logger;
		}

		$this->_getConversions();
		$this->_getDefaultCategory();
		$this->_importCategories($obj);
	}
	private function _getConversions()
	{
		$conversions = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'artefact_conversions_rules`', false, false);

		while ($row = Db::getInstance()->nextRow($conversions)) {
			$this->_conversions[$row['imports_conversions_type']][$row['imports_conversions_rules_symbol']] = $row['imports_conversions_rules_original'];
		}

		$languages = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'lang`', false, false);

		while ($row = Db::getInstance()->nextRow($languages)) {
			$this->_languages[Tools::strtolower($row['iso_code'])] = $row['id_lang'];
		}
	}

	/**
	* catégorie par défaut ici c'est : 2
	*
	* @param
	* @return _default_category_id
	*/
	private function _getDefaultCategory()
	{
		$category = Db::getInstance()->ExecuteS('SELECT id_category FROM '._DB_PREFIX_.'shop LIMIT 1');
	
		if (!empty($category[0]['id_category'])) {
			$this->_default_category_id = $category[0]['id_category'];
		}

		return $this->_default_category_id;
	}

	private function _importCategories($obj)
	{
		if (empty($obj)) {
			return false;
		}

		$data = array();
		$categories = array();
		$parentsInsert = array();
		foreach ($obj as $values) {
			$data['action'] = (string)$values->attributes()->action;
			$data['id'] = (int)$values->CategorieId;
			$data['IdParent'] = (int)$values->CategorieIdParent;
			$data['CategorieImage'] = (string)$values->CategorieImage;
			$data['CategorieOrdre'] = (int)$values->CategorieOrdre;
			$data['CategorieUrl'] = (string)$values->CategorieUrl;
			foreach ($values->CategorieInfos->CategorieInfo as $languages) {
				$language = Tools::strtolower((string)$languages->attributes()->lang);
				$data['CategorieInfos'][$language]['name'] = (string)$languages->CategorieNom;
				$data['CategorieInfos'][$language]['description'] = (string)$languages->CategorieDescription;
			}
			//gestion des Tags : remplissage
			$data['Tags'] = array();
			if (isset($values->Tags->Tag)) {
				$data['Tags']['Title'] = (string)$values->Tags->Tag->Title;
				$data['Tags']['Description'] = (string)$values->Tags->Tag->Description;
				$data['Tags']['Keywords'] = (string)$values->Tags->Tag->Keywords;
			}
			if ($data['action'] == 'delete') {
				$categories[] = $data;
				$this->_deleteCategory($data['id']);
			} else {
				$id_parent = $this->_importCategory($data);
				if ($id_parent == 0) {
					$tab = array(
							'id_category_conversions' => $data['id'],
							'id_parent_conversions' => $data['IdParent'],
							);
					$parentsInsert[] = $tab;
				}
			}

			// $categories[] = $data;
		}

		foreach ($categories as $category) {
			$this->_parentUpdate($category);
		}

		foreach ($parentsInsert as $parentInsert) {
			$this->insertIdParent($parentInsert);
		}


		$this->fillInsertLevelDepth();

		//pour generer nLeft et nRight
		//Category::regenerateEntireNtree();
	}

	private function _importCategory($category)
	{
		// back-office use on categories ?
		$updateCategories = true;
		if (Configuration::get('PS_ARF_ALLOW_CAT_UPD') == true) {
			$updateCategories = false;

			if ($category['action'] == 'create') {
				$updateCategories = true;
			}
		}

		$date = date('Y-m-d H:i:s');
		$category_id = !empty($this->_conversions['category'][$category['id']]) ? $this->_conversions['category'][$category['id']] : 0;
		//recupération id de catégorie parent  id_parent pas encore ajouter il faut tester si il existe ou si id_parent ==1
		if (!empty($this->_conversions['category'][$category['IdParent']])) {
			$id_parent = $this->_conversions['category'][$category['IdParent']];
		} else {
			$id_parent = ($category['IdParent'] == 1) ? $this->_default_category_id : 0;
		}

		//récupération des informations liées au catégorie parent
		$parent_category = new Category((int)$id_parent);
		if (empty($category_id)) {
			//probleme level_depth
			$addTab = array(
				'id_category' => 0,
				'id_parent' => $id_parent,
				'level_depth' => Validate::isLoadedObject($parent_category) ? (int)$parent_category->level_depth + 1 : 0,
				'active' => (Configuration::get('PS_CATEGORIE_ACTIVATION') == 1) ? 0 : 1,
				'position' => pSQL($category['CategorieOrdre']),
				'date_add' => pSQL($date),
				'date_upd' => pSQL($date),
			);
			Db::getInstance()->insert('category', $addTab);
			//Db::getInstance()->autoExecute(_DB_PREFIX_.'category', $addTab, 'INSERT');
			$category_id = Db::getInstance()->Insert_ID();
			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS CATEGORIES : INSERT CATEGORY_GROUP ', Db::getInstance()->getMsgError());
				}
			}
			// let's insert categories permissions into category_group table
			for ($i = 1; $i < 4; ++$i) {
				Db::getInstance()->Execute('INSERT IGNORE INTO '._DB_PREFIX_.'category_group (`id_category`, `id_group`) 
											VALUES ('.pSQL($category_id).', '.$i.')');
				if (Db::getInstance()->getMsgError()) {
					if (isset($this->_logger)) {
						$this->_logger->logInfo('SQL ERROR  CLASS CATEGORIES : INSERT CATEGORY_GROUP ', Db::getInstance()->getMsgError());
					}
				}
			}

			$addTab = array(
				'imports_conversions_rules_symbol' => $category['id'],
				'imports_conversions_rules_original' => $category_id,
				'imports_conversions_type' => 'category',
			);
			Db::getInstance()->insert('artefact_conversions_rules', $addTab);
			//Db::getInstance()->autoExecute(_DB_PREFIX_.'artefact_conversions_rules', $addTab, 'INSERT');
			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS CATEGORIES : INSERT artefact_conversions_rules ', Db::getInstance()->getMsgError());
				}
			}

			if (isset($this->_logger)) {
				$this->_logger->logInfo(' CLASS CATEGORIES : Updated | ID_ARTIFACT='.$category['id']."\t|\tID=".$category_id);
			}

			$this->_conversions['category'][$category['id']] = $category_id;
		} 
		else {
			//probleme level_depth
			$addTab = array(
				'id_parent' => $id_parent,
				'level_depth' => Validate::isLoadedObject($parent_category) ? (int)$parent_category->level_depth + 1 : 0,
				'active' => 1,
				'date_add' => pSQL($date),
				'date_upd' => pSQL($date),
			);

			if ($updateCategories == false) {
				$addTab['position'] = pSQL($category['CategorieOrdre']);
			}

			Db::getInstance()->update('category', $addTab, 'id_category = '.$category_id);
			//Db::getInstance()->autoExecute(_DB_PREFIX_.'category', $addTab, 'UPDATE', 'id_category = '.$category_id);
			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS CATEGORIES : UPDATE CATEGORY', Db::getInstance()->getMsgError());
				}
			}
		}

		Db::getInstance()->Execute('INSERT IGNORE INTO '._DB_PREFIX_.'category_shop (`id_category`, `id_shop`, `position`) 
									 SELECT "'.pSQL($category_id).'", id_shop,1 FROM '._DB_PREFIX_.'shop');
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS CATEGORIES : INSERT CATEGORY_SHOP ', Db::getInstance()->getMsgError());
			}
		}

		if ($updateCategories == true) {
			foreach ($category['CategorieInfos'] as $language => $values) {
				if (empty($this->_languages[$language])) {
					continue;
				}

				if (!empty($values['name'])) {
					$link_rewrite = Tools::strtolower(str_replace(' ', '-', trim(Tools::substr($values['name'], 0, 120))));
					$link_rewrite = mb_convert_encoding(Tools::link_rewrite($link_rewrite), 'UTF-8');
				} else {
					$link_rewrite = '';
				}

				Db::getInstance()->Execute('INSERT INTO '._DB_PREFIX_.'category_lang (`id_category`, `id_lang`, `name`, `description`, `link_rewrite`) 
											VALUES (
												"'.pSQL($category_id).'", 
												"'.pSQL($this->_languages[$language]).'",
												"'.pSQL($values['name']).'", 
												"'.pSQL($values['description']).'",
												"'.pSQL($link_rewrite).'"
											)
											ON DUPLICATE KEY UPDATE 
											name        = "'.pSQL($values['name']).'", 
											link_rewrite= "'.pSQL($link_rewrite).'",
											description = "'.pSQL($values['description']).'"');
				if (Db::getInstance()->getMsgError()) {
					if (isset($this->_logger)) {
						$this->_logger->logInfo('SQL ERROR  CLASS CATEGORIES : INSERT CATEGORY_LANG ', Db::getInstance()->getMsgError());
					}
				}

				//code du tags
				if (!empty($category['Tags'])) {
					$addTab1 = array(
						'meta_title' => $category['Tags']['Title'],
						'meta_keywords' => $category['Tags']['Keywords'],
						'meta_description' => $category['Tags']['Description'],
					);
					Db::getInstance()->update('category_lang', $addTab1, 'id_category = '.$category_id);
					//Db::getInstance()->autoExecute(_DB_PREFIX_.'category_lang', $addTab1, 'UPDATE', 'id_category = '.$category_id);
					if (Db::getInstance()->getMsgError()) {
						if (isset($this->_logger)) {
							$this->_logger->logInfo('SQL ERROR  CLASS CATEGORIES : UPDATE CATEGORY_LANG ', Db::getInstance()->getMsgError());
						}
					}
				}
			}

			if (!empty($category['CategorieImage'])) {
				$image_path = dirname(__FILE__).'/../../../../arte-fact/inputs/images/arbre/'.$category['CategorieImage'];
				$new_image_path = _PS_CAT_IMG_DIR_.$category_id.'.jpg';
				if (file_exists($image_path)) {
					if (file_exists($new_image_path)) {
						@unlink($new_image_path);
					}

					if (copy($image_path, $new_image_path)) {
						$imagesTypes = ImageType::getImagesTypes('categories');
						foreach ($imagesTypes as $k => $imageType) {
							imageResize($new_image_path, _PS_CAT_IMG_DIR_.$category_id.'-'.Tools::stripslashes($imageType['name']).'.jpg', (int)($imageType['width']), (int)($imageType['height']));
						}
					}
				}
			}
		}

		return $id_parent;
	}

	private function _deleteCategory($category_id)
	{
		$category_id = !empty($this->_conversions['category'][$category_id]) ? $this->_conversions['category'][$category_id] : 0;
		if (empty($category_id)) {
			return false;
		}

		$category_obj = new Category($category_id);
		$category_obj->delete();
		// Delete conversion
		Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'artefact_conversions_rules 
								  WHERE imports_conversions_type = "category" AND imports_conversions_rules_original = '.$category_id);
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS CATEGORIES : DELETE artefact_conversions_rules ', Db::getInstance()->getMsgError());
			}
		}
		unset($this->_conversions['category'][$category_id]);
	}

	/**
	* if action==delete 
	*
	* @param $category
	* @return 
	*/
	private function _parentUpdate($category)
	{
		$category_id = !empty($this->_conversions['category'][$category['id']]) ? $this->_conversions['category'][$category['id']] : 0;
		if (!empty($category_id)) {
			$addTab = array(
				'id_parent' => !empty($this->_conversions['category'][$category['IdParent']]) ? $this->_conversions['category'][$category['IdParent']] : $this->_default_category_id,
				'active' => 1,
			);
			Db::getInstance()->update('category', $addTab, 'id_category = '.$category_id);
			//Db::getInstance()->autoExecute(_DB_PREFIX_.'category', $addTab, 'UPDATE', 'id_category = '.$category_id);
			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS CATEGORIES : UPDATE CATEGORY ', Db::getInstance()->getMsgError());
				}
			}
		}
	}

	/**
	* mise à jour de id_parent where id_parent==0
	*
	* @param $InsertIdParent
	* @return 
	*/
	private function insertIdParent($InsertIdParent)
	{
		$id_category = $this->_conversions['category'][$InsertIdParent['id_category_conversions']];
		$id_parent = $this->_conversions['category'][$InsertIdParent['id_parent_conversions']];
		Db::getInstance()->update('category', array(
		//Db::getInstance()->autoExecute(_DB_PREFIX_.'category', array(
					'id_parent' => (int)$id_parent, ), 'id_category='.$id_category);
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS CATEGORIES : UPDATE CATEGORY ', Db::getInstance()->getMsgError());
			}
		}
	}

	/**
	* parcour de la BD est mise à jour de level en cas de l'insertion
	*
	* @param 
	* @return 
	*/
	private function fillInsertLevelDepth()
	{
		$sql = 'SELECT * FROM '._DB_PREFIX_.'category WHERE id_category !=1 AND id_category !='.$this->_default_category_id.'';
		if ($results = Db::getInstance()->ExecuteS($sql)) {
			foreach ($results as $row) {
				if ($row['level_depth'] == 0) {
					$level = $this->calcLevelDepth($row['id_parent']);
					Db::getInstance()->update('category', array(
					//Db::getInstance()->autoExecute(_DB_PREFIX_.'category', array(
					'level_depth' => (int)$level, ), 'id_category='.$row['id_category']);
				}
			}
		}
	}

	/**
	* fonction recursive calcule level depth
	*
	* @param $id_parent
	* @return calcLevelDepth
	*/
	private function calcLevelDepth($id_parent)
	{
		$parent_category = new Category((int)$id_parent);
		if ($parent_category->level_depth != 0) {
			return (int)$parent_category->level_depth + 1;
		}

		return $this->calcLevelDepth($parent_category->id_parent) + 1;
	}
}
