<?php
/**
* NOTICE OF LICENSE.
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software
* and associated documentation files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge, publish, distribute,
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial
* portions of the Software.
*
* The software is provided "as is", without warranty of any kind, express or implied including 
* but not limited to the warranties of merchantability, fitness for a particular purpose and
* noninfrigement. In no event shall the authors or copyright holdes be liable for any claim, 
* damages or the liability, whether in an action of contract, tort or otherwise, arising from, 
* out of or in connection with the software or the use or other dealings in the software.
*
* @author    Diabolo SARL <ecommerce@diazol.com>
* @copyright 2011 - 2016 Diabolo SARL
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

require_once 'brands.class.php';
require_once 'categories.class.php';
require_once 'taxes.class.php';
require_once 'products.class.php';
require_once 'synch.class.php';
require_once 'publication.class.php';

class ArtifactImport
{
	public $logger;

	public function runImport()
	{
		set_time_limit(0);
		$synch_active = new SynchActive();
		$synch_active->activate();
		$this->_sync();
		$this->_dailyStockFeed();
	}

	public function __construct()
	{
		if (Configuration::get('PS_ARTIFACT_DEBUG') == true) {
			require_once dirname(__FILE__).'/../../vendors/KLogger.php';
			$Directory = _PS_ROOT_DIR_.'/arte-fact/logs/';
			$this->logger = new Klogger($Directory, KLogger::DEBUG);
		}
	}

	private function _sync()
	{
		// let's first check what to deal with ?
		$import_dir = dirname(__FILE__).'/../../../../arte-fact/inputs/files_xml/';
		$version = $this->isArtefactOldVersion($import_dir);
		if ($version) {
			$artEfact_description_xml = $import_dir.'description-artEfact-web.xml';
			$artEfact_import_xml = $import_dir.'artEfact-web.xml';
			$this->_readXMLFile($artEfact_description_xml, $artEfact_import_xml);
		} else {
			$file_Array = array();
			if (file_exists($import_dir)) {
				$files_dir = scandir($import_dir);
				foreach ($files_dir as $k => $file) {
					$pos = strpos(Tools::strtolower($file), 'description');
					if ($pos !== false) {
						$debut = Tools::substr(basename($file), 0, 12);
						$file_Array[] = $debut;
					}
				}
				sort($file_Array);
			} elseif ($this->logger) {
				$this->logger->logInfo('import directory not found', $import_dir);
			}

			$countFiles = count($file_Array);
			for ($i = 0; $i < $countFiles; ++$i) {
				$artEfact_description_xml = $import_dir.''.$file_Array[$i].'-description-artEfact-web.xml';
				$artEfact_import_xml = $import_dir.''.$file_Array[$i].'-artEfact-web.xml';
				$this->_readXMLFile($artEfact_description_xml, $artEfact_import_xml);
			}
		}
		$this->_viderImage();
		if (Configuration::get('PS_CATEGORIE_ACTIVATION') == 1) {
			$publication = new ArtifactPublication();
			$publication->publicationActiveCategories();
		}

		if (Configuration::get('PS_BRAND_ACTIVATION') == 1) {
			$publication = new ArtifactPublication();
			$publication->publicationActiveBrands();
		}
	}

	private function _dailyStockFeed()
	{
		$import_dir = dirname(__FILE__).'/../../../../arte-fact/inputs/files_xml/';
		if (!file_exists($import_dir)) {
			echo "le répertoire d'import n'a pas été trouvé.";
			if (isset($this->logger)) {
				$this->logger->logInfo('import directory not found', $import_dir);
			}

			return false;
		}

		$dailyStockFiles = array();
		$files_dir = scandir($import_dir);

		foreach ($files_dir as $k => $file) {
			$pos = strpos(Tools::strtolower($file), 'deltastock');
			if ($pos !== false) {
				$dailyStockFiles[filemtime($import_dir.$file)] = $file;
			}
		}
		foreach ($dailyStockFiles as $time => $file) {
			if (isset($this->logger)) {
				$this->logger->logInfo('traietement de : ', $file);
			}

			try {
				$obj = @new SimpleXMLElement(trim($import_dir.$file), null, true);
				if (!empty($obj)) {
					$products_obj = new ArtifactImportProducts('');
					$products_obj->importDailyStock($obj->Produits->Produit);
				}
				@unlink(trim($import_dir.$file));
			} catch (Exception $e) {
				if (isset($this->logger)) {
					$this->logger->logInfo('erreur xml : ', $e->getMessage());
				}

				exit();
			}
		}
	}

	private function _readXMLFile($artEfact_description_xml, $artEfact_import_xml)
	{
		$sync = array();

		if (is_file($artEfact_description_xml)) {
			$desc = @new SimpleXMLElement(trim($artEfact_description_xml), null, true);

			if ($desc_products = $desc->Produits) {
				if ($desc_products->attributes()) {
					$attributes = (array)$desc_products->attributes();
					$sync['product'] = !empty($attributes['@attributes']['granularite'])   ? (string)$attributes['@attributes']['granularite']   : '';
				}
			}

			if (!file_exists($artEfact_import_xml)) {
				if (isset($this->logger)) {
					$this->logger->logError('import file not found', $artEfact_description_xml);
				}

				return false;
			}

			libxml_use_internal_errors(true);
			$doc = simplexml_load_file($artEfact_import_xml);
			if (!$doc) {
				echo 'les fichiers de descriptions xml (artefact/inputs/files_xml) sont mal-formatés.<br> Consultez votre fichier de log (artefact/log/) et faites le parvenir au support diazol <br /> maintenance@diazol.com';

				if (isset($this->logger)) {
					$this->logger->logError('le fichier xml suivant : '.$artEfact_import_xml.' contient des erreurs et ne peut être traité.');
				}

				$xml = '';
				$errors = libxml_get_errors();

				foreach ($errors as $error) {
					if (isset($this->logger)) {
						$this->logger->logError($this->displayXmlErrors($error, $xml));
					}
				}
				libxml_clear_errors();

				return false;
			} else {
				$obj = @new SimpleXMLElement(trim($artEfact_import_xml), null, true);
				$version = $obj->attributes()->version;

				if (Configuration::get('PS_ARTIFACT_VERSION') != $version) {
					$filename = _PS_ROOT_DIR_.'/arte-fact/logs/import_products_'.date('Y_m_d-H-i-s').'.log';

					$versionLog = "Sorry, Import can't be run. \n You have wrong versions of application.\n".
							'Your version: '.Configuration::get('PS_ARTIFACT_VERSION')."\n".
							'Need version: '.$version."\n";

					if (isset($this->logger)) {
						$this->logger->logInfo($versionLog);
					}


					$templateVars = array();
					$templateVars['{your_version}'] = Configuration::get('PS_ARTIFACT_VERSION');
					$templateVars['{need_version}'] = $version;

					Mail::Send((int)Language::getIdByIso('fr'), 'erp-version-error', Mail::l('Import error!'), $templateVars, Configuration::get('PS_SHOP_EMAIL'), Configuration::get('PS_SHOP_NAME'), Configuration::get('PS_SHOP_EMAIL'), Configuration::get('PS_SHOP_NAME'), null, null, dirname(__FILE__).'/../../mails/');

					return false;
				}

				if (!empty($obj->Categories->Categorie)) {
					$categories_obj = new ArtifactImportCategories();
					$categories_obj->import($obj->Categories->Categorie);
				}
				if (!empty($obj->Marques->Marque)) {
					$brands_obj = new ArtifactImportBrands();
					$brands_obj->import($obj->Marques->Marque);
				}
				if (!empty($obj->TVAs->TVA)) {
					$taxes_obj = new ArtifactImportTaxes();
					$taxes_obj->import($obj->TVAs->TVA);
				}
				if (!empty($obj->Produits->Produit)) {
					$products_obj = new ArtifactImportProducts($sync);
					if (isset($products_obj->_sync['product']) && $products_obj->_sync['product'] == 'stocksUniquement') {
						$products_obj->importDailyStock($obj->Produits->Produit);
					} else {
						$products_obj->import($obj->Produits->Produit);
					}
				}
				// let's regenerate cattree as needed
				if (!empty($obj->Categories->Categorie)) {
					$this->category = new Category(1, 1);
					$this->category->regenerateEntireNtree();
				}
				@unlink($artEfact_import_xml);
				@unlink($artEfact_description_xml);
			}
		} else {
			if (isset($this->logger)) {
				$this->logger->logInfo('import file not found', $artEfact_description_xml);
			}
		}
	}

	private function isArtefactOldVersion($import_dir)
	{
		if (file_exists($import_dir)) {
			$file = $import_dir.'description-artEfact-web.xml';
			if (file_exists($file)) {
				return true;
			}

			return false;
		}
	}

	private function _viderImage()
	{
		$source = _PS_ROOT_DIR_.'/arte-fact/inputs/';
		$this->deleteFile($source.'images/arbre');
		$this->deleteFile($source.'images/marques');
		$this->deleteFile($source.'images/produits');
	}

	private function deleteFile($repertoire)
	{
		$fichiers = scandir($repertoire);
		unset($fichiers[0], $fichiers[1]);//Suppression de . et ..
		foreach ($fichiers as $f) {
			if (isset($this->logger)) {
				$this->logger->logInfo('image '.$f." est présente dans le dossier d'image mais non lister dans le fichier XML");
			}

			@unlink($repertoire.'/'.$f);
		}
	}

	private function displayXmlErrors($error, $xml)
	{
		$return = $xml[$error->line - 1]."<br />\n";
		$return .= str_repeat('-', $error->column)."^<br />\n";

		switch ($error->level) {
			case LIBXML_ERR_WARNING:
				$return .= "Warning $error->code: ";
				break;
			case LIBXML_ERR_ERROR:
				$return .= "Error $error->code: ";
				break;
			case LIBXML_ERR_FATAL:
				$return .= "Fatal Error $error->code: ";
				break;
		}

		$return .= trim($error->message).
			"<br />\n  Line: $error->line".
			"<br />\n  Column: $error->column";

		if ($error->file) {
			$return .= "<br />\n  File: $error->file";
		}

		return "$return <br />\n\n--------------------------------------------<br />\n\n";
	}
}
