<?php
/**
* NOTICE OF LICENSE.
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software
* and associated documentation files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge, publish, distribute,
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial
* portions of the Software.
*
* The software is provided "as is", without warranty of any kind, express or implied including
* but not limited to the warranties of merchantability, fitness for a particular purpose and
* noninfrigement. In no event shall the authors or copyright holdes be liable for any claim,
* damages or the liability, whether in an action of contract, tort or otherwise, arising from,
* out of or in connection with the software or the use or other dealings in the software.
*
* @author    Diabolo SARL <ecommerce@diazol.com>
* @copyright 2011 - 2016 Diabolo SARL
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

class ArtifactImportProducts
{
	private $_conversions;
	private $_languages;
	private $_logger;
	private $listeCategory;

	public function __construct($sync)
	{
		$log = new ArtifactImport();
		if (isset($log->logger)) {
			$this->_logger = $log->logger;
		}

		// let's first check if we have all enum values in conversion tables due to product_origine in v 1.5.8
		if (version_compare(Configuration::get('PS_ARTIFACT_VERSION'), '1.5.8', '>=') === true)  {
			$this->checkConversionsEnumValue(_DB_PREFIX_."artefact_conversions_rules", "imports_conversions_type");
		}
		$this->_sync = $sync;
	}

    private function  checkConversionsEnumValue( $table, $field )
   {
	$sql="SHOW COLUMNS FROM ". $table ." WHERE Field = '". $field ."'";
    $type = Db::getInstance()->ExecuteS($sql);
    preg_match("/^enum\(\'(.*)\'\)$/", $type[0]['Type'], $matches);
    $enum = explode("','", $matches[1]);
    if(count($enum)<=11) {
    	$sql="ALTER TABLE `"._DB_PREFIX_."artefact_conversions_rules` CHANGE `imports_conversions_type` `imports_conversions_type` enum('brand','category','taxes','taxes_group','attribute_group','attribute','feature','feature_value','product','declinaison','product_origine', 'attribute_to_product') COLLATE 'utf8_general_ci' NOT NULL DEFAULT 'category' AFTER `imports_conversions_rules_id`";
		Db::getInstance()->Execute($sql);

		// let's empty declinaison conversion rules due to 
		$sql='DELETE FROM '._DB_PREFIX_.'artefact_conversions_rules WHERE imports_conversions_type="declinaison"';
		Db::getInstance()->Execute($sql);
		if (Db::getInstance()->getMsgError()) {
			if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCTS checkConversionsEnumValue : Deleting declinaison conversions_rules ",Db::getInstance()->getMsgError());
		}
		else {
			if(isset($this->_logger))  $this->_logger->logInfo("CLASS PRODUCTS checkConversionsEnumValue : Cleaning declinaiton conversions_rules successfull ");
		}

    }
	return true;
    }

	private function categoryChild($id)
	{
		$s = 'SELECT id_category FROM '._DB_PREFIX_."category WHERE id_parent = $id";
		if ($r = Db::getInstance()->ExecuteS($s)) {
			foreach ($r as $row) {
				$this->listeCategory[]=$row['id_category'];
				$this->categoryChild($row['id_category']);
			}
		}
		return true;
	}

    public function getProductParentsCategories($id_category, $id_lang = null)
    {
        $context = Context::getContext()->cloneContext();
        $context->shop = clone($context->shop);

        if (is_null($id_lang)) {
            $id_lang = $context->language->id;
        }

        $categories = null;
        $id_current = $id_category;
        if (count(Category::getCategoriesWithoutParent()) > 1 && Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE') && count(Shop::getShops(true, null, true)) != 1) {
            $context->shop->id_category = (int)Configuration::get('PS_ROOT_CATEGORY');
        } elseif (!$context->shop->id) {
            $context->shop = new Shop(Configuration::get('PS_SHOP_DEFAULT'));
        }
        $id_shop = $context->shop->id;
        while (true) {
            $sql = '
			SELECT c.*, cl.*
			FROM `'._DB_PREFIX_.'category` c
			LEFT JOIN `'._DB_PREFIX_.'category_lang` cl
				ON (c.`id_category` = cl.`id_category`
				AND `id_lang` = '.(int)$id_lang.Shop::addSqlRestrictionOnLang('cl').')';
            if (Shop::isFeatureActive() && Shop::getContext() == Shop::CONTEXT_SHOP) {
                $sql .= ' LEFT JOIN `'._DB_PREFIX_.'category_shop` cs ON (c.`id_category` = cs.`id_category` AND cs.`id_shop` = '.(int)$id_shop.')';
            }
            $sql .= ' WHERE c.`id_category` = '.(int)$id_current;
            if (Shop::isFeatureActive() && Shop::getContext() == Shop::CONTEXT_SHOP) {
                $sql .= ' AND cs.`id_shop` = '.(int)$context->shop->id;
            }
            $root_category = Category::getRootCategory();
            if (Shop::isFeatureActive() && Shop::getContext() == Shop::CONTEXT_SHOP
                && (!Tools::isSubmit('id_category') || (int)Tools::getValue('id_category') == (int)$root_category->id || (int)$root_category->id == (int)$context->shop->id_category)) {
                $sql .= ' AND c.`id_parent` != 0';
            }

            $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($sql);

            if ($result) {
                $categories[] = $result;
            } elseif (!$categories) {
                $categories = array();
            }
            if (!$result || ($result['id_category'] == $context->shop->id_category)) {
                return $categories;
            }
            $id_current = $result['id_parent'];
        }
    }


	public function import($obj)
	{
		if (empty($obj)) {
			return false;
		}

		$listeCategory = array();

		$this->_getConversions();

		$firstRun=(Configuration::get('PS_ARF_FIRST_RUN')==true) ? true : false ;
		if($firstRun==true) {
			$this->_resetProductsReference();
		}
		$this->_importProducts($obj);
	}

	private function _setDefaultProduct()
	{
		Db::getInstance()->Execute('UPDATE IGNORE `'._DB_PREFIX_.'product_attribute` SET default_on = 1');
		Db::getInstance()->Execute('UPDATE IGNORE `'._DB_PREFIX_.'product_attribute_shop` SET default_on = NULL');
		Db::getInstance()->Execute(
			'UPDATE IGNORE `'._DB_PREFIX_.'product_attribute_shop` AS pas
			JOIN `'._DB_PREFIX_.'product_attribute` AS pa
			ON pas.id_product_attribute = pa.id_product_attribute AND pa.default_on = 1
			SET pas.default_on = 1'
		);
	}

	private function _getConversions()
	{
		$conversions = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'artefact_conversions_rules`', false, false);
		while ($row = Db::getInstance()->nextRow($conversions)) {
			$symbol =($row['imports_conversions_type']=='product' || $row['imports_conversions_type']=='declinaison' || $row['imports_conversions_type']=='product_origine') ? $row['imports_conversions_rules_symbol'] : $symbol = Tools::link_rewrite(Tools::replaceAccentedChars((string)$row['imports_conversions_rules_symbol']));
			$this->_conversions[$row['imports_conversions_type']][$symbol] = $row['imports_conversions_rules_original'];
		}

		$languages = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'lang` WHERE active=1', false, false);

		while ($row = Db::getInstance()->nextRow($languages)) {
			$this->_languages[mb_strtolower($row['iso_code'])] = (int)$row['id_lang'];
		}
		unset($conversions, $languages);
	}

	private function _getArtifactReferenceOrigine($artifactReferenceOrigine) {
	
		// let's check if we've already record a product_origine
		$product_id = !empty($this->_conversions['product_origine'][$artifactReferenceOrigine]) ? $this->_conversions['product_origine'][$artifactReferenceOrigine] : 0;
		if(empty($product_id)) {
			// create a conversion rules
			$product_id=$this->_getProductId($artifactReferenceOrigine);
			if(!empty($product_id)) {
				$addTab = array(
					'imports_conversions_rules_symbol'    => $artifactReferenceOrigine,
					'imports_conversions_rules_original'  => $product_id,
					'imports_conversions_type'            => 'product_origine'
				);
				Db::getInstance()->insert('artefact_conversions_rules', $addTab);
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCTS _getArtifactReferenceOrigine : INSERT FROM artefact_conversions_rules ",Db::getInstance()->getMsgError());
				}
				$this->_conversions['product_origine'][$artifactReferenceOrigine] = $product_id;
				return $artifactReferenceOrigine;
			}
		}
		else {
			// let's get Artifact current Reference for replacing all concerned refs !
			$artifactCurrentReference = Db::getInstance()->getValue('SELECT artifact_reference FROM '._DB_PREFIX_.'product WHERE id_product = "'.(int)($product_id).'"');
			if(!empty($artifactCurrentReference)) {
				return $artifactCurrentReference;
			}
			else {
				return $artifactReferenceOrigine;
			}
		}
	
	}

	private function _updateProductArtifactReference($artifactNewRefence, $artifactOldReference) {

		$product_id  = $artifactOldReference;
		$product_id = !empty($this->_conversions['product'][$artifactOldReference]) ? $this->_conversions['product'][$artifactOldReference] : 0;
		if(!empty($product_id)) {
			// update product table
			$addTab = array(
				'artifact_reference'    => pSQL($artifactNewRefence)
			);
			Db::getInstance()->update('product', $addTab, 'id_product = '.$product_id);
			if (Db::getInstance()->getMsgError()) {
				if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCTS _updateProductArtifactReference : UPDATE FROM artefact_conversions_rules ",Db::getInstance()->getMsgError());
			}
			// update product artefact_conversions_rules
			$sql='UPDATE  `'._DB_PREFIX_.'artefact_conversions_rules` SET imports_conversions_rules_symbol = "'.pSQL($artifactNewRefence).'" WHERE imports_conversions_rules_symbol = "'.pSQL($artifactOldReference).'" AND imports_conversions_type="product"';
			Db::getInstance()->Execute($sql);
			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT _updateProductArtifactReference : UPDATE FROM imports_conversions_rules_symbol artifactNewRefence ', Db::getInstance()->getMsgError());
				}
			}
			//pp($sql);
			$this->_conversions['product'][$artifactNewRefence] = $product_id;
		}
	}

	private function _updateDeclinaisonArtifactReference( $declinaisons_array, $produitArtifactReference, $produitArtifactOldReference) {

		if (empty($declinaisons_array)) {
			return false;
		}

		foreach ($declinaisons_array as $key => $declinaisons) {
			/*
			<DeclinaisonReference>LBPLPL-SDE-WI--2-SansBleu</DeclinaisonReference>
			Missing information in xml 
			<DeclinaisonReferenceOrigin>LBPLPL-SEA-SE--1-SansBleu</DeclinaisonReferenceOrigin>
			*/
			// let's replace current reference with the old one
			//$declinaisons['DeclinaisonArtifactOldReference'] = str_replace ($produitArtifactReference, $produitArtifactOldReference,(string)$produitDeclinaison->DeclinaisonReference);

			$declinaisonArtifactOldReference= str_replace ($produitArtifactReference, $produitArtifactOldReference,(string)$declinaisons['DeclinaisonArtifactReference']);
			$id_product_attribute = !empty($this->_conversions['declinaison'][$declinaisonArtifactOldReference]) ? $this->_conversions['declinaison'][$declinaisonArtifactOldReference] : 0;
			if(!empty($id_product_attribute)) {
				// update product_attribute table
				$addTab = array(
					'artifact_reference'    => pSQL($declinaisons['DeclinaisonArtifactReference'])
				);
				Db::getInstance()->update('product_attribute', $addTab, 'id_product_attribute = '.$id_product_attribute);
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCTS _updateDeclinaisonArtifactReference : INSERT FROM artefact_conversions_rules ",Db::getInstance()->getMsgError());
				}
				// update product_attribute table
				$sql='UPDATE  `'._DB_PREFIX_.'artefact_conversions_rules` SET imports_conversions_rules_symbol ="'.pSQL($declinaisons['DeclinaisonArtifactReference']) .'" WHERE imports_conversions_rules_symbol = "'.pSQL($declinaisonArtifactOldReference).'" AND imports_conversions_type="declinaison"';
				//pp($sql);
				Db::getInstance()->Execute($sql);
				if (Db::getInstance()->getMsgError()) {
					if (isset($this->_logger)) {
						$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT _updateDeclinaisonArtifactReference : UPDATE FROM imports_conversions_rules_symbol DeclinaisonArtifactReference ', Db::getInstance()->getMsgError());
					}
				}
				$this->_conversions['declinaison'][$declinaisons['DeclinaisonReference']] = $id_product_attribute;
			}
		}
	}


	private function _resetProductsReference()
	{
		Db::getInstance()->Execute('UPDATE  `'._DB_PREFIX_.'product` SET artifact_reference = NULL');
		Db::getInstance()->Execute('UPDATE  `'._DB_PREFIX_.'product_attribute` SET artifact_reference = NULL');
	}


	private function _productGetPriceHT($ttc, $tvaid)
	{
		$taxid = $this->_conversions['taxes'][$tvaid];
		$tvaRate = Db::getInstance()->getValue('SELECT rate FROM '._DB_PREFIX_.'tax WHERE id_tax = "'.(int)($taxid).'"');
		if($tvaRate) {
			return number_format((float)($ttc / (1 + ($tvaRate/100))), 6, '.', '');
		}
	}

	private function _importProducts($obj)
	{
		if (empty($obj)) {
			return false;
		}

		$ProduitImages=array();
		$artifactReferencesDifferences=$referenceOrigine=false;
		if (version_compare(Configuration::get('PS_ARTIFACT_VERSION'), '1.5.8', '>=') === true)  {
			$referenceOrigine=true;
		}

		foreach ($obj as $values) {
			//$data['ProduitDebutDisponibilite']  = (string)$values->ProduitDebutDisponibilite;  = not on Prestashop
			//$data['ProduitFinDisponibilite']    = (string)$values->ProduitFinDisponibilite;    = not on Prestashop
			//$data['ProduitDateCreation']        = (string)$values->ProduitDateCreation;        = not on Prestashop
			//$data['ProduitDateModification']    = (string)$values->ProduitDateModification;    = not on Prestashop
			//$data['ProduitCommentaire']         = (string)$values->ProduitCommentaire;         = not on Prestashop
			//$data['ProduitStockStatut']         = (string)$values->ProduitStockStatut;         = not on Prestashop
			$data = array();
			$data['action'] = (string)$values->attributes()->action;
			$data['ProduitReference'] = (string)$values->ProduitReference;
			$data['ProduitReferenceSite'] = (string)$values->ProduitReferenceSite;
			$data['ProduitArtifactReference'] = (string)$values->ProduitReference;
			$data['ProduitReferenceFournisseur'] = (string)$values->ProduitReferenceFournisseur;
			$data['ProduitOccasion'] = (string)$values->ProduitOccasion;

			$artifactReferencesDifferences=false;
			if ($referenceOrigine==true)  {
				/*
					<ProduitReference>LBPLPL-SDE-WI--2</ProduitReference>
					<ProduitReferenceOrigine>LBPLPL-SEA-SE--1</ProduitReferenceOrigine>
				*/
				$data['ProduitArtifactOldReference'] = (string)$values->ProduitReferenceOrigine;
			}

			if ($this->_sync['product'] != 'imagesUniquement') {
				$data['MarqueId'] = (int)$values->MarqueId;
				$data['ProduitStatut'] = (int)$values->ProduitStatut;
				$data['ProduitStock'] = (int)$values->ProduitStock;
				$data['ProduitPrixHT'] = (float)(str_replace(',', '.', $values->ProduitPrixHT));
				$data['ProduitPrixTTC'] = (float)(str_replace(',', '.', $values->ProduitPrixTTC));
				if(isset($values->ProduitPrixAchatHT)) {
					$data['ProduitPrixAchatHT']         = (float)(str_replace(',', '.', $values->ProduitPrixAchatHT));
				}
				$data['TVAId'] = (int)$values->TVAId;
				$data['ProduitOccasion'] = (string)$values->ProduitOccasion;
				$data['ProduitPoids'] = (float)$values->ProduitPoids;

				foreach ($values->ProduitInfos->ProduitInfo as $languages) {
					$language = mb_strtolower((string)$languages->attributes()->lang);
					$data['ProduitInfos'][$language]['name'] = (string)((empty($languages->ProduitNomAlternatif)) ? $languages->ProduitNom : $languages->ProduitNomAlternatif);
					if($languages->ProduitDescription=='<p></p><hr></hr>' ) {
						$data['ProduitInfos'][$language]['description'] = '';
					}
					else {
						$data['ProduitInfos'][$language]['description'] = (string)$languages->ProduitDescription;
					}
				}

				if (empty($data['ProduitInfos'])) {
					$data['stock_update_only'] = true;
				}

				if($data['ProduitPrixHT']=='0' && $data['ProduitPrixTTC']!=0) {
					$data['ProduitPrixHT']=$this->_productGetPriceHT($data['ProduitPrixTTC'], $data['TVAId']);
				}

				$data['Categories'] = array();

				foreach ($values->Categories as $categories) {
					$data['Categories'][] = (int)$categories->CategorieId;
				}
			}
			$data['ProduitImages'] = array();

			/*let's delete image only if requested... granularity describe in xml description is available in _sync array*/
			if (isset($this->_sync['product']) && ($this->_sync['product'] == 'tout' || $this->_sync['product'] == 'imagesUniquement')) {
				foreach ($values->ProduitImages->ProduitImage as $images) {
					$image = array();
					if ($images->attributes()) {
						$attributes = (array)$images->attributes();
						$image['link'] = (string)$images;
						$image['type'] = !empty($attributes['@attributes']['type'])   ? (string)$attributes['@attributes']['type']   : '';
						$image['nature'] = !empty($attributes['@attributes']['nature']) ? (string)$attributes['@attributes']['nature'] : '';
					
						if (!empty($image['link']) && !empty($image['type'])) {
							$data['ProduitImages'][] = $image;
						}
					}
				}
			}
			$attributs_array = array();
			$colorsDeclinaisons=array();
			$i = true;
			$stocker = false;
			$not_exist = array();
			$data['ProduitDeclinaisons'] = array();

			foreach ($values->ProduitDeclinaisons->ProduitDeclinaison as $produitDeclinaison) {
				$declinaisons = $image = array();
				foreach($produitDeclinaison->Attributs->Attribut as $attributs) {
					$attributs_id = mb_strtolower((string)$attributs->attributes()->id);
					if($attributs_id=='taille' &&  mb_strtolower($attributs)=='tu') {
						$declinaisons['convertToProduct']['taille']=true;
						continue;
					}
					elseif($attributs_id=='couleur' &&  mb_strtolower($attributs)=='tu') {
						$declinaisons['convertToProduct']['couleur']=true;
						continue;
					}
					elseif($attributs_id=='couleur' &&  mb_strtolower($attributs)=='unique') {
						$declinaisons['convertToProduct']['couleur']=true;
						continue;
					}
					elseif($attributs_id=='couleur' &&  mb_strtolower($attributs)=='sans') {
						$declinaisons['convertToProduct']['couleur']=true;
						continue;
					}
					elseif($attributs_id=='couleur' &&  mb_strtolower($attributs)=='none') {
						$declinaisons['convertToProduct']['couleur']=true;
						continue;
					}
					elseif($attributs_id=='couleur' &&  mb_strtolower($attributs)=='unique') {
						//$declinaisons['Attributs'][$attributs_id] = (string)$attributs;
						$declinaisons['convertToProduct']['couleur']=true;
						//$declinaisons['reset']=true;
						continue;
					}
					elseif($attributs_id=='couleur' &&  mb_strtolower($attributs)=='0') {
						//$declinaisons['Attributs'][$attributs_id] = (string)$attributs;
						$declinaisons['convertToProduct']['couleur']=true;
						//$declinaisons['reset']=true;
						continue;
					}
					else {
						$declinaisons['Attributs'][$attributs_id] = (string)$attributs;
						if($attributs_id=='couleur' && !in_array($declinaisons['Attributs']['couleur'], $colorsDeclinaisons)) 
							$colorsDeclinaisons[]=$declinaisons['Attributs']['couleur'];
						if($i==true)
							$attributs_array[]=$attributs_id;
					}
				}

				if(isset($declinaisons['convertToProduct']['taille']) && isset($declinaisons['convertToProduct']['couleur'])) {
					$data['convertToProduct']=(string)$produitDeclinaison->DeclinaisonReference;
				}
				if(isset($declinaisons['convertToProduct'])) {
					unset($declinaisons['convertToProduct']);
				}

				$declinaisons['DeclinaisonReference'] = (string)$produitDeclinaison->DeclinaisonReference;
				$declinaisons['DeclinaisonArtifactReference'] = (string)$produitDeclinaison->DeclinaisonReference;
				$declinaisons['DeclinaisonReferenceSite'] = (string)$produitDeclinaison->DeclinaisonReferenceFournisseur;
				$declinaisons['DeclinaisonReferenceFournisseur'] = (string)$produitDeclinaison->DeclinaisonReferenceFournisseur;

				if ($i == true) {
					if ($this->checkAttributSchemeColorSize($attributs_array)) {
						$stocker = true;
					}
					$i = false;
				}

				$declinaisons['DeclinaisonColor'] = '';
				if (Configuration::get('PS_ARTIFACT_VERSION') >= '1.5.4') {
					/*
						<Couleur>
							<Nom>Bleu</Nom>
							<Code></Code>
							<CouleurPrimaire>#0000FF</CouleurPrimaire>
						</Couleur>
					*/
					foreach ($produitDeclinaison->Couleur as $color) {
						if (!empty($color->Code)) {
							// we have to sure that this code is an hexa value
							if (preg_match('/^#[a-f0-9]{6}$/i', (string)$color->Code)) {
								$declinaisons['DeclinaisonColor'] = (string)$color->Code;
							} elseif (preg_match('/^[a-f0-9]{6}$/i', (string)$color->Code)) {
							//Check for a hex color string without hash 'c1c2b4'
								$declinaisons['DeclinaisonColor'] = '#'.(string)$color->Code;
							}
						}
						if (empty($declinaisons['DeclinaisonColor']) && !empty($color->CouleurPrimaire)) {
							if (preg_match('/^#[a-f0-9]{6}$/i', (string)$color->CouleurPrimaire)) {
								$declinaisons['DeclinaisonColor'] = (string)$color->CouleurPrimaire;
							} elseif (preg_match('/^[a-f0-9]{6}$/i', (string)$color->CouleurPrimaire)) {
							//Check for a hex color string without hash 'c1c2b4'
								$declinaisons['DeclinaisonColor'] = '#'.(string)$color->CouleurPrimaire;
							}
						}
					}
				}

				$declinaisons['DeclinaisonEAN'] = (string)$produitDeclinaison->DeclinaisonEAN;
				if ($this->_sync['product'] != 'imagesUniquement') {
					$declinaisons['DeclinaisonStock'] = (int)$produitDeclinaison->DeclinaisonStock;
					$declinaisons['DeclinaisonPrixHT'] = (float)(str_replace(',', '.', $produitDeclinaison->DeclinaisonPrixHT));
					if(isset($produitDeclinaison->DeclinaisonPrixAchatHT) && 
						($data['ProduitPrixAchatHT']!=(float)(str_replace(',', '.', $produitDeclinaison->DeclinaisonPrixAchatHT)))
					) {
						$declinaisons['DeclinaisonPrixAchatHT']=(float)(str_replace(',', '.', $produitDeclinaison->DeclinaisonPrixAchatHT));
					}
					else {
						$declinaisons['DeclinaisonPrixAchatHT']=0;
					}
				}
				$declinaisons['DeclinaisonImage'] = (string)$produitDeclinaison->DeclinaisonImage;
				if ($this->_sync['product'] != 'imagesUniquement') {
					$declinaisons['ProduitPrixHT'] = (string)$data['ProduitPrixHT'];
				}

				if($declinaisons['DeclinaisonPrixHT']=='0') {
					$declinaisons['DeclinaisonPrixHT']=$this->_productGetPriceHT(str_replace(',', '.', $produitDeclinaison->DeclinaisonPrixTTC), $data['TVAId']);
				}

				// Images
				if (!empty($produitDeclinaison->DeclinaisonImage)) {
					foreach ($produitDeclinaison->DeclinaisonImage as $DeclinaisonImage) {
						if($this->inArrayR((string)$DeclinaisonImage, $data['ProduitImages'])==false) {
							$attributes = (array)$DeclinaisonImage->attributes();
							$image['link'] = (string)$DeclinaisonImage;
						
							$image['type'] = !empty($attributes['@attributes']['type'])   ? (string)$attributes['@attributes']['type']   : '';
							$image['nature'] = !empty($attributes['@attributes']['nature']) ? (string)$attributes['@attributes']['nature'] : '';

							if ($stocker) {
								$image['color'] = (string)$declinaisons['Attributs']['couleur'];
								$image['reference'][] = (string)$declinaisons['DeclinaisonReference'];
							} else {
								$image['reference'] = (string)$declinaisons['DeclinaisonReference'];
							}
							if (!empty($image['link']) && !empty($image['type'])) {
								$data['ProduitImages'][] = $image;
							}
						}
					}
				}
				// let's update $data['ProduitImages'][index]['reference'] with correct images for each colors
				/*
				if ($stocker && !empty($produitDeclinaison->DeclinaisonImage)) {
					$index = $this->indexOfRef($data['ProduitImages'], $declinaisons['Attributs']['couleur']);
					if (!empty($index)) {
						foreach ($index as $_index) {
							$data['ProduitImages'][$_index]['reference'][] = $declinaisons['DeclinaisonReference'];
							$data['ProduitImages'][$_index]['reference'] = array_unique($data['ProduitImages'][$_index]['reference']);
						}
					}
				}
				*/
				$data['ProduitDeclinaisons'][] = $declinaisons;
			}
			if($stocker) {
				$ProduitImages=$data['ProduitImages'];
				for($i=0;$i <= count($data['ProduitImages']); $i++) {
					for($j=0;$j <= count($data['ProduitDeclinaisons']) ; $j++) {
						//pp($data['ProduitDeclinaisons'][$j]);
						if(!empty($data['ProduitDeclinaisons'][$j]['DeclinaisonImage']) && isset($data['ProduitImages'][$i]['link']) && ($data['ProduitDeclinaisons'][$j]['DeclinaisonImage']==$data['ProduitImages'][$i]['link'])) {
							$ProduitImages[$i]['reference'][]=$data['ProduitDeclinaisons'][$j]['DeclinaisonReference'];
							$ProduitImages[$i]['reference']=array_unique($ProduitImages[$i]['reference']);
						}
					}
				}
				$data['ProduitImages']=$ProduitImages;
				unset($ProduitImages);
			}
			unset($colorsDeclinaisons);
			/*
			pp($colorsDeclinaisons);
			pp($ProduitImages);
			pp($data['ProduitImages']);
			exit;
			*/
			
			if ($this->_sync['product'] != 'imagesUniquement') {
				$data['ProduitComposition'] = $composite = array();
				//gestion des pack : remplissage
				if (isset($values->ProduitComposition)) {
					foreach ($values->ProduitComposition->Compose as $compose) {
						$composite['ComposeReference'] = (string)$compose->ComposeReference;
						$composite['ComposeQuantite'] = (int)$compose->ComposeQuantite;

						if (!empty($composite)) {
							$data['ProduitComposition'][] = $composite;
						}
					}
				}
				//fin gestion pack : remplissage

				//gestion des Tags : remplissage
				$data['Tags'] = array();
				if (isset($values->Tags->Tag)) {
					$data['Tags']['Title'] = (string)$values->Tags->Tag->Title;
					$data['Tags']['Description'] = (string)$values->Tags->Tag->Description;
					$data['Tags']['Keywords'] = (string)$values->Tags->Tag->Keywords;
				}
				//fin gestion des Tags : remplissage

				$data['ProduitProprietes'] = array();
				$propriete = array();
				foreach ($values->ProduitProprietes->ProduitPropriete as $proprietes) {
					$language = mb_strtolower((string)$proprietes->attributes()->lang);
					$propriete['lang'] = $language;
					$propriete['nom'] = (string)$proprietes->attributes()->nom;
					$propriete['valeur'] = (string)$proprietes->attributes()->valeur;

					if ($propriete['nom'] == 'Poids' || $propriete['nom'] =='Poids en kg') {
						$data['ProduitPoids'] = (float) str_replace(',', '.',$propriete['valeur']);
						continue;
					}
					elseif($propriete['nom']=='Ref E-Commerce') {
						$data['ProduitReferenceSite'] = (string)$values->ProduitReferenceSite;
					}
					elseif($propriete['nom']=='eco-mobilier' && !empty($propriete['valeur'])) {
						$ecoTaxData=explode(" ", $propriete['valeur']);
						$data['ecotax']= (float) str_replace(',', '.',$ecoTaxData[0]) ;
					}
					else {
						$data['ProduitProprietes'][$propriete['nom']][$propriete['lang']] = $propriete['valeur'];
					}
				}
				unset($propriete, $language);
				$data['ProduitAccessories'] = array();
				foreach ($values->ProduitComplementaires->ProduitComplementaire as $complementaire) {
					$reference = (string)$complementaire;
					if (!empty($reference)) {
						$data['ProduitAccessories'][] = $reference;
					}
				}

				foreach ($values->ProduitEquivalents->ProduitEquivalent as $equivalent) {
					$reference = (string)$equivalent;
					if (!empty($reference)) {
						$data['ProduitAccessories'][] = $reference;
					}
				}

				$data['Soldes'] = array();
				foreach ($values->Soldes->Solde as $solde) {
					// xml solde could be empty
					if (isset($solde->SoldePrix)) {
						$tmpSolde = array();
						$attribute = array();
						$attributes = (array)$solde->attributes();

						if (isset($attributes['@attributes']['quantite'])) {
							$tmpSolde['quantite'] = $attributes['@attributes']['quantite'];
						} else {
							foreach ($attributes as $key => $value) {
								$attribute[$key] = $value;
							}
						}

						$tmpSolde['type'] = (string)$solde->SoldePrix->attributes()->type;
						if ($tmpSolde['type'] == 'prixTTC') {
							$tmpSolde['value'] = (float)(str_replace(',', '.', $data['ProduitPrixTTC']) - str_replace(',', '.', $solde->SoldePrix));
						} else {
							$tmpSolde['value'] = (float)(str_replace(',', '.', $solde->SoldePrix));
						}
						if(!empty($solde->SoldeDateDebut)) {
							list($day, $month, $year) = sscanf(((string)$solde->SoldeDateDebut), '%02d/%02d/%04d');
							$tmpSolde['SoldeDateDebut'] = "$year-$month-$day 00:00:01";
						}
						else {
							$tmpSolde['SoldeDateDebut'] = "";
						}
						if(!empty($solde->SoldeDateFin)) {
							list($day, $month, $year) = sscanf(((string)$solde->SoldeDateFin), '%02d/%02d/%04d');
							$tmpSolde['SoldeDateFin'] = "$year-$month-$day 23:59:59";
						}
						else {
							$tmpSolde['SoldeDateFin'] = "";
						}
						if (!empty($attribute)) {
							$tmpSolde['attribute'] = (array)$attribute;
						}
						array_push($data['Soldes'], $tmpSolde);
						unset($tmpSolde);
					}
				}
			}

			// let's check if we need to update reference...
			if($data['action']=='modify') {
				if($referenceOrigine==true && ($data['ProduitArtifactReference']!= $data['ProduitArtifactOldReference'])) {
					//pp($data['ProduitArtifactReference'] . ' '. $data['ProduitArtifactOldReference']);
					// let's first check if product has already changed...
					$ArtifactCurrentReference=$this->_getArtifactReferenceOrigine($data['ProduitArtifactOldReference']);
					$this->_updateProductArtifactReference($data['ProduitArtifactReference'], $ArtifactCurrentReference);
					$this->_updateDeclinaisonArtifactReference( $data['ProduitDeclinaisons'], $data['ProduitArtifactReference'], $ArtifactCurrentReference);
				}
			}

			if ($data['action'] == "delete") {
				$this->_deleteProduct($data['ProduitReference']);
			} 
			else {
				if($data['ProduitOccasion']==1 && $data['ProduitStock']>0) {
					$this->_importProduct($data);
				}
				else {
					$this->_importProduct($data);
				}
			}
		}
	}

	private function _importProduct($product)
	{
		$date = date('Y-m-d H:i:s');
		$firstRun=(Configuration::get('PS_ARF_FIRST_RUN')==true) ? true : false ;

		if (empty($product['ProduitReference'])) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : ProduitReference is empty, product can't be imported");
			}

			Mail::Send((int)Language::getIdByIso('fr'), 'erp-import-error', Mail::l('Import error!'), array(), Configuration::get('PS_SHOP_EMAIL'), Configuration::get('PS_SHOP_NAME'), Configuration::get('PS_SHOP_EMAIL'), Configuration::get('PS_SHOP_NAME'), null, null, dirname(__FILE__).'/../../mails/');

			return false;
		}

		$product_id = $this->_getProductId($product['ProduitArtifactReference']);
		$root_category = Category::getRootCategory();

		if($firstRun==true) {
			$product_id  = $product['ProduitReferenceSite'];
			if(!empty($product_id)) {
				// let's record a conversion due to unsync product
				$addTab = array(
					'imports_conversions_rules_symbol'    => $product['ProduitArtifactReference'],
					'imports_conversions_rules_original'  => (int)$product_id,
					'imports_conversions_type'            => 'product'
				);

				Db::getInstance()->insert('artefact_conversions_rules', $addTab);
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCTS FIRST RUN : INSERT FROM artefact_conversions_rules ",Db::getInstance()->getMsgError());
				}
				$this->_conversions['product'][$product['ProduitArtifactReference']] = $product_id;
			}
		}
		else {
			// due to unsync product, let's check conversion table
			$product_id = !empty($this->_conversions['product'][$product['ProduitArtifactReference']]) ? $this->_conversions['product'][$product['ProduitArtifactReference']] : 0;
			if (empty($product_id)) {
				if($product['action']=='modify') {
					// product has never been sync in create mode, we need to insert name and default Category
					$product['action']='create';
				}
			}
			elseif($product['action']=='create') {
				$product['action']='modify';
			}
		}

		if ($this->_sync['product'] != 'imagesUniquement') {
			$brand_id = !empty($this->_conversions['brand'][$product['MarqueId']])         ? $this->_conversions['brand'][$product['MarqueId']]         : 0;
			$category_id = !empty($this->_conversions['category'][$product['Categories'][0]]) ? $this->_conversions['category'][$product['Categories'][0]] : 0;
			if(empty($category_id)) {
				$category_id      = !empty($root_category->id_category) ? $root_category->id_category : 2 ;
			}
			$taxes_group_id = !empty($this->_conversions['taxes_group'][$product['TVAId']])      ? $this->_conversions['taxes_group'][$product['TVAId']]      : 0;
			$tax_id = !empty($this->_conversions['taxes'][$product['TVAId']])            ? $this->_conversions['taxes'][$product['TVAId']]            : 0;

			$on_sale = 0;
			$price = $product['ProduitPrixHT'];
			if (count($product['Soldes']) > 0) {
				// let's check if we've got a valid discount price for today !
				$currentSale=false;
				$today= new Datetime($date);
				foreach ($product['Soldes'] as $solde) {
					if(!empty($solde['SoldeDateDebut']) && !empty($solde['SoldeDateFin'])) {
						$startDate= new Datetime($solde['SoldeDateDebut']);
						$endDate= new Datetime($solde['SoldeDateFin']);
						if($this->isDateBetweenDates(new Datetime($date), $startDate, $endDate)== true) {
							$currentSale=true;
							break;
						}
					}
				}
				if($currentSale==true) {
					$on_sale = 1;
				}
			}

			$price = $product['ProduitPrixHT'];
			if (isset($product['ecotax'])) {
				$ecoTaxHT=(float) $product['ecotax']/1.2;
				$price=$price-$ecoTaxHT;
			}
			$addTab = array(
				'id_manufacturer' => (int)$brand_id,
				'id_tax_rules_group' => (int)$taxes_group_id,
				'id_category_default' => (int)$category_id,
				'quantity' => pSQL($product['ProduitStock']),
				'minimal_quantity' => 1,
				'price' => isset($product['Soldes']['quantite']) ? pSQL($product['ProduitPrixHT']) : pSQL($price),
				'reference' => pSQL($product['ProduitReferenceFournisseur']),
				'artifact_reference' => pSQL($product['ProduitArtifactReference']),
				'supplier_reference' => pSQL($product['ProduitReferenceFournisseur']),
				'available_for_order' => 1,
				'show_price' => 1,
				'on_sale' => $on_sale,
				/*
				'active' => 1,
				*/
				'ecotax' => (isset($product['ecotax']) ? $product['ecotax'] : 0),
				'condition' => ($product['ProduitOccasion'] == 1 ? 'used' : 'new'),
				'weight' => pSQL($product['ProduitPoids']),
				'date_upd' => pSQL($date),
			);

			if(isset($product['ProduitPrixAchatHT'])) {
				$addTab['wholesale_price'] = (float)($product['ProduitPrixAchatHT']);
			}
			if($product['ProduitOccasion']==1) {
				$addTab['price']=$this->_productGetPriceHT($product['ProduitPrixTTC'], $product['TVAId']);
			}

			if($firstRun==true && !empty($product['ProduitReferenceSite']) || ($firstRun==false && $product['action']=='modify')) { 
				unset($addTab['reference']);
				unset($addTab['id_category_default']);
				unset($addTab['active']);
				/*
				unset($addTab['weight']);
				*/
			}

			if (isset($product['Soldes']['quantite'])) {
				$addTab['unity'] = (int)$product['Soldes']['quantite'];

				if ($product['Soldes']['type'] == 'pourcent') {
					$unit_price_ratio = 1 / (1 - $product['Soldes']['value']);
				} elseif ($product['Soldes']['type'] == 'prixHT') {
					$unit_price_ratio = $product['ProduitPrixHT'] / $product['Soldes']['value'];
				} elseif ($product['Soldes']['type'] == 'prixTTC') {
					$unit_price_ratio = $product['ProduitPrixTTC'] / $product['Soldes']['value'];
				}

				$addTab['unit_price_ratio'] = $unit_price_ratio;
			}

			if($firstRun==false) {
				if ($product['ProduitStatut'] == 3) {
					$addTab['active'] = 0;
				} elseif ($product['ProduitStatut'] == 4 && empty($product['ProduitStock'])) {
					$addTab['active'] = 0;
				}
				else {
					unset($addTab['active']);
				}
			}

			if (empty($product_id)) {

				// in case sync doesn't perform as expected...
				if($product['action']=='modify') {
					$product['action']='create';
				}

				$product['out_of_stock'] = (Configuration::get('PS_ORDER_OUT_OF_STOCK')==true) ? 2 : 0;

				$addTab['id_product'] = 0;
				$addTab['date_add'] = pSQL($date);

				Db::getInstance()->insert('product', $addTab);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'product', $addTab, 'INSERT');
				if (Db::getInstance()->getMsgError()) {
					if (isset($this->_logger)) {
						$this->_logger->logInfo("SQL ERROR CLASS PRODUCT : product can't be insert ", Db::getInstance()->getMsgError());
					}
				}
				$product_id = Db::getInstance()->Insert_ID();
				if (empty($product_id)) {
					return false;
				}
				// let's first check if a conversion has been recorded...
				if(!isset($this->_conversions['product'][$product['ProduitArtifactReference']])) {
					// let's insert a conversion rules 
					$inserTab = array(
						'imports_conversions_rules_symbol'    => pSQL($product['ProduitArtifactReference']),
						'imports_conversions_rules_original'  => $product_id,
						'imports_conversions_type'            => 'product'
					);
					Db::getInstance()->insert('artefact_conversions_rules', $inserTab, false, true, Db::INSERT_IGNORE);
					if (Db::getInstance()->getMsgError()) {
						if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : INSERT FROM artefact_conversions_rules ",Db::getInstance()->getMsgError());
					}
					$this->_conversions['product'][$product['ProduitArtifactReference']] = $product_id;
					unset($inserTab);
				}

				$addTab['id_product'] = $product_id;
				$addTab['id_shop'] = (int)Context::getContext()->shop->id;
				unset($addTab['id_manufacturer']);
				unset($addTab['quantity']);
				unset($addTab['supplier_reference']);
				unset($addTab['artifact_reference']);
				unset($addTab['reference']);
				unset($addTab['weight']);

				Db::getInstance()->insert('product_shop', $addTab);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'product_shop', $addTab, 'INSERT');

				if (Db::getInstance()->getMsgError()) {
					if (isset($this->_logger)) {
						$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT PRODUCT_SHOP  ', Db::getInstance()->getMsgError());
					}
				}

				if (isset($this->_logger)) {
					$this->_logger->logInfo(' CLASS PRODUCT : lpl='.$product['ProduitReference']."\t\t\t|\tID=".$product_id);
				}
			} else {
				// let's first retriev out_of_stock information before deleting this entry.
				$product['out_of_stock']= Db::getInstance()->getValue('SELECT out_of_stock FROM '._DB_PREFIX_.'stock_available WHERE id_product = "'.pSQL($product_id).'"');

				if (!empty($product['stock_update_only'])) {
					unset($addTab);
					$addTab['quantity'] = pSQL($product['ProduitStock']);
				}

				Db::getInstance()->update('product', $addTab, 'id_product = '.$product_id);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'product', $addTab, 'UPDATE', 'id_product = '.$product_id);
				if (Db::getInstance()->getMsgError()) {
					if (isset($this->_logger)) {
						$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : PRODUCT UPDATE ', Db::getInstance()->getMsgError());
					}
				}

				if (empty($product['stock_update_only'])) {
					unset($addTab['id_manufacturer']);
					unset($addTab['quantity']);
					unset($addTab['reference']);
					unset($addTab['supplier_reference']);
					unset($addTab['artifact_reference']);
					unset($addTab['weight']);
					Db::getInstance()->update('product_shop', $addTab, 'id_product = '.$product_id);
					//Db::getInstance()->autoExecute(_DB_PREFIX_.'product_shop', $addTab, 'UPDATE', 'id_product = '.$product_id);
					if (Db::getInstance()->getMsgError()) {
						if (isset($this->_logger)) {
							$this->_logger->logInfo('SQL ERROR: UPDATE PRODUCT_SHOP ', Db::getInstance()->getMsgError());
						}
					}

					if (isset($this->_logger)) {
						$this->_logger->logInfo(' CLASS PRODUCT : Updated | reference='.$product['ProduitArtifactReference']."\t|\tID=".$product_id);
					}
				}
			}

			Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'stock_available WHERE id_product = "'.pSQL($product_id).'" AND id_product_attribute =0');
			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : DELETE STOCK_AVAILABLE', Db::getInstance()->getMsgError());
				}
			}

			// case something went wrong inserting stock available...
			if(empty($product['out_of_stock'])) {
				$product['out_of_stock'] = (Configuration::get('PS_ORDER_OUT_OF_STOCK')==true) ? 2 : 0;
			}

			$addStock = array(
				'id_product' => pSQL($product_id),
				'id_product_attribute' => 0,
				'id_shop' => (int)Context::getContext()->shop->id,
				'quantity' => pSQL($product['ProduitStock']),
				'depends_on_stock' => 0,
				'out_of_stock' => pSQL($product['out_of_stock']),
			);
			/*
				'reserved_quantity' => 0,
				'physical_quantity' => 0,
			*/
			Db::getInstance()->insert('stock_available', $addStock);
			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT STOCK_AVAILABLE', Db::getInstance()->getMsgError());
				}
			}
			if (!empty($product['stock_update_only'])) {
				return true;
			}

			// back-office use on product description  or on product name ?
			$updateProductDesc = $updateProductName = true;
			if (Configuration::get('PS_ARF_ALLOW_PROD_UPD_DESC') == true) {
				$updateProductDesc = false;
				if ($product['action'] == 'create') {
					$updateProductDesc = true;
				}
			}
			if (Configuration::get('PS_ARF_ALLOW_PROD_UPD_NAME') == true) {
				$updateProductName = false;
				if ($product['action'] == 'create') {
					$updateProductName = true;
				}
			}

			if($product['action']=='create' &&  $firstRun==false) {
				$updateProductDesc=true;
				$updateProductName=true;
			}

			if ($updateProductDesc == true || $updateProductName == true) {
				foreach ($product['ProduitInfos'] as $language => $values) {
					if (empty($this->_languages[$language])) {
						continue;
					}

					if (!empty($values['name'])) {
						$link_rewrite = mb_strtolower(str_replace(' ', '-', trim(Tools::substr($values['name'], 0, 120))));
						$link_rewrite = mb_convert_encoding(Tools::link_rewrite($link_rewrite), 'UTF-8');
					} else {
						$link_rewrite = '';
					}

					$description = (strpos($values['description'], '<tt>') !== false) ? Tools::substr($values['description'], 4, -5) : $values['description'];
					$productDescription = $this->_productShortDesc($description, $this->_languages[$language]);

					if ($updateProductDesc == true && $updateProductName == true) {
						$sql='INSERT INTO '._DB_PREFIX_.'product_lang (`id_product`, `id_lang`, `name`,`description_short`, `description`, `link_rewrite`)
							VALUES (
								"'.(int)($product_id).'",
								"'.pSQL($this->_languages[$language]).'",
								"'.pSQL($values['name']).'",
								"'.addslashes($productDescription['short']).'",
								"'.addslashes($productDescription['long']).'",
								"'.pSQL($link_rewrite).'"
							)
							ON DUPLICATE KEY UPDATE
							name                     = "'.pSQL($values['name']).'",
							description_short        = "'.addslashes($productDescription['short']).'",
							link_rewrite             = "'.pSQL($link_rewrite).'",
							description              = "'.addslashes($productDescription['long']).'"';
					}
					elseif ($updateProductDesc == true && $updateProductName == false) {
						$sql='INSERT INTO '._DB_PREFIX_.'product_lang (`id_product`, `id_lang`,`description_short`, `description`)
							VALUES (
								"'.(int)($product_id).'",
								"'.pSQL($this->_languages[$language]).'",
								"'.addslashes($productDescription['short']).'",
								"'.addslashes($productDescription['long']).'"
							)
							ON DUPLICATE KEY UPDATE
							description_short        = "'.addslashes($productDescription['short']).'",
							description              = "'.addslashes($productDescription['long']).'"';
					}
					else {
						$sql='INSERT INTO '._DB_PREFIX_.'product_lang (`id_product`, `id_lang`, `link_rewrite`)
							VALUES (
								"'.(int)($product_id).'",
								"'.pSQL($this->_languages[$language]).'",
								"'.pSQL($link_rewrite).'"
							)
							ON DUPLICATE KEY UPDATE
							name                     = "'.pSQL($values['name']).'",
							link_rewrite             = "'.pSQL($link_rewrite).'"';
					}
					Db::getInstance()->Execute($sql);
					if (Db::getInstance()->getMsgError()) {
						if (isset($this->_logger)) {
							$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT PRODUCT_LANG ', Db::getInstance()->getMsgError());
						}
					}
					//code du tags
					if (!empty($product['Tags'])) {
						$addTab1 = array(
							'meta_title' => pSQL($product['Tags']['Title']),
							'meta_keywords' => pSQL($product['Tags']['Keywords']),
							'meta_description' => pSQL($product['Tags']['Description']),
						);
						Db::getInstance()->update('product_lang', $addTab1, 'id_product = '.(int)$product_id);
						//Db::getInstance()->autoExecute(_DB_PREFIX_.'product_lang', $addTab1, 'UPDATE', 'id_product = '.(int)$product_id);
					}
				}
			}

			// back-office use on categories ?
			$updateProductCategories = true;
			if (Configuration::get('PS_ARF_ALLOW_PROD_UPD_CAT') == true) {
				$updateProductCategories = false;
				if ($product['action'] == 'create') {
					$updateProductCategories = true;
				}
			}
			if($product['action']=='create' &&  $firstRun==false) {
				$updateProductCategories=true;
				$sql='INSERT INTO '._DB_PREFIX_.'category_product (`id_product`, `id_category`)
					VALUES (
						"'.pSQL($product_id).'",
						"'.pSQL($category_id).'"
					)';
				Db::getInstance()->Execute($sql);
				unset($sql);
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : INSERT CATEGORY_PRODUCT ",Db::getInstance()->getMsgError());
				}
				else {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL CLASS PRODUCT : INSERT CATEGORY_PRODUCT ", $product_id. ' -> '. $category_id);
				}

			}
			if ($updateProductCategories == true) {
				// Artifact is sending us a single category ?
				if (count($product['Categories']) == 1 && Configuration::get('PS_PRODUCT_CATEGORIES_PARENT') == true) {
					$category_id = !empty($this->_conversions['category'][$product['Categories'][0]]) ? $this->_conversions['category'][$product['Categories'][0]] : 0;

					if (!empty($category_id)) {
						$parents = $this->getProductParentsCategories($category_id);
						unset($product['Categories']);
						
						foreach ($parents as $parent) {
							if ($parent['is_root_category'] == false) {
								$product['Categories'][] = (int)$parent['id_category'];
							} elseif ($product['ProduitStatut'] == 1) {
								$product['Categories'][] = (int)$parent['id_category'];
							}
						}
						foreach ($product['Categories'] as $key => $category) {
							$sql = 'INSERT INTO '._DB_PREFIX_.'category_product (`id_product`, `id_category`)
								VALUES (
									"'.(int)($product_id).'",
									"'.(int)($category).'"
								)';

							Db::getInstance()->Execute($sql);
							unset($sql);
							if (Db::getInstance()->getMsgError()) {
								if (isset($this->_logger)) {
									$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT CATEGORY_PRODUCT ', Db::getInstance()->getMsgError());
								}
							} else {
								if (isset($this->_logger)) {
									$this->_logger->logInfo('SQL CLASS PRODUCT : INSERT CATEGORY_PRODUCT ', $product_id.' -> '.$category);
								}
							}
						}

						Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'category_product WHERE id_product = "'.(int)($product_id).'"');
						if (Db::getInstance()->getMsgError()) {
							if (isset($this->_logger)) {
								$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : DELETE CATEGORY_PRODUCT ', Db::getInstance()->getMsgError());
							}
						}

						foreach ($product['Categories'] as $key => $category) {

							//$category_id = !empty($this->_conversions['category'][$category]) ? $this->_conversions['category'][$category] : 0;
							//if (empty($category_id)) {
							//	continue;
							//}
							//
							$sql = 'INSERT INTO '._DB_PREFIX_.'category_product (`id_product`, `id_category`)
								VALUES (
									"'.(int)($product_id).'",
									"'.(int)($category).'"
								)';

							Db::getInstance()->Execute($sql);
							unset($sql);
							if (Db::getInstance()->getMsgError()) {
								if (isset($this->_logger)) {
									$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT CATEGORY_PRODUCT ', Db::getInstance()->getMsgError());
								}
							} else {
								if (isset($this->_logger)) {
									$this->_logger->logInfo('SQL CLASS PRODUCT : INSERT CATEGORY_PRODUCT ', $product_id.' -> '.$category);
								}
							}
						}
					}
				} else {
					Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'category_product WHERE id_product = "'.pSQL($product_id).'"');
					if (Db::getInstance()->getMsgError()) {
						if (isset($this->_logger)) {
							$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : DELETE CATEGORY_PRODUCT ', Db::getInstance()->getMsgError());
						}
					}

					foreach ($product['Categories'] as $key => $category) {
						$category_id = !empty($this->_conversions['category'][$category]) ? $this->_conversions['category'][$category] : 0;
						if (empty($category_id)) {
							continue;
						}

						Db::getInstance()->Execute(
							'INSERT INTO '._DB_PREFIX_.'category_product (`id_product`, `id_category`)
							VALUES (
								"'.(int)($product_id).'",
								"'.(int)($category_id).'"
							)'
						);

						if (Db::getInstance()->getMsgError()) {
							if (isset($this->_logger)) {
								$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT CATEGORY_PRODUCT ', Db::getInstance()->getMsgError());
							}
						}
					}
					if ($product['ProduitStatut'] == 1) {
						Db::getInstance()->Execute(
							'INSERT IGNORE INTO '._DB_PREFIX_.'category_product (`id_product`, `id_category`)
							VALUES (
								"'.(int)($product_id).'",
								"2"
							)'
						);
						if (Db::getInstance()->getMsgError()) {
							if (isset($this->_logger)) {
								$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT CATEGORY_PRODUCT WHERE ProduitStatut=1', Db::getInstance()->getMsgError());
							}
						}
					}
				}
			} // end updateProductCategories=true;

			Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'accessory WHERE id_product_1 = "'.pSQL($product_id).'"');
			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : DELETE ACCESSORY', Db::getInstance()->getMsgError());
				}
			}
			foreach ($product['ProduitAccessories'] as $key => $reference) {

				$accessory_product_id = !empty($this->_conversions['product'][$reference]) ? $this->_conversions['product'][$reference] : 0;

				if (empty($accessory_product_id)) {
					continue;
				}

				Db::getInstance()->Execute(
					'INSERT INTO '._DB_PREFIX_.'accessory (`id_product_1`, `id_product_2`)
					VALUES (
						"'.(int)($product_id).'",
						"'.(int)($accessory_product_id).'"
					)'
				);
				if (Db::getInstance()->getMsgError()) {
					if (isset($this->_logger)) {
						$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT ACCESSORY', Db::getInstance()->getMsgError());
					}
				}
			}

			$updateFeatures=(Configuration::get('PS_ARF_ALLOW_PROD_SPECS_UPD')==true) ? false : true ;
			if($updateFeatures==false && count($product['ProduitProprietes'])>0) {
				// dérives case

				$productObject=new Product($product_id);
				$productObject->deleteFeatures();

				foreach ($product['ProduitProprietes'] as $propriete_name => $proprietes) {

					if (empty($proprietes['fr'])) {
						continue;
					}

					$feature = $this->truncateWord((string)$propriete_name, 325);
					$id_feature = !empty($this->_conversions['feature'][Tools::link_rewrite(Tools::replaceAccentedChars($feature))]) ? $this->_conversions['feature'][Tools::link_rewrite(Tools::replaceAccentedChars($feature))] : 0;

					$feature_value = $this->truncateWord((string)$proprietes['fr'], 325);
					$id_feature_value = !empty($this->_conversions['feature_value'][Tools::link_rewrite(Tools::replaceAccentedChars($feature_value))]) ? $this->_conversions['feature_value'][Tools::link_rewrite(Tools::replaceAccentedChars($feature_value))] : 0;

					if (empty($id_feature)) {
						$addTab = array(
							'id_feature' => 0,
						);

						Db::getInstance()->insert('feature', $addTab);
						//Db::getInstance()->autoExecute(_DB_PREFIX_.'feature', $addTab, 'INSERT');
						if (Db::getInstance()->getMsgError()) {
							if (isset($this->_logger)) {
								$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT FEATURE ', Db::getInstance()->getMsgError());
							}
						}
						$id_feature = Db::getInstance()->Insert_ID();

						$addTab = array(
								'id_feature' => (int)$id_feature,
								'id_shop' => (int)Context::getContext()->shop->id,
						);
						Db::getInstance()->insert('feature_shop', $addTab);
						//Db::getInstance()->autoExecute(_DB_PREFIX_.'feature_shop', $addTab, 'INSERT');
						if (Db::getInstance()->getMsgError()) {
							if (isset($this->_logger)) {
								$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT INTO FEATURE_SHOP', Db::getInstance()->getMsgError());
							}
						}

						$addTab = array(
							'imports_conversions_rules_symbol' => addslashes($feature),
							'imports_conversions_rules_original' => (int)$id_feature,
							'imports_conversions_type' => 'feature',
						);
						Db::getInstance()->insert('artefact_conversions_rules', $addTab);
						//Db::getInstance()->autoExecute(_DB_PREFIX_.'artefact_conversions_rules', $addTab, 'INSERT');
				
						if (Db::getInstance()->getMsgError()) {
							if (isset($this->_logger)) {
								$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT artefact_conversions_rules ', Db::getInstance()->getMsgError());
							}
						}

						$this->_conversions['feature'][Tools::link_rewrite(Tools::replaceAccentedChars($feature))] = $id_feature;

						foreach ($this->_languages as $code => $language_id) {
							Db::getInstance()->Execute(
								'INSERT INTO '._DB_PREFIX_.'feature_lang (`id_feature`, `id_lang`, `name`)
								VALUES (
									"'.(int)($id_feature).'",
									"'.(int)($language_id).'",
									"'.addslashes($feature).'"
								)
								ON DUPLICATE KEY UPDATE
								name        = "'.addslashes($feature).'"'
							);
							if (Db::getInstance()->getMsgError()) {
								if (isset($this->_logger)) {
									$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT INTO FROM FEATURE_LANG ', Db::getInstance()->getMsgError());
								}
							}
						}
					}

					if (empty($id_feature_value)) {
						$addTab = array(
							'id_feature_value' => 0,
							'id_feature' => (int)$id_feature,
						);

						Db::getInstance()->insert('feature_value', $addTab);
						//Db::getInstance()->autoExecute(_DB_PREFIX_.'feature_value', $addTab, 'INSERT');
						if (Db::getInstance()->getMsgError()) {
							if (isset($this->_logger)) {
								$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT FEATURE_VALUE WHERE NOT EXIST IN _conversions ARRAY', Db::getInstance()->getMsgError());
							}
						}

						$id_feature_value = Db::getInstance()->Insert_ID();

						$addTab = array(
							'imports_conversions_rules_symbol' => addslashes($feature_value),
							'imports_conversions_rules_original' => (int)$id_feature_value,
							'imports_conversions_type' => 'feature_value',
						);
						Db::getInstance()->insert('artefact_conversions_rules', $addTab);
						//Db::getInstance()->autoExecute(_DB_PREFIX_.'artefact_conversions_rules', $addTab, 'INSERT');
				
						if (Db::getInstance()->getMsgError()) {
							if (isset($this->_logger)) {
								$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT artefact_conversions_rules WHERE NOT EXIST FEATURE IN _conversions ARRAY', Db::getInstance()->getMsgError());
							}
						}

						$this->_conversions['feature_value'][Tools::link_rewrite(Tools::replaceAccentedChars($feature_value))] = $id_feature_value;
						$update_feature_value_lang = false;
					} else {
						$update_feature_value_lang = true;
					}

					foreach ($proprietes as $language => $value) {
						if (empty($this->_languages[$language]) || ($language == 'fr' && $update_feature_value_lang)) {
							continue;
						}

						Db::getInstance()->Execute(
							'INSERT INTO '._DB_PREFIX_.'feature_value_lang (`id_feature_value`, `id_lang`, `value`)
							VALUES (
								"'.(int)($id_feature_value).'",
								"'.(int)($this->_languages[$language]).'",
								"'.addslashes($value).'"
							)
							ON DUPLICATE KEY UPDATE
							value        = "'.addslashes($feature_value).'"'
						);
						if (Db::getInstance()->getMsgError()) {
							if (isset($this->_logger)) {
								$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT INTO FROM FEATURE_VALUE_LANG', Db::getInstance()->getMsgError());
							}
						}
					}

					$addTab = array(
						'id_product' => (int)$product_id,
						'id_feature_value' => (int)$id_feature_value,
						'id_feature' => (int)$id_feature,
					);

					Db::getInstance()->insert('feature_product', $addTab);
					if (Db::getInstance()->getMsgError()) {
						if (isset($this->_logger)) {
							$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT FEATURE_PRODUCT ', Db::getInstance()->getMsgError());
						}
					}

					$id_feature_value = Db::getInstance()->Insert_ID();
				}
			}
			// end if updateFeatures is false


			// an Artifact product with declinaison might be a simple product taille=TU // couleur=sans
			if(isset($product['convertToProduct'])) {
				// let's record a conversion in order to export a declinaison instead of a product
				$convertTab = array(
					'imports_conversions_rules_symbol'    => pSQL($product['convertToProduct']),
					'imports_conversions_rules_original'  => $product_id,
					'imports_conversions_type'            => 'attribute_to_product'
				);

				Db::getInstance()->insert('artefact_conversions_rules', $convertTab, false, true, Db::INSERT_IGNORE);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'artefact_conversions_rules', $convertTab, 'INSERT');
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCTS / convertToProduct : INSERT FROM artefact_conversions_rules ",Db::getInstance()->getMsgError());
				}
				$this->_conversions['declinaisons'][$product_id] = $product_id;
				// let's unset Declinaison
				unset($product['ProduitDeclinaisons'], $convertTab);
			}

			if(isset($product['ProduitDeclinaisons'])) {
				$this->_importProduitDeclinaisons($product['ProduitDeclinaisons'], $product_id, $product['Soldes']);
			}

			// let's deal with rebate prices
			if($firstRun==false && count($product['Soldes'])>0 ) {
				//pp($product['Soldes']);
				$this->_importSoldeProduits($product_id, $product['Soldes']);
			}
		}

		$imagesTypes = ImageType::getImagesTypes('products');
		$product_obj = new Product($product_id);

		// back-office use on images ?
		$updateProductImages = true;
		if (Configuration::get('PS_ARF_ALLOW_PROD_UPD_IMG') == true) {
			$updateProductImages = false;
			if ($product['action'] == 'create') {
				$updateProductImages = true;
			}
		}
		if(count($product['ProduitImages'])==0) {
			$updateProductImages = false;
		}

		if ($updateProductImages == true) {

			//let's delete image only if requested... granularity describe in xml description is available in _sync array 
			if (isset($this->_sync['product']) && ($this->_sync['product'] == 'tout' || $this->_sync['product'] == 'imagesUniquement')) {
				if (!empty($product['ProduitImages'])) {
					$status = $product_obj->deleteImages();
					if (!$status) {
						if (isset($this->_logger)) {
							$this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : can't remove image(s) to product ID = ".$product_id, $status);
						}
					}
				}
			}

			$declinations_images = array();
			$cover=false;
			if(is_array($product['ProduitImages']) && count($product['ProduitImages'])>0) {
				foreach ($product['ProduitImages'] as $k => $image) {
					if ($image['nature'] == 'image') {
						$image_path = dirname(__FILE__).'/../../../../arte-fact/inputs/images/produits/'.$image['link'];
						$ext = pathinfo($image_path, PATHINFO_EXTENSION);

						if (file_exists($image_path)) {
							$imageNew = new Image();
							$imageNew->id_product = (int)($product_id);
							$imageNew->position = Image::getHighestPosition($product_obj->id) + 1;
							if (Image::getHighestPosition($product_obj->id) == 0) {
								$imageNew->cover = 1;
								$cover=true;
							}

							// A new id is generated for image when calling add()
							if ($imageNew->add()) {
								$new_image_path = _PS_PROD_IMG_DIR_.$product_id.'-'.$imageNew->id.'.'.$ext;
								if (file_exists($new_image_path)) {
									@unlink($new_image_path);
								}

								$new_path = $imageNew->getPathForCreation();
								if (copy($image_path, $new_image_path)) {
									//@unlink($image_path);
									$imagesTypes = ImageType::getImagesTypes('products');
									foreach ($imagesTypes as $k => $imageType) {
										ImageManager::resize($new_image_path, $new_path.'-'.$imageType['name'].'.jpg', (int)($imageType['width']), (int)($imageType['height']), $ext);
										if($imageType['name']=='small_default' && $cover==true) {
											ImageManager::resize($new_image_path, _PS_TMP_IMG_DIR_.'product_mini_'.(int)($product_id).'_1.jpg', (int)($imageType['width']), (int)($imageType['height']), $ext);
										}
									}
									if (!empty($image['reference'])) {
										if (is_array($image['reference'])) {
											foreach ($image['reference'] as $imageRef) {
												if (array_key_exists($imageRef, $declinations_images))  {
													if(count($declinations_images[$imageRef])>0){
														if($this->inArrayR($imageNew->id, $declinations_images[$imageRef])==false) {
															$declinations_images[$imageRef][] = $imageNew->id;
														}
													}
													else {
														$declinations_images[$imageRef][] = $imageNew->id;
													}
												}
												else {
													if(isset($declinations_images[$imageRef]) && count($declinations_images[$imageRef])>0){
														if($this->inArrayR($imageNew->id, $declinations_images[$imageRef])==false) {
															$declinations_images[$imageRef][] = $imageNew->id;
														}
													}
													else {
														$declinations_images[$imageRef][] = $imageNew->id;
													}
												}
											}
										} else {
											$declinations_images[$image['reference']][] = $imageNew->id;
										}
									}
								}
							}
							if (!$imageNew->update()) {
								$file['error'] = Tools::displayError('Error while updating status');
								continue;
							}
							// Associate image to shop from context
							$shops = Shop::getContextListShopID();
							$imageNew->associateTo($shops);
							Image::moveToNewFileSystem();

						}
					}
				}
			}
			/*
			pp($declinations_images);
			exit;
			*/
			if (!empty($declinations_images)) {
				foreach ($declinations_images as $key => $value) {

					$sql='SELECT id_product_attribute FROM '._DB_PREFIX_.'product_attribute WHERE artifact_reference = "'.pSQL($key).'"';
					$id_product_attribute = Db::getInstance()->getValue($sql);

					if (Db::getInstance()->getMsgError()) {
						if (isset($this->_logger)) {
							$this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : can't SELECT id_product_attribute FROM PRODUCT_ATTRIBUTE ", Db::getInstance()->getMsgError());
						}
					}

					if (empty($id_product_attribute)) {
						if (isset($this->_logger)) {
							$this->_logger->logInfo('CANT FIND  id_product_attribute TO  INSERT FROM PRODUCT_ATTRIBUTE_IMAGE ', $key);
						}
						continue;
					}

					Db::getInstance()->Execute(
						'DELETE FROM '._DB_PREFIX_.'product_attribute_image WHERE id_product_attribute = '.(int)$id_product_attribute
					);
					if (Db::getInstance()->getMsgError()) {
						if (isset($this->_logger)) {
							$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : DELETE FROM PRODUCT_ATTRIBUTE_IMAGE ', Db::getInstance()->getMsgError());
						}
					}
					if (is_array($value)) {
						foreach ($value as $val) {
							$addTab = array(
								'id_product_attribute' => (int)$id_product_attribute,
								'id_image' => (int)$val,
							);
							Db::getInstance()->insert('product_attribute_image', $addTab);
							//Db::getInstance()->autoExecute(_DB_PREFIX_.'product_attribute_image', $addTab, 'INSERT');
							if (Db::getInstance()->getMsgError()) {
								if (isset($this->_logger)) {
									$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT FROM PRODUCT_ATTRIBUTE_IMAGE ', Db::getInstance()->getMsgError());
								}
							}
							else {
								if (isset($this->_logger)) {
									$this->_logger->logInfo('CLASS PRODUCT : INSERT FROM PRODUCT_ATTRIBUTE_IMAGE '. $id_product_attribute . ' - '.  $val);
								}
							}
						}
					} else {
						$addTab = array(
							'id_product_attribute' => (int)$id_product_attribute,
							'id_image' => (int)$value,
						);
						Db::getInstance()->insert('product_attribute_image', $addTab);
						//Db::getInstance()->autoExecute(_DB_PREFIX_.'product_attribute_image', $addTab, 'INSERT');
						if (Db::getInstance()->getMsgError()) {
							if (isset($this->_logger)) {
								$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT FROM PRODUCT_ATTRIBUTE_IMAGE ', Db::getInstance()->getMsgError());
							}
						}
						else {
							if (isset($this->_logger)) {
								$this->_logger->logInfo('CLASS PRODUCT : INSERT FROM PRODUCT_ATTRIBUTE_IMAGE '. $id_product_attribute . ' - '.  $value);
							}
						}
					}
				}
			}
		} // end update product images

		//gestion des pack :BD
		if (isset($product['ProduitComposition'])) {
			$this->_produitComposite($product['ProduitComposition'], $product_id);
		}

		//fin gestion pack :BD

		if (Configuration::get('PS_PRODUCT_ACTIVATION') == 1) {
			$publication = new ArtifactPublication();
			if ($publication->isActiveProduct($product_id) == true) {
				$publication->setActiveProduct($product_id);
			}
		}

		// let's index the product
		if (Configuration::get('PS_SEARCH_INDEXATION')) {
			$newSearch = new Search();
			$newSearch::indexation(false, $product_id);
			unset($newSearch);
		}

		// let's called hooks
		$newHook = new Hook();
		$product= new Product($product_id);
		$newHook::exec('actionProductSave', array('id_product' => $product_id, 'product' => $product));
		$newHook::exec('actionProductUpdate', array('id_product' => $product_id, 'product' => $product));
		unset($newHook);
	}

	private function _importProduitDeclinaisons($declinaisons_array, $product_id, $soldes = null)
	{
		if (empty($declinaisons_array) || empty($product_id)) {
			return false;
		}

		$firstRun= (Configuration::get('PS_ARF_FIRST_RUN')==true) ? true : false;

		if (!is_null($soldes) && is_array($soldes)) {
			$targetDeclinaisons = false;
			foreach ($soldes as $solde) {
				if (array_key_exists('attribute', $solde)) {
					$targetDeclinaisons = true;
				}
			}
		}

		if (empty($declinaisons_array[0]['deltaStock'])) {
			Db::getInstance()->Execute('UPDATE '._DB_PREFIX_.'product_attribute SET supplier_reference = "should_be_removed" WHERE id_product = '.(int)$product_id);
			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : UPDATE FROM PRODUCT_ATTRIBUTE in EMPTY deltaStock ', Db::getInstance()->getMsgError());
				}
			}
		}

		$attributeExists=false;
		if($firstRun==true) {
			foreach ($declinaisons_array as $key => $declinaisons) {
				if(isset($declinaisons['DeclinaisonReferenceSite']) && !empty($declinaisons['DeclinaisonReferenceSite'])) {
					$attributeExists=true;
					$sql = 'SELECT id_attribute FROM '._DB_PREFIX_.'product_attribute_combination WHERE id_product_attribute="'.pSQL($declinaisons['DeclinaisonReferenceSite']).'"';
					$arrayAttribut=array();
					if ($results = Db::getInstance()->ExecuteS($sql)){
						foreach ($results as $row) {
							$id_attribute_group = 
							$tmpArray['id_attribute']=$row['id_attribute'];
							$tmpArray['id_attribute_group']=Db::getInstance()->getValue('SELECT id_attribute_group FROM '._DB_PREFIX_.'attribute WHERE id_attribute = "'. $row['id_attribute'] .'"');
							$tmpArray['attribute_group_name']=Db::getInstance()->getValue('SELECT name FROM '._DB_PREFIX_.'attribute_group_lang WHERE id_lang="1" AND id_attribute_group = "'. $tmpArray['id_attribute_group'] .'"');
					
							array_push($arrayAttribut, $tmpArray);
						}
					}
				}
			}
		}

		/// TODO :: Removing attributes trash
		foreach ($declinaisons_array as $key => $declinaisons) {

			if($firstRun==true) {
				$id_product_attribute  = $declinaisons['DeclinaisonReferenceSite'];
			}
			else {
				//$sql='SELECT id_product_attribute FROM '._DB_PREFIX_.'product_attribute WHERE artifact_reference = "'.pSQL(Tools::replaceAccentedChars((string)$declinaisons['DeclinaisonArtifactReference'])).'"';
				$sql='SELECT id_product_attribute FROM '._DB_PREFIX_.'product_attribute WHERE artifact_reference = "'.pSQL(Tools::replaceAccentedChars((string)$declinaisons['DeclinaisonArtifactReference'])).'"';
				$id_product_attribute = Db::getInstance()->getValue($sql);
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : SELECT FROM PRODUCT_ATTRIBUTE ",Db::getInstance()->getMsgError());
				}
				unset($sql);
			}

			if (!empty($declinaisons['deltaStock']) && !empty($id_product_attribute)) {
				$addTab = array(
					'quantity' => $declinaisons['DeclinaisonStock'],
				);
				Db::getInstance()->update('product_attribute', $addTab, 'id_product_attribute = '.$id_product_attribute);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'product_attribute', $addTab, 'UPDATE', 'id_product_attribute = '.$id_product_attribute);
				if (Db::getInstance()->getMsgError()) {
					if (isset($this->_logger)) {
						$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : UPDATE FROM PRODUCT_ATTRIBUTE WHERE NOT EMPTY deltaStock ', Db::getInstance()->getMsgError());
					}
				}
			} elseif (!empty($id_product_attribute)) {
				$addTab = array(
					'quantity' => (int)$declinaisons['DeclinaisonStock'],
					'price' => (float)$declinaisons['DeclinaisonPrixHT'] - $declinaisons['ProduitPrixHT'],
					/*
					'supplier_reference' => '',
					*/
				);

				if(isset($declinaisons['DeclinaisonPrixAchatHT'])) {
					$addTab['wholesale_price'] = (float)($declinaisons['DeclinaisonPrixAchatHT']);
				}

				if($firstRun==true) {
					// let's update reference to Artifact for further use
					//$addTab['artifact_reference']=pSQL(Tools::replaceAccentedChars((string)$declinaisons['DeclinaisonReference']));
					$addTab['artifact_reference']=pSQL((string)$declinaisons['DeclinaisonReference']);
					// some errors on prices at import so let's remove price
					unset($addTab['wholesale_price']);
					unset($addTab['price']);

					$inserTab = array(
						'imports_conversions_rules_symbol'    =>  pSQL($declinaisons['DeclinaisonReference']),
						'imports_conversions_rules_original'  => $id_product_attribute,
						'imports_conversions_type'            => 'declinaison'
					);

					Db::getInstance()->insert('artefact_conversions_rules', $inserTab);
					//Db::getInstance()->autoExecute(_DB_PREFIX_.'artefact_conversions_rules', $inserTab, 'INSERT');
					if (Db::getInstance()->getMsgError()) {
						if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS DECLINAISON : INSERT FROM artefact_conversions_rules ",Db::getInstance()->getMsgError());
					}
					$this->_conversions['declinaison'][$declinaisons['DeclinaisonReference']] = $id_product_attribute;
					unset($inserTab);

				}
				else {
					// let's check we already have a conversion
					
					if(isset($this->_conversions['declinaison'][$declinaisons['DeclinaisonReference']]) 
						&& $this->_conversions['declinaison'][$declinaisons['DeclinaisonReference']] != $id_product_attribute) {
						// let's update imports_conversions_rules_original i
						$sql="UPDATE "._DB_PREFIX_."artefact_conversions_rules SET imports_conversions_rules_original=". $id_product_attribute ." WHERE imports_conversions_rules_symbol='". pSQL($declinaisons['DeclinaisonReference']). "' AND imports_conversions_type='declinaison'";
						Db::getInstance()->ExecuteS($sql);
						if (Db::getInstance()->getMsgError()) {
							if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS DECLINAISON : UPDATE FROM artefact_conversions_rules failed ",Db::getInstance()->getMsgError());
						}
						else {
							if(isset($this->_logger))  $this->_logger->logInfo("SQL CLASS DECLINAISON : Wrong conversion updated ",$declinaisons['DeclinaisonReference']);
						}
						$this->_conversions['declinaison'][$declinaisons['DeclinaisonReference']] = $id_product_attribute;
						$addTab['artifact_reference']=$declinaisons['DeclinaisonReference'];
					}

					if(!isset($this->_conversions['declinaison'][$declinaisons['DeclinaisonReference']])) {

						$inserTab = array(
							'imports_conversions_rules_symbol'    =>  pSQL($declinaisons['DeclinaisonReference']),
							'imports_conversions_rules_original'  => $id_product_attribute,
							'imports_conversions_type'            => 'declinaison'
						);
						Db::getInstance()->insert('artefact_conversions_rules', $inserTab);
						if (Db::getInstance()->getMsgError()) {
							if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS DECLINAISON : INSERT FROM artefact_conversions_rules ",Db::getInstance()->getMsgError());
						}
						else {
							if(isset($this->_logger))  $this->_logger->logInfo("SQL CLASS DECLINAISON : Missing conversion inserted",$declinaisons['DeclinaisonReference']);
						}
						$this->_conversions['declinaison'][$declinaisons['DeclinaisonReference']] = $id_product_attribute;
						unset($inserTab);
						$addTab['artifact_reference']=$declinaisons['DeclinaisonReference'];
					}

				}
/*
				pp($addTab);
				pp($id_product_attribute);
				exit;
*/
				Db::getInstance()->update('product_attribute', $addTab, 'id_product_attribute = '.$id_product_attribute);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'product_attribute', $addTab, 'UPDATE', 'id_product_attribute = '.$id_product_attribute);
				if (Db::getInstance()->getMsgError()) {
					if (isset($this->_logger)) {
						$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : UPDATE FROM PRODUCT_ATTRIBUTE WHERE EMPTY deltaStock ', Db::getInstance()->getMsgError());
					}
				}
				else {
					$this->_logger->logInfo('SQL CLASS PRODUCT :  UPDATE FROM PRODUCT_ATTRIBUTE WHERE EMPTY deltaStock ', $product_id .' - ' . $id_product_attribute);
				}

				$addTab['id_shop'] = (int)Context::getContext()->shop->id;
				$addTab['id_product'] = $product_id ;
				unset($addTab['quantity']);
				unset($addTab['supplier_reference']);
				unset($addTab['artifact_reference']);
				unset($addTab['reference']);
				/* let's check if we already inserted a product_attribute in order to set default... */
				$sql = 'SELECT artifact_reference FROM '._DB_PREFIX_.'product_attribute WHERE id_product ='.(int)$product_id.'  AND default_on=1';
				if (count($results = Db::getInstance()->ExecuteS($sql))==0) {
					$addTab['default_on'] = '1';
				}

				Db::getInstance()->update('product_attribute_shop', $addTab, 'id_product_attribute = '.$id_product_attribute);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'product_attribute_shop', $addTab, 'UPDATE', 'id_product_attribute = '.$id_product_attribute);
				if (Db::getInstance()->getMsgError()) {
					if (isset($this->_logger)) {
						$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : UPDATE FROM PRODUCT_ATTRIBUTE_SHOP WHERE EMPTY deltaStock ', Db::getInstance()->getMsgError());
					}
				}

				// don't touch anything if product exist before artifact sync...
				/*
				if (isset($declinaisons['DeclinaisonColor']) && !empty($declinaisons['DeclinaisonColor'])) {
					$this->_attributDeclinaisonUpdateColor($declinaisons['Attributs'], $id_product_attribute, $declinaisons['DeclinaisonColor']);
				}
				*/

				// check if we have a rebate
				// time to check if we've got a match in this delinaison..
				if ($targetDeclinaisons == true) {
					$declinaisonsSoldes = array();
					foreach ($soldes as $solde) {
						if (array_key_exists('attribute', $solde)) {
							asort($solde['attribute']['@attributes']);

							if ($solde['attribute']['@attributes'] == $declinaisons['Attributs']) {
								array_push($declinaisonsSoldes, $solde);
							}
						}
					}

					if (is_array($declinaisonsSoldes) && count($declinaisonsSoldes) > 0) {
						if($firstRun==false) {
							$this->_importSoldeDeclinaisons($product_id, $id_product_attribute, $declinaisonsSoldes);
						}
					}
				}
			}
			else {
				$addTab = array(
					'id_product' => (int)$product_id,
					'artifact_reference' => pSQL((string)$declinaisons['DeclinaisonReference']),
					'reference' => pSQL($declinaisons['DeclinaisonReference']),
					'ean13' => pSQL($declinaisons['DeclinaisonEAN']),
					'quantity' => pSQL($declinaisons['DeclinaisonStock']),
					'price' => pSQL($declinaisons['DeclinaisonPrixHT'] - $declinaisons['ProduitPrixHT']),
					'unit_price_impact' => 0,
				);

				/* let's check if we already inserted a product_attribute in order to set default... */
				$sql = 'SELECT artifact_reference FROM '._DB_PREFIX_.'product_attribute WHERE id_product ='.(int)$product_id.'  AND default_on=1';
				if (count($results = Db::getInstance()->ExecuteS($sql))==0) {
					$addTab['default_on'] = '1';
				}
				Db::getInstance()->insert('product_attribute', $addTab);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'product_attribute', $addTab, 'INSERT');
				if (Db::getInstance()->getMsgError()) {
					if (isset($this->_logger)) {
						$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT FROM PRODUCT_ATTRIBUTE ', Db::getInstance()->getMsgError());
					}
				}
				else {
					$this->_logger->logInfo('SQL CLASS PRODUCT :  INSERT FROM PRODUCT_ATTRIBUTE ', $product_id .' - ' . Db::getInstance()->Insert_ID());
				}

				$id_product_attribute = Db::getInstance()->Insert_ID();
				$addTab['id_product_attribute'] = (int)$id_product_attribute;
				$addTab['available_date'] = '0000-00-00';
				$addTab['id_shop'] = (int)Context::getContext()->shop->id;
				//unset($addTab['id_product']);
				unset($addTab['reference']);
				unset($addTab['supplier_reference']);
				unset($addTab['artifact_reference']);
				unset($addTab['ean13']);
				unset($addTab['quantity']);

				Db::getInstance()->insert('product_attribute_shop', $addTab);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'product_attribute_shop', $addTab, 'INSERT');
				if (Db::getInstance()->getMsgError()) {
					if (isset($this->_logger)) {
						$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT FROM PRODUCT_ATTRIBUTE_SHOP ', Db::getInstance()->getMsgError());
					}
				}

				$inserTab = array(
					'imports_conversions_rules_symbol'    =>  pSQL($declinaisons['DeclinaisonReference']),
					'imports_conversions_rules_original'  => $id_product_attribute,
					'imports_conversions_type'            => 'declinaison'
				);

				Db::getInstance()->insert('artefact_conversions_rules', $inserTab);
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS DECLINAISON : INSERT FROM artefact_conversions_rules ",Db::getInstance()->getMsgError());
				}
				$this->_conversions['declinaison'][$declinaisons['DeclinaisonReference']] = $id_product_attribute;
				unset($inserTab);


				// check if we have a rebate
				// time to check if we've got a match in this delinaison..
				if ($targetDeclinaisons == true) {
					$tmpAttributes=$declinaisons['Attributs'];
					$declinaisonsSoldes=array();
					foreach($soldes as $solde) {
						$tmpSolde=array();
						asort($solde['attribute']['@attributes']);
						asort($tmpAttributes);

						while(list($key, $value) = each($solde['attribute']['@attributes'])) {
							$tmpSolde[str_replace('_', ' ', $key)]=$value;
						}
						if(array_key_exists('attribute', $solde)) {
							if($tmpSolde==$tmpAttributes) {
								array_push($declinaisonsSoldes, $solde);
							}
						}
					}
					if(is_array($declinaisonsSoldes) && count($declinaisonsSoldes)>0) {
						$this->_importSoldeDeclinaisons($product_id, $id_product_attribute, $declinaisonsSoldes);
					}
					unset($tmpAttributes, $tmpSolde);
				}

				if($firstRun===true && empty($declinaisons['DeclinaisonReferenceSite']) ) {
					if($attributeExists===true) {
						$this->_attributDeclinaisonExist($declinaisons['Attributs'], $id_product_attribute,$declinaisons['DeclinaisonColor'], $arrayAttribut, $product_id);
					}
					else {
						$this->_attributDeclinaison($declinaisons['Attributs'], $id_product_attribute,$declinaisons['DeclinaisonColor']);
					}
				}
				else {
					if(isset($declinaisons['Attributs'])) {
						$this->_attributDeclinaison($declinaisons['Attributs'], $id_product_attribute,$declinaisons['DeclinaisonColor']);
					}
				}
			}

			$id = Db::getInstance()->getValue('SELECT id_shop FROM '._DB_PREFIX_.'stock_available WHERE id_product_attribute = "'.$id_product_attribute.'"');
			$out_of_stock = Db::getInstance()->getValue('SELECT out_of_stock FROM '._DB_PREFIX_.'stock_available WHERE id_product = "'.(int)$product_id.'"');

			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : SELECT FROM STOCK_AVAILABLE ', Db::getInstance()->getMsgError());
				}
			} elseif (empty($id)) {
				$addTab1 = array(
					'id_product' => (int)$product_id,
					'id_product_attribute' => (int)$id_product_attribute,
					'id_shop' => (int)Context::getContext()->shop->id,
					'quantity' => pSQL($declinaisons['DeclinaisonStock']),
					'out_of_stock' => $out_of_stock
				);
				Db::getInstance()->insert('stock_available', $addTab1);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'stock_available', $addTab1, 'INSERT');
				if (Db::getInstance()->getMsgError()) {
					if (isset($this->_logger)) {
						$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT FROM STOCK_AVAILABLE ', Db::getInstance()->getMsgError());
					}
				}
			} else {
				Db::getInstance()->Execute(
					'UPDATE `'._DB_PREFIX_.'stock_available`
					SET `quantity`='.pSQL($declinaisons['DeclinaisonStock']).'
					WHERE `id_product_attribute`='.(int)$id_product_attribute.''
				);
				if (Db::getInstance()->getMsgError()) {
					if (isset($this->_logger)) {
						$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : UPDATE FROM STOCK_AVAILABLE ', Db::getInstance()->getMsgError());
					}
				}
			}

			$newHook = new Hook();
			$newHook::exec('actionProductAttributeUpdate', array('id_product_attribute' => (int)$id_product_attribute));
			unset($newHook);
		}

		/*
		if (empty($declinaisons_array[0]['deltaStock'])) {
			Db::getInstance()->Execute(
				'DELETE FROM `'._DB_PREFIX_.'product_attribute_combination`
				WHERE `id_product_attribute` IN (
				SELECT `id_product_attribute` FROM `'._DB_PREFIX_.'product_attribute`
				WHERE supplier_reference = "should_be_removed" AND `id_product` = '.(int)$product_id.')'
			);
			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : DELETE FROM PRODUCT_ATTRIBUTE_COMPINATION ', Db::getInstance()->getMsgError());
				}
			}
			Db::getInstance()->Execute(
				'DELETE FROM '._DB_PREFIX_.'product_attribute
				WHERE supplier_reference = "should_be_removed" AND id_product = '.(int)$product_id
			);
			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : DELETE FROM PRODUCT_ATTRIBUTE ', Db::getInstance()->getMsgError());
				}
			}
		}
		*/

	}

	private function _getProductId($artifactReference)
	{
		$product_id = Db::getInstance()->getValue('SELECT id_product FROM '._DB_PREFIX_.'product WHERE artifact_reference = "'.pSQL($artifactReference).'"');
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : SELECT FROM PRODUCT ', Db::getInstance()->getMsgError());
			}
		}

		return $product_id;
	}

	private function _deleteProduct($reference)
	{
		$product_id = $this->_getProductId($reference);
		if (empty($product_id)) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : PRODUCT WITH REFERENCE = '.$reference.' NOT EXIST');
			}
			return false;
		}
		$product_obj = new Product($product_id);
		$product_obj->delete();
		if (isset($this->_logger)) {
			$this->_logger->logInfo('CLASS PRODUCT : PRODUCT WITH REFERENCE = '.$reference.' DELETED');
		}

	}

	public function importDailyStock($obj)
	{
		if (empty($obj)) {
			return false;
		}
		$log = new ArtifactImport();
		if (isset($log->logger)) {
			$this->_logger = $log->logger;
		}

		$this->_getConversions();
		$this->_processingDailyXML($obj);
	}

	private function _processingDailyXML($obj)
	{
		$products = array();

		foreach ($obj as $values) {
			$data = array();
			$data['action'] = (string)$values->attributes()->action;
			$data['ProduitReference'] = (string)$values->ProduitReference;
			$data['ProduitArtifactReference'] = (string)$values->ProduitReference;
			$data['ProduitStock'] = (int)$values->ProduitStock;
			$data['ProduitDeclinaisons'] = array();

			foreach ($values->ProduitDeclinaisons->ProduitDeclinaison as $produitDeclinaison) {
				$declinaisons = array();

				foreach ($produitDeclinaison->Attributs->Attribut as $attributs) {
					$attributs_id = mb_strtolower((string)$attributs->attributes()->id);
					if($attributs_id=='taille' &&  mb_strtolower($attributs)=='tu') {
						continue;
					}
					elseif($attributs_id=='taille' &&  mb_strtolower($attributs)=='sans') {
						continue;
					}
					elseif($attributs_id=='couleur' &&  mb_strtolower($attributs)=='tu') {
						continue;
					}
					elseif($attributs_id=='couleur' &&  mb_strtolower($attributs)=='sans') {
						continue;
					}
					elseif($attributs_id=='couleur' &&  mb_strtolower($attributs)=='none') {
						continue;
					}
					elseif($attributs_id=='couleur' &&  mb_strtolower($attributs)=='unique') {
						continue;
					}
					elseif($attributs_id=='couleur' &&  mb_strtolower($attributs)=='0') {
						continue;
					}
					elseif($attributs_id=='couleurs' &&  mb_strtolower($attributs)=='sans') {
						continue;
					}
					else {
						$declinaisons['Attributs'][$attributs_id] = (string)$attributs;
					}
				}
				if(isset($declinaisons['Attributs'])){
					$declinaisons['deltaStock'] = true;
					$declinaisons['DeclinaisonReference'] = (string)$produitDeclinaison->DeclinaisonReferenceFournisseur;
					$declinaisons['DeclinaisonArtifactReference'] = (string)$produitDeclinaison->DeclinaisonReference;
					$declinaisons['DeclinaisonEAN'] = (string)$produitDeclinaison->DeclinaisonEAN;
					$declinaisons['DeclinaisonStock'] = (string)$produitDeclinaison->DeclinaisonStock;
					$declinaisons['DeclinaisonPrixHT'] = (string)$produitDeclinaison->DeclinaisonPrixHT;
					$data['ProduitDeclinaisons'][] = $declinaisons;
				}
			}
			
			$products[] = $data;
		}

		foreach ($products as $product) {
			if ($product['action'] == 'delete') {
				$this->_deleteProduct($product['ProduitReference']);
				continue;
			}
			$this->_importProductDailyUpdate($product);
		}

	}

	private function _importProductDailyUpdate($product)
	{
		$product_id = $this->_getProductId($product['ProduitArtifactReference']);

		if (empty($product_id)) {
			return false;
		}

		$addTab = array(
			'quantity' => pSQL($product['ProduitStock']),
		);
		Db::getInstance()->update('product', $addTab, 'id_product = '.(int)$product_id);
		//Db::getInstance()->autoExecute(_DB_PREFIX_.'product', $addTab, 'UPDATE', 'id_product = '.(int)$product_id);
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : UPDATE FROM PRODUCT ', Db::getInstance()->getMsgError());
			}
		}
		else {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('UPDATE STOCK PRODUCT REF : ' . $product['ProduitReference'] .' -  ID ; '. $product_id );
			}
		}
		Db::getInstance()->Execute(
			'UPDATE `'._DB_PREFIX_.'stock_available`
			SET `quantity`='.pSQL($product['ProduitStock']).'
			WHERE `id_product_attribute`=0 AND `id_product` = '.(int)$product_id.''
		);
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : UPDATE FROM STOCK_AVAILABLE ', Db::getInstance()->getMsgError());
			}
		}

		if (Configuration::get('PS_PRODUCT_ACTIVATION') == 1) {
			$publication = new ArtifactPublication();
			if ($publication->isActiveProduct($product_id) == true) {
				$publication->setActiveProduct($product_id);
			}
			else {
				$publication->setNoActiveProduct($product_id);
			}
		}

		$this->_importProduitDeclinaisons($product['ProduitDeclinaisons'], $product_id);
	}

	private function _attributeValueInsert ($attribute_group_id, $attributs_group, $attributs_value, $color) {

		// let's prepare color insertion....
		if($attributs_group=='couleur' && isset($color)) {
			$tmp_color=(!empty($color)) ? $color : '';
		}
		else {
			$tmp_color = '';
		}

		$addTab = array(
			'id_attribute'       => 0,
			'id_attribute_group' => $attribute_group_id,
			'color'              => $tmp_color,
			'position'           => Attribute::getHigherPosition($attribute_group_id)+1
		);

		Db::getInstance()->insert('attribute', $addTab);
		//Db::getInstance()->autoExecute(_DB_PREFIX_.'attribute', $addTab, 'INSERT');
		if (Db::getInstance()->getMsgError()) {
			if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : INSERT FROM ATTRIBUTES ",Db::getInstance()->getMsgError());
		}
		$id_attribute = Db::getInstance()->Insert_ID();

		// let's insert attribute in shop
		$addTab = array(
			'id_attribute' =>$id_attribute,
			'id_shop' => (int)Context::getContext()->shop->id
		);

		Db::getInstance()->insert('attribute_shop', $addTab);
		//Db::getInstance()->autoExecute(_DB_PREFIX_.'attribute_shop', $addTab, 'INSERT');
		if (Db::getInstance()->getMsgError()) {
			if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : INSERT INTO ATTRIBUTE_SHOP ",Db::getInstance()->getMsgError());
		}

		Hook::exec('actionAttributeSave', array('id_attribute' => $id_attribute));

		$id_lang_fr=Language::getIdByIso('fr');

		foreach ($this->_languages as $code => $language_id) {
			if($id_lang_fr==$language_id) {
				Db::getInstance()->Execute(
					'INSERT INTO '._DB_PREFIX_.'attribute_lang (`id_attribute`, `id_lang`, `name`)
					 VALUES (
						"'.pSQL($id_attribute).'",
						"'.pSQL($language_id).'",
						"'.addslashes($attributs_value).'"
					 )
					 ON DUPLICATE KEY UPDATE
					 name        = "'.addslashes($attributs_value).'"'
				);
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : IN INSERT INTO ATTRIBUTE_LANG ",Db::getInstance()->getMsgError());
				}
			}
		}
		return $id_attribute;

	}

	private function _attributesLists ($id_attribute_group) {

		$id_lang=Language::getIdByIso('fr');

		$sql='SELECT *
			FROM `'._DB_PREFIX_.'attribute` a
			'.Shop::addSqlAssociation('attribute', 'a').'
			LEFT JOIN `'._DB_PREFIX_.'attribute_lang` al
				ON (a.`id_attribute` = al.`id_attribute` AND al.`id_lang` = '.(int)$id_lang.')
			WHERE a.`id_attribute_group` = '.(int)$id_attribute_group.'
			ORDER BY `position` ASC';

		return Db::getInstance()->executeS($sql);

	}

	private function _attributDeclinaison($declinaisons, $id_product_attribute, $color){

		foreach ($declinaisons as $attributs_group => $attributs_value) {

			$AttributeGroup=new AttributeGroup();
			$attributeGroups=$AttributeGroup->getAttributesGroups(Language::getIdByIso('fr'));

			$attribute_group_id=0;

			if(is_array($attributeGroups) && count($attributeGroups)>0) {
				foreach($attributeGroups as $attribute_Group) {
					if(strtolower($attribute_Group['name'])==strtolower($attributs_group)) {
						$attribute_group_id=$attribute_Group['id_attribute_group'];
						break;
					}
				}
			}

			if($attribute_group_id>0) {

				// now let's search for an existing value...
				$attributeValues=$AttributeGroup->getAttributes(Language::getIdByIso('fr'), $attribute_group_id);
				//$attributeValues=$this->_attributesLists($attribute_group_id);
				$isAttribute=Attribute::isAttribute($attribute_group_id, $attributs_value, Language::getIdByIso('fr'));

				/*
				pp('exists : ' . $attribute_group_id);
				pp('attribute value exist : ' . $isAttribute);
				pp($attributeValues);
				pp($attributs_value);
				*/
				if(is_array($attributeValues) && count($attributeValues)>0) {

					$id_attribute=0;

					foreach($attributeValues as $attribute_Value) {
						if(strtolower($attribute_Value['name'])==strtolower($attributs_value)) {
							$id_attribute=$attribute_Value['id_attribute'];
							break;
						}
					}

					if($id_attribute==0) {
						$id_attribute=$this->_attributeValueInsert($attribute_group_id, $attributs_group, $attributs_value, $color);
					}

				}
				else {
					$id_attribute=$this->_attributeValueInsert($attribute_group_id, $attributs_group, $attributs_value, $color);
				}
			}
			else {

				// let's add a new attribute_group
				$addTab = array(
					'id_attribute_group' => 0,
					'position' => $AttributeGroup->getHigherPosition() + 1,
					'is_color_group'     =>($attributs_group=='couleur')? 1 : 0,
					'group_type'         =>($attributs_group=='couleur')? 'color' : 'select'
				);
				Db::getInstance()->insert('attribute_group', $addTab);

				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : INSERT FROM ATTRIBUTE_GROUP ",Db::getInstance()->getMsgError());
				}

				$attribute_group_id = Db::getInstance()->Insert_ID();

				//pp('create : ' . $attribute_group_id);

				//insertion dans la table attribute_group_shop
				$addTab = array(
					'id_attribute_group' =>$attribute_group_id,
					'id_shop' => (int)Context::getContext()->shop->id
				);

				Db::getInstance()->insert('attribute_group_shop', $addTab);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'attribute_group_shop', $addTab, 'INSERT');

				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : INSERT FROM ATTRIBUTES_GROUP_SHOP ",Db::getInstance()->getMsgError());
				}

				$id_lang_fr=Language::getIdByIso('fr');

				foreach ($this->_languages as $code => $language_id) {
					if($id_lang_fr==$language_id) {
						Db::getInstance()->Execute(
							'INSERT INTO '._DB_PREFIX_.'attribute_group_lang (`id_attribute_group`, `id_lang`, `name`, `public_name`)
							 VALUES (
								"'.pSQL($attribute_group_id).'",
								"'.pSQL($language_id).'",
								"'.addslashes($attributs_group).'",
								"'.addslashes($attributs_group).'"
							)
							 ON DUPLICATE KEY UPDATE
							 name        = "'.addslashes($attributs_group).'",
							 public_name = "'.addslashes($attributs_group).'"'
						);
						if (Db::getInstance()->getMsgError()) {
							if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : INSERT INTO FROM ATTRIBUTE_GROUP_LANG ",Db::getInstance()->getMsgError());
						}
					}
				}

				Hook::exec('actionAttributeGroupSave', array('id_attribute_group' => $attribute_group_id));

				// let's add a new attribute
				$id_attribute=$this->_attributeValueInsert($attribute_group_id, $attributs_group, $attributs_value, $color);

			}

			$addTab = array(
				'id_attribute'         => $id_attribute,
				'id_product_attribute' => $id_product_attribute
			);

			Db::getInstance()->insert('product_attribute_combination', $addTab);
			//Db::getInstance()->autoExecute(_DB_PREFIX_.'product_attribute_combination', $addTab, 'INSERT');
			if (Db::getInstance()->getMsgError()) {
				if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : IN INSERT PRODUCT_ATTRIBUTE ",Db::getInstance()->getMsgError());
			}

			unset($AttributeGroup, $attributeGroups, $attributeValues);

		}

	}

	private function _attributDeclinaisonExist($declinaisons, $id_product_attribute, $color, $scheme, $id_product) {

		// a new variation has been added to an existing product...
		// we've got a scheme, let's used it to have a correct product...

		if(is_array($scheme) && count($scheme)>0) {

			foreach($scheme as $attributeGroups) {

				foreach ($declinaisons as $attributs_group => $attributs_value) {

					if(strtolower($attributs_group)==strtolower($attributeGroups['attribute_group_name'])) {

						$attribute_group_id=$attributeGroups['id_attribute_group'];

						$attributeRuleSymbol=Tools::link_rewrite(Tools::replaceAccentedChars(mb_strtolower($attributs_group.'-'.$id_product.'-'.$attributs_value)));

						//ajout de l'attribut
						// let's prepare color insertion....
						if($attributs_group=='couleur' && isset($color)) {
							$tmp_color=(!empty($color)) ? $color : '';
						}
						else {
							$tmp_color = '';
						}
						$addTab = array(
							'id_attribute'       => 0,
							'id_attribute_group' => $attribute_group_id,
							'color'              => $tmp_color,
							'position'           => Attribute::getHigherPosition($attribute_group_id)+1
						);

						Db::getInstance()->insert('attribute', $addTab);
						//Db::getInstance()->autoExecute(_DB_PREFIX_.'attribute', $addTab, 'INSERT');
						if (Db::getInstance()->getMsgError()) {
							if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : INSERT FROM ATTRIBUTES ",Db::getInstance()->getMsgError());
						}
						$id_attribute = Db::getInstance()->Insert_ID();
						$addTab = array(
							'imports_conversions_rules_symbol'    => addslashes($attributeRuleSymbol),
							'imports_conversions_rules_original'  => $id_attribute,
							'imports_conversions_type'            => 'attribute'
						);
						Db::getInstance()->insert('artefact_conversions_rules', $addTab);
						//Db::getInstance()->autoExecute(_DB_PREFIX_.'artefact_conversions_rules', $addTab, 'INSERT');
						if (Db::getInstance()->getMsgError()) {
							if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : INSERT FROM artefact_conversions_rules FOR NEW ATTRIBUTE ",Db::getInstance()->getMsgError());
						}
						$this->_conversions['attribute'][$attributeRuleSymbol] = $id_attribute;

						//insertion dans la table attribut_shop
						$addTab = array(
							'id_attribute' =>$id_attribute,
							'id_shop' => (int)Context::getContext()->shop->id
						);
						Db::getInstance()->insert('attribute_shop', $addTab);
						//Db::getInstance()->autoExecute(_DB_PREFIX_.'attribute_shop', $addTab, 'INSERT');
						if (Db::getInstance()->getMsgError()) {
							if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : INSERT FROM ATTRIBUTE_SHOP FOR NEW ATTRIBUTE ",Db::getInstance()->getMsgError());
						}

						$id_lang_fr=Language::getIdByIso('fr');

						foreach ($this->_languages as $code => $language_id) {
							if($id_lang_fr==$language_id) {
								Db::getInstance()->Execute(
									'INSERT INTO '._DB_PREFIX_.'attribute_lang (`id_attribute`, `id_lang`, `name`)
									 VALUES (
										"'.pSQL($id_attribute).'",
										"'.pSQL($language_id).'",
										"'.addslashes($attributs_value).'"
									 )
									 ON DUPLICATE KEY UPDATE
									 name        = "'.addslashes($attributs_value).'"'
								);
								if (Db::getInstance()->getMsgError()) {
									if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : IN INSERT INTO ATTRIBUTE_LANG ",Db::getInstance()->getMsgError());
								}
							}
						}

						$addTab = array(
							'id_attribute'         => $id_attribute,
							'id_product_attribute' => $id_product_attribute
						);

						Db::getInstance()->insert('product_attribute_combination', $addTab);
						//Db::getInstance()->autoExecute(_DB_PREFIX_.'product_attribute_combination', $addTab, 'INSERT');
						if (Db::getInstance()->getMsgError()) {
							if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : IN INSERT PRODUCT_ATTRIBUTE ",Db::getInstance()->getMsgError());
						}
					}
				}
			}
		}
	}

	private function _attributDeclinaisonUpdateColor($declinaisons, $id_product_attribute, $color)
	{
		$sql = 'SELECT id_product FROM `'._DB_PREFIX_.'product_attribute`
				WHERE id_product_attribute="'.(int)$id_product_attribute.'"';

		$id_product = Db::getInstance()->getValue($sql);
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : SELECT', Db::getInstance()->getMsgError());
			}
		}
		foreach ($declinaisons as $attributs_group => $attributs_value) {

			$attributeRuleSymbol=Tools::link_rewrite(Tools::replaceAccentedChars(mb_strtolower($attributs_group.'-'.$id_product.'-'.$attributs_value)));
			$id_attribute = !empty($this->_conversions['attribute'][$attributeRuleSymbol])
								 ? $this->_conversions['attribute'][$attributeRuleSymbol]
								 : 0;

			if (!empty($id_attribute) &&  $attributs_group=='couleur') {
				$addTab = array(
					'color' => $color,
				);
				Db::getInstance()->update('attribute', $addTab, "id_attribute = ".$id_attribute);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'attribute', $addTab, 'UPDATE', "id_attribute = ".$id_attribute);
				if (Db::getInstance()->getMsgError()) {
					if(isset($this->_logger))  $this->_logger->logInfo("SQL ERROR  CLASS PRODUCT : IN UPDATE PRODUCT_ATTRIBUTE COLOR ",Db::getInstance()->getMsgError());
				}
			}
		}
	}

	private function _getProductAttributId($reference)
	{
		$product_id = Db::getInstance()->getValue('SELECT id_product FROM '._DB_PREFIX_.'product_attribute WHERE artifact_reference = "'.$reference.'"');
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : IN SELECT PRODUCT_ATTRIBUTE FUNCTION _getProductAttributId ', Db::getInstance()->getMsgError());
			}
		}

		return $product_id;
	}

	private function _produitComposite($produitComposite, $product_id)
	{
		//configuration
		$value = Db::getInstance()->getValue('SELECT value FROM '._DB_PREFIX_.'configuration WHERE name="PS_PACK_FEATURE_ACTIVE"');
		if ($value == 0) {
			$addTab = array('value' => 1);
			Db::getInstance()->update('configuration', $addTab, 'name="PS_PACK_FEATURE_ACTIVE"');
			//Db::getInstance()->autoExecute(_DB_PREFIX_.'configuration', $addTab, 'UPDATE', 'id_configuration=147');
			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : SELECT CONFIGURATION IN FUNCTION _produitComposite ', Db::getInstance()->getMsgError());
				}
			}
		}
		//gestion des pack :BD
		foreach ($produitComposite as $key => $ProduitComposition) {
			$id_product_attribute_item = ($this->_getProductAttributId($ProduitComposition['ComposeReference']) !=0) ?  $this->_getProductAttributId($ProduitComposition['ComposeReference']) : 0;
			if(!empty($id_product_attribute_item)) {
				$combination= new Combination($id_product_attribute_item);
				$id_product_item=$combination->id_product;
			}
			else {
				$id_product_item = ($this->_getProductId($ProduitComposition['ComposeReference']) != 0) ? $this->_getProductId($ProduitComposition['ComposeReference']) : 0;
			}
			$quantity = Db::getInstance()->getValue('SELECT quantity   FROM '._DB_PREFIX_.'pack WHERE id_product_pack = '.$product_id.' AND id_product_item='.$id_product_item.' AND id_product_attribute_item='.$id_product_attribute_item);

			if (empty($quantity)) {
				$addTabComposit = array(
					'id_product_pack' => (int)$product_id,
					'id_product_item' => (int)$id_product_item,
					'id_product_attribute_item' => (int)$id_product_attribute_item,
					'quantity' => pSQL($ProduitComposition['ComposeQuantite']),
				);
				Db::getInstance()->insert('pack', $addTabComposit);
				//Db::getInstance()->autoExecute(_DB_PREFIX_.'pack', $addTabComposit, 'INSERT');

				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT PACK IN FUNCTION _produitComposite ', Db::getInstance()->getMsgError());
				}

				//Db::getInstance()->autoExecute(
				Db::getInstance()->update(
					'product',
					array('cache_is_pack' => 1),
					'id_product= '.(int)$product_id
				);

				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : UPDATE PRODUCT IN FUNCTION _produitComposite ', Db::getInstance()->getMsgError());
				}
			} elseif ($quantity != $ProduitComposition['ComposeQuantite']) {
				//Db::getInstance()->autoExecute(
				Db::getInstance()->update(
					'pack',
					array('quantity' => $ProduitComposition['ComposeQuantite']),
					'id_product_pack = '.(int)$product_id.' 
					AND id_product_item='.(int)$id_product_item .' 
					AND id_product_attribute_item='.(int)$id_product_attribute_item
				);
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : UPDATE PACK IN FUNCTION _produitComposite ', Db::getInstance()->getMsgError());
				}
			}
		}
		//fin gestion pack :BD
	}

	private function _importSoldeProduits($id_product, $soldes)
	{
		/*
		$specPrice = new SpecificPrice();
		$specPrice->deleteByProductId($id_product);
		*/
		// let's check if any rebate is related to the product
		$targetProduct = false;

		foreach ($soldes as $solde) {
			if (!isset($solde['attribute'])) {
				$targetProduct = true;
				$this->_insertSpecificPrice($id_product, $solde, 0, 0);
			}
		}
		// do we need to insert a priority...
		if ($targetProduct == true) {
			Db::getInstance()->Execute(
				'INSERT IGNORE INTO '._DB_PREFIX_.'specific_price_priority (`id_product`, `priority`)
				VALUES (
					"'.(int)($id_product).'",
					"id_shop;id_currency;id_country;id_group"
				)'
			);

			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT SPECIFIC_PRICE_RULE IN FUNCTION _importSoldeProduits ', Db::getInstance()->getMsgError());
				}
			}
		}
	}

	private function _importSoldeDeclinaisons($id_product, $id_product_attribut, $soldes)
	{
		//$arrayProductAttributeResult;

		//$id_specific_price_rule = $this->_insertSolde($soldes);
		//$id_specific_price_rule_condition_group=$this->insertConditionGroup($id_specific_price_rule);

		foreach ($soldes as $solde) {
			$this->_insertSpecificPrice($id_product, $solde, 0, $id_product_attribut);
		}


		// insert specific price priority !
		Db::getInstance()->Execute(
			'INSERT IGNORE INTO '._DB_PREFIX_.'specific_price_priority (`id_product`, `priority`)
			VALUES (
				"'.(int)($id_product).'",
				"id_shop;id_currency;id_country;id_group"
			)'
		);

		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT SPECIFIC_PRICE_RULE IN FUNCTION _importSoldeProduits ', Db::getInstance()->getMsgError());
			}
		}
	}

	/**
	* insertion dans la table specific_price_rule
	*
	* @param $solde
	* @return
	*/
	private function _insertSolde($solde)
	{
		$addTab = array(
			'name' => 'artifact',
			'id_shop' => (int)Context::getContext()->shop->id,
			'id_currency' => 1,
			'id_country' => 0,
			'id_group' => 0,
			'from_quantity' => (isset($solde['quantité'])) ? (int)$solde['quantité'] :  1,
			'price' => -1,
			'reduction' => ($solde['type'] === 'percentage') ? $solde['value'] * 100 : $solde['value'],
			'reduction_type' => ($solde['type'] === 'pourcent') ? 'percentage' : 'amount',
			'from' => (isset($solde['SoldeDateDebut'])) ? $solde['SoldeDateDebut'] : '',
			'to' => (isset($solde['SoldeDateFin'])) ? $solde['SoldeDateFin'] : '',
		);
		Db::getInstance()->insert('specific_price_rule', $addTab);
		//Db::getInstance()->autoExecute(_DB_PREFIX_.'specific_price_rule', $addTab, 'INSERT');
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT SPECIFIC_PRICE_RULE IN FUNCTION _insertSolde ', Db::getInstance()->getMsgError());
			}
		}

		return Db::getInstance()->Insert_ID();
	}

	/**
	* insertion dans la table specific_price_rule_condition_group
	*
	* @param $id_specific_price_rule
	* @return
	*/
	private function insertConditionGroup($id_specific_price_rule)
	{
		$addTab = array(
			'id_specific_price_rule' => (int)$id_specific_price_rule,
		);
		Db::getInstance()->insert('specific_price_rule_condition_group', $addTab);
		//Db::getInstance()->autoExecute(_DB_PREFIX_.'specific_price_rule_condition_group', $addTab, 'INSERT');
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT SPECIFIC_PRICE_RULE_GROUP IN FUNCTION insertConditionGroup ', Db::getInstance()->getMsgError());
			}
		}

		return Db::getInstance()->Insert_ID();
	}

	/**
	* insertion dans la table specific_price_rule_condition
	*
	* @param $rules_symbol
	* @param $id_specific_price_rule_condition_group
	* @return
	*/
	private function _insertCondition($rules_symbol, $id_specific_price_rule_condition_group)
	{
		$sql = 'SELECT * FROM '
		._DB_PREFIX_."artefact_conversions_rules WHERE imports_conversions_rules_symbol ='".$rules_symbol."'";
		if ($row = Db::getInstance()->getRow($sql)) {
			$type = $row['imports_conversions_type'];
			$value = $row['imports_conversions_rules_original'];
		}
		$addTab = array(
			'id_specific_price_rule_condition_group' => (int)$id_specific_price_rule_condition_group,
			'type' => $type,
			'value' => $value,
		);
		Db::getInstance()->insert('specific_price_rule_condition', $addTab);
		//Db::getInstance()->autoExecute(_DB_PREFIX_.'specific_price_rule_condition', $addTab, 'INSERT');
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT SPECIFIC_PRICE_RULE_CONDITION IN FUNCTION _insertCondition ', Db::getInstance()->getMsgError());
			}
		}

		return $value;
	}

	/**
	* insertion dans la table specific_price
	*
	* @param $id_specific_price_rule
	* @param $id_product
	* @param $id_product_attribut
	* @param $solde
	* @return
	*/
	private function _insertSpecificPrice($id_product, $solde, $id_specific_price_rule = 0, $id_product_attribut = 0)
	{
		$addTab = array(
			'id_specific_price_rule' => $id_specific_price_rule,
			'id_cart' => 0,
			'id_product' => $id_product,
			'id_shop' => (int)Context::getContext()->shop->id,
			'id_shop_group' => 0,
			'id_currency' => 1,
			'id_country' => 0,
			'id_group' => 0,
			'id_customer' => 0,
			'id_product_attribute' => $id_product_attribut,
			'price' => -1,
			'from_quantity' => (isset($solde['quantite'])) ? (int)$solde['quantite'] : 1,
			'reduction' => $solde['value'],
			'reduction_type' => ($solde['type'] === 'pourcent') ? 'percentage' : 'amount',
			'from' => (isset($solde['SoldeDateDebut'])) ? $solde['SoldeDateDebut'] : '',
			'to' => (isset($solde['SoldeDateFin'])) ? $solde['SoldeDateFin'] : '',
		);

		Db::getInstance()->insert('specific_price', $addTab);
		//Db::getInstance()->autoExecute(_DB_PREFIX_.'specific_price', $addTab, 'INSERT');
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : INSERT SPECIFIC_PRICE IN FUNCTION _insertSpecificPrice ', Db::getInstance()->getMsgError());
			}
		}
	}

	/**
	* return ensemble des id_product_attribute qui ont comme declinaison  id_attribute entrant
	*
	* @param $id_attribute
	* @return arrayAttribut
	*/
	private function _getProductAttribute($id_attribute)
	{
		$sql = 'SELECT id_product_attribute FROM '._DB_PREFIX_.'product_attribute_combination WHERE id_attribute='.$id_attribute;
		$arrayAttribut = array();
		if ($results = Db::getInstance()->ExecuteS($sql)) {
			foreach ($results as $row) {
				$arrayAttribut[] = $row['id_product_attribute'];
			}
		}
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : SELECT PRODUCT_ATTRIBUTE_COMBINATION IN FUNCTION _getProductAttribute ', Db::getInstance()->getMsgError());
			}
		}
		return $arrayAttribut;
	}

	/**
	* return  id_product à partir  id_product_attribut
	*
	* @param $id_product_attribut
	* @return id_product
	*/
	private function _getIdProduct($id_product_attribut)
	{
		$id_product = Db::getInstance()->getValue(
			'SELECT id_product FROM '._DB_PREFIX_.'product_attribute WHERE id_product_attribute = "'.$id_product_attribut.'"'
		);
		if (Db::getInstance()->getMsgError()) {
			if (isset($this->_logger)) {
				$this->_logger->logInfo('SQL ERROR  CLASS PRODUCT : SELECT PRODUCT_ATTRIBUTE IN FUNCTION _getIdProduct ', Db::getInstance()->getMsgError());
			}
		}

		return $id_product;
	}

	private function checkAttributSchemeColorSize($attributs)
	{
		if (!empty($attributs) && count($attributs) == 2) {
			if (($attributs[0] == 'taille' && $attributs[1] == 'couleur') || ($attributs[1] == 'taille' && $attributs[0] == 'couleur')) {
				return true;
			}
		}

		return false;
	}

	private function indexOfRef($data, $couleur)
	{
		$index = array();
		foreach ($data as $key => $values) {
			if (isset($values['reference']) && is_array($values['reference'])) {
				foreach ($values['reference'] as $value) {
					$pos = strpos($value, $couleur);
					if ($pos) {
						$index[] = $key;
					}
				}
			}
		}
		return $index;
	}

	private function isDateBetweenDates(DateTime $date, DateTime $startDate, DateTime $endDate) {
		return $date > $startDate && $date < $endDate;
	}

	private function inArrayR($needle, $haystack, $strict = false)
	{
		foreach ($haystack as $item) {
			if (($strict ? $item === $needle : $item == $needle) || (is_array($item) && $this->inArrayR($needle, $item, $strict))) {
				return true;
			}
		}
		return false;
	}


	private function truncateWord($word, $taille)
	{
		if (Tools::strlen($word) < $taille) {
			return $word;
		} else {
			$taille = strrpos($word, ' ', $taille - Tools::strlen($word));
			if (Db::getInstance()->getMsgError()) {
				if (isset($this->_logger)) {
					$this->_logger->logInfo('ERREUR LENGHT OF WORD : '.$word);
				}
			}
			return Tools::substr($word, 0, $taille);
		}
	}

	private function _productShortDesc($desc, $lang)
	{
		$prodDesc = $desc;
		$description = array(
			'short' => '',
			'long' => '',
		);
		$mark = false;

		if (!empty($desc)) {
			$tmp = explode('...</p><p></p>', $desc);
			if (count($tmp) > 1) {
				$prodDesc = $tmp[0];
				$mark = true;
				$description['long'] = preg_replace('/<p>\p{Z}*<\/p>/u', '', $desc);
				//$description['long'] = preg_replace('/<p>\p{Z}*<\/p>/u', '', $tmp[1]);
			} else {
				if (Tools::strlen($desc) > 125) {
					$prodDesc = $this->_tokenTruncate($desc, 125);
					$description['long'] = preg_replace('/<p>\p{Z}*<\/p>/u', '', $desc);
					$mark = true;
				} else {
					$description['short'] = $desc;
				}
			}

			/*
			if ($mark == true) {
				$moreInformations = ($lang == '1') ? 'Plus d\'informations' : 'More information';
				$prodDesc .= '<p><a class="btn" style="font-size:13px;padding:7px 12px 5px 12px;border:1px solid #ccc;" href="#tab2">'.$moreInformations.'</a></p>';
				$description['short'] = $prodDesc;
			}
			*/
			unset($tmp);
		}
		return $description;
	}

	private function _tokenTruncate($text, $length = 100, $options = array())
	{
		$default = array(
			'ending' => '...', 'exact' => false, 'html' => true,
		);
		$options = array_merge($default, $options);
		extract($options);

		if ($html) {
			if (mb_strlen(preg_replace('/<.*?>/', '', $text)) <= $length) {
				return $text;
			}

			$totalLength = mb_strlen(strip_tags($ending));
			$openTags = array();
			$truncate = '';

			preg_match_all('/(<\/?([\w+]+)[^>]*>)?([^<>]*)/', $text, $tags, PREG_SET_ORDER);
			foreach ($tags as $tag) {
				if (!preg_match('/img|br|input|hr|area|base|basefont|col|frame|isindex|link|meta|param/s', $tag[2])) {
					if (preg_match('/<[\w]+[^>]*>/s', $tag[0])) {
						array_unshift($openTags, $tag[2]);
					} elseif (preg_match('/<\/([\w]+)[^>]*>/s', $tag[0], $closeTag)) {
						$pos = array_search($closeTag[1], $openTags);
						if ($pos !== false) {
							array_splice($openTags, $pos, 1);
						}
					}
				}
				$truncate .= $tag[1];

				$contentLength = mb_strlen(preg_replace('/&[0-9a-z]{2,8};|&#[0-9]{1,7};|&#x[0-9a-f]{1,6};/i', ' ', $tag[3]));
				if ($contentLength + $totalLength > $length) {
					$left = $length - $totalLength;
					$entitiesLength = 0;
					if (preg_match_all('/&[0-9a-z]{2,8};|&#[0-9]{1,7};|&#x[0-9a-f]{1,6};/i', $tag[3], $entities, PREG_OFFSET_CAPTURE)) {
						foreach ($entities[0] as $entity) {
							if ($entity[1] + 1 - $entitiesLength <= $left) {
								--$left;
								$entitiesLength += mb_strlen($entity[0]);
							} else {
								break;
							}
						}
					}

					$truncate .= mb_substr($tag[3], 0, $left + $entitiesLength);
					break;
				} else {
					$truncate .= $tag[3];
					$totalLength += $contentLength;
				}

				if ($totalLength >= $length) {
					break;
				}
			}
		} else {
			if (mb_strlen($text) <= $length) {
				return $text;
			} else {
				$truncate = mb_substr($text, 0, $length - mb_strlen($ending));
			}
		}
		if (!$exact) {
			$spacepos = mb_strrpos($truncate, ' ');
			if (isset($spacepos)) {
				if ($html) {
					$bits = mb_substr($truncate, $spacepos);
					preg_match_all('/<\/([a-z]+)>/', $bits, $droppedTags, PREG_SET_ORDER);
					if (!empty($droppedTags)) {
						foreach ($droppedTags as $closingTag) {
							if (!in_array($closingTag[1], $openTags)) {
								array_unshift($openTags, $closingTag[1]);
							}
						}
					}
				}
				$truncate = mb_substr($truncate, 0, $spacepos);
			}
		}
		$truncate .= $ending;

		if ($html) {
			foreach ($openTags as $tag) {
				$truncate .= '</'.$tag.'>';
			}
		}

		return $truncate;
	}
}
?>