<?php
/**
* NOTICE OF LICENSE.
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software
* and associated documentation files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge, publish, distribute,
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial
* portions of the Software.
*
* The software is provided "as is", without warranty of any kind, express or implied including 
* but not limited to the warranties of merchantability, fitness for a particular purpose and
* noninfrigement. In no event shall the authors or copyright holdes be liable for any claim, 
* damages or the liability, whether in an action of contract, tort or otherwise, arising from, 
* out of or in connection with the software or the use or other dealings in the software.
*
* @author    Diabolo SARL <ecommerce@diazol.com>
* @copyright 2011 - 2016 Diabolo SARL
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

require_once _PS_ROOT_DIR_.'/config/config.inc.php';

class SynchActive extends Exception
{
    private $racine;
    private $source;
    private $monZip;

    public function __construct()
    {
        $this->racine = _PS_ROOT_DIR_.'/arte-fact/bkp/';
        $this->source = _PS_ROOT_DIR_.'/arte-fact/inputs/';
        $this->monZip = new ZipArchive();
    }

    public function activate()
    {
        if ($this->existFile()) {
            if (Configuration::get('PS_ARTIFACT_ACTIVE_SYNC') == true) {
                $this->isDirCreate($this->racine);
                $this->sauvegardeSynch();
            }
        }
    }

    private function sauvegardeSynch()
    {
        try {
            $path = $this->createDirectories();
            $date = date('Y-m-d_H_i_s');
            $this->dirToZip($path.'/'.$date.'.zip');
        } catch (SynchException $ex) {
            echo 'Error : '.$ex->getMessage();
        }
    }

    private function createDirectories()
    {
        try {
            $year = date('Y');
            $this->isDirCreate($this->racine.$year);
            $month = date('m');
            $this->isDirCreate($this->racine.$year.'/'.$month);

            return $this->racine.$year.'/'.$month;
        } catch (SynchException $ex) {
            echo 'Error : '.$ex->getMessage();
        }
    }

    private function isDirCreate($path)
    {
        if (is_dir($path)) {
            return true;
        } else {
            if (mkdir($path)) {
                return true;
            } else {
                return false;
            }
        }
    }
    private function dirToZip($path)
    {
        if ($this->monZip->open($path)) {
            if ($this->monZip->open($path, ZipArchive::CREATE)) {
                $this->monZip->addEmptyDir('files_xml');
                $this->monZip->addEmptyDir('images/arbre');
                $this->monZip->addEmptyDir('images/marques');
                $this->monZip->addEmptyDir('images/produits');
                $this->setFile('files_xml/');
                $this->setFile('images/arbre/');
                $this->setFile('images/marques/');
                $this->setFile('images/produits/');
                $this->monZip->close();
            } else {
                echo 'exemple.zip error';
            }
        }
    }

    private function setFile($repertoire)
    {
        $fichiers = scandir($this->source.$repertoire);
        unset($fichiers[0], $fichiers[1]);//Suppression de . et ..
        foreach ($fichiers as $f) {
            if (!$this->monZip->addFile($this->source.$repertoire.$f, $repertoire.$f)) {
                echo 'Erreur sur le fichier '.$f.'';
            }
        }
    }

    private function existFile()
    {
        $repertoire = $this->source.'files_xml/';
        if (count(scandir($repertoire)) > 2) {
            return true;
        } else {
            return false;
        }
    }
}
