<?php
/**
* NOTICE OF LICENSE.
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Permission is hereby granted, free of charge, to any person obtaining a copy of this software
* and associated documentation files (the "Software"), to deal in the Software without restriction,
* including without limitation the rights to use, copy, modify, merge, publish, distribute,
* sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
*
* The above copyright notice and this permission notice shall be included in all copies or substantial
* portions of the Software.
*
* The software is provided "as is", without warranty of any kind, express or implied including 
* but not limited to the warranties of merchantability, fitness for a particular purpose and
* noninfrigement. In no event shall the authors or copyright holdes be liable for any claim, 
* damages or the liability, whether in an action of contract, tort or otherwise, arising from, 
* out of or in connection with the software or the use or other dealings in the software.
*
* @author    Diabolo SARL <ecommerce@diazol.com>
* @copyright 2011 - 2016 Diabolo SARL
* @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*/

class ArtifactImportTaxes
{
    private $_conversions;
    private $_languages;
    private $_logger;
    private $_zoneEuro = array(
        'AT', 'BE', 'BG', 'CY', 'CZ', 'DE', 'DK', 'EE', 'ES', 'FI', 'FR', 'GB', 'GR', 'HU', 'HR', 'IE', 'IT', 'LU', 'LT', 'LV', 'MT', 'NL', 'PL', 'PT', 'RO', 'SE', 'SI', 'SK',
    );

    public function import($obj)
    {
        if (empty($obj)) {
            return false;
        }


        $log = new ArtifactImport();

        if (isset($log->logger)) {
            $this->_logger = $log->logger;
        }


        $this->_getConversions();
        $this->_importTaxes($obj);
    }

    private function _getConversions()
    {
        $conversions = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'artefact_conversions_rules`', false, false);

        while ($row = Db::getInstance()->nextRow($conversions)) {
            $this->_conversions[$row['imports_conversions_type']][$row['imports_conversions_rules_symbol']] = $row['imports_conversions_rules_original'];
        }

        $languages = Db::getInstance()->ExecuteS('SELECT * FROM `'._DB_PREFIX_.'lang`', false, false);

        while ($row = Db::getInstance()->nextRow($languages)) {
            $this->_languages[Tools::strtolower($row['iso_code'])] = $row['id_lang'];
        }
    }

    private function _importTaxes($obj)
    {
        if (empty($obj)) {
            return false;
        }

        $data = array();
        $taxes = array();

        foreach ($obj as $values) {
            $data['action'] = (string)$values->attributes()->action;
            $data['id'] = (int)$values->TVAId;
            $data['TVADescription'] = (string)$values->TVADescription;
            $data['TVAValeur'] = ((float)$values->TVAValeur) * 100;
            $data['TVAZone'] = (string)$values->TVAValeur->attributes()->zone;

            $taxes[] = $data;
        }

        foreach ($taxes as $tax) {
            if ($tax['action'] == 'delete') {
                $this->_deleteTax($tax['id']);
                continue;
            }

            $this->_importTax($tax);
        }
    }

    private function _importTax($tax)
    {
        $date = date('Y-m-d H:i:s');
        $tax_id = !empty($this->_conversions['taxes'][$tax['id']]) ? $this->_conversions['taxes'][$tax['id']] : 0;
        if (empty($tax_id)) {
            $addTab = array(
                'id_tax' => 0,
                'active' => 1,
                'rate' => pSQL($tax['TVAValeur']),
            );

            Db::getInstance()->insert('tax', $addTab);
            //Db::getInstance()->autoExecute(_DB_PREFIX_.'tax', $addTab, 'INSERT');
            if (Db::getInstance()->getMsgError()) {
                if (isset($this->_logger)) {
                    $this->_logger->logInfo('SQL ERROR  CLASS TAXES : INSERT TAX  ', Db::getInstance()->getMsgError());
                }
            } else {
                if (isset($this->_logger)) {
                    $this->_logger->logInfo('CLASS TAXES : INSERT TAX  ', $tax['TVAValeur']);
                }
            }

            $tax_id = Db::getInstance()->Insert_ID();

            $addTab = array(
                'imports_conversions_rules_symbol' => $tax['id'],
                'imports_conversions_rules_original' => $tax_id,
                'imports_conversions_type' => 'taxes',
            );

            Db::getInstance()->insert('artefact_conversions_rules', $addTab);
            //Db::getInstance()->autoExecute(_DB_PREFIX_.'artefact_conversions_rules', $addTab, 'INSERT');
            if (Db::getInstance()->getMsgError()) {
                if (isset($this->_logger)) {
                    $this->_logger->logInfo('SQL ERROR  CLASS TAXES : INSERT artefact_conversions_rules  ', Db::getInstance()->getMsgError());
                }
            } else {
                if (isset($this->_logger)) {
                    $this->_logger->logInfo('CLASS TAXES : INSERT artefact_conversions_rules tax  ', $tax_id);
                }
            }

            $this->_conversions['taxes'][$tax['id']] = $tax_id;
        } else {
            $addTab = array(
                'active' => 1,
                'rate' => pSQL($tax['TVAValeur']),
            );
            Db::getInstance()->update('tax', $addTab, 'id_tax = '.$tax_id);
            //Db::getInstance()->autoExecute(_DB_PREFIX_.'tax', $addTab, 'UPDATE', 'id_tax = '.$tax_id);
            if (Db::getInstance()->getMsgError()) {
                if (isset($this->_logger)) {
                    $this->_logger->logInfo('SQL ERROR  CLASS TAXES : UPDATE TAX  ', Db::getInstance()->getMsgError());
                }
            } else {
                if (isset($this->_logger)) {
                    $this->_logger->logInfo('CLASS TAXES : UPDATE TAX  ', $tax_id);
                }
            }
        }

        foreach ($this->_languages as $code => $language_id) {
            Db::getInstance()->Execute('INSERT INTO '._DB_PREFIX_.'tax_lang (`id_tax`, `id_lang`, `name`) 
											VALUES (
												"'.pSQL($tax_id).'", 
												"'.pSQL($language_id).'",
												"TVA FR '.pSQL($tax['TVADescription']).'"
											)
											ON DUPLICATE KEY UPDATE 
											name        = "TVA FR '.pSQL($tax['TVADescription']).'"');
            if (Db::getInstance()->getMsgError()) {
                if (isset($this->_logger)) {
                    $this->_logger->logInfo('SQL ERROR  CLASS TAXES : INSERT TAX_LANG  ', Db::getInstance()->getMsgError());
                }
            }
        }

        $tax_rules_group_id = !empty($this->_conversions['taxes_group'][$tax['id']]) ? $this->_conversions['taxes_group'][$tax['id']] : 0;

        if (empty($tax_rules_group_id)) {
            $addTab = array(
                'id_tax_rules_group' => 0,
                'active' => 1,
                'name' => pSQL('TVA FR '.$tax['TVADescription']),
                'date_add' => date('Y-m-d H:i:s'),
                'date_upd' => date('Y-m-d H:i:s')
            );

            Db::getInstance()->insert('tax_rules_group', $addTab);
            //Db::getInstance()->autoExecute(_DB_PREFIX_.'tax_rules_group', $addTab, 'INSERT');
            $tax_rules_group_id = Db::getInstance()->Insert_ID();

            $addTab = array(
                'imports_conversions_rules_symbol' => $tax['id'],
                'imports_conversions_rules_original' => $tax_rules_group_id,
                'imports_conversions_type' => 'taxes_group',
            );

            Db::getInstance()->insert('artefact_conversions_rules', $addTab);
            //Db::getInstance()->autoExecute(_DB_PREFIX_.'artefact_conversions_rules', $addTab, 'INSERT');
            if (Db::getInstance()->getMsgError()) {
                if (isset($this->_logger)) {
                    $this->_logger->logInfo('SQL ERROR  CLASS TAXES : INSERT artefact_conversions_rules  ', Db::getInstance()->getMsgError());
                }
            } else {
                if (isset($this->_logger)) {
                    $this->_logger->logInfo('CLASS TAXES : INSERT artefact_conversions_rules taxes_group  ', $tax['id']);
                }
            }

            $this->_conversions['taxes_group'][$tax['id']] = $tax_rules_group_id;
        } else {
            $addTab = array(
                'active' => 1,
                'name' => pSQL('TVA FR '.$tax['TVADescription']),
                'date_upd' => date('Y-m-d H:i:s')
            );
            Db::getInstance()->update('tax_rules_group', $addTab, 'id_tax_rules_group = '.$tax_rules_group_id);
            //Db::getInstance()->autoExecute(_DB_PREFIX_.'tax_rules_group', $addTab, 'UPDATE', 'id_tax_rules_group = '.$tax_rules_group_id);
            if (Db::getInstance()->getMsgError()) {
                if (isset($this->_logger)) {
                    $this->_logger->logInfo('SQL ERROR  CLASS TAXES : UPDATE TAX_RULES_GROUP  ', Db::getInstance()->getMsgError());
                }
            } else {
                if (isset($this->_logger)) {
                    $this->_logger->logInfo('CLASS TAXES : UPDATE TAX_RULES_GROUP  ', $tax_rules_group_id);
                }
            }
        }

        if ($tax['action'] == 'create') {
			if($tax['TVAZone']=='EU') {
                foreach ($this->_zoneEuro as $euroCountry) {
                    Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'tax_rule WHERE `id_tax_rules_group` = "'.pSQL($tax_rules_group_id).'" AND id_country = "'.pSQL(Country::getByIso($euroCountry)).'"');

                    if (Db::getInstance()->getMsgError()) {
                        if (isset($this->_logger)) {
                            $this->_logger->logInfo('SQL ERROR  CLASS TAXES : DELETE TAX_RULES ', Db::getInstance()->getMsgError());
                        }
                    } else {
                        if (isset($this->_logger)) {
                            $this->_logger->logInfo('CLASS TAXES : DELETE TAX_RULES FOR ', $euroCountry);
                        }
                    }
                    Db::getInstance()->Execute('INSERT IGNORE INTO '._DB_PREFIX_.'tax_rule (`id_tax_rules_group`, `id_country`, `id_state`, `id_tax`,`behavior`,`zipcode_from`, `zipcode_to`, `description`) 
														VALUES (
															"'.pSQL($tax_rules_group_id).'", 
															"'.pSQL(Country::getByIso($euroCountry)).'",
															"0",
															"'.pSQL($tax_id).'",
															"0",
															"0",
															"0",
															""
														)');

                    if (Db::getInstance()->getMsgError()) {
                        if (isset($this->_logger)) {
                            $this->_logger->logInfo('SQL ERROR  CLASS TAXES : INSERT TAX_RULES  ', Db::getInstance()->getMsgError());
                        }
                    } else {
                        if (isset($this->_logger)) {
                            $this->_logger->logInfo('CLASS TAXES : INSERT TAX_RULES FOR ', $euroCountry);
                        }
                    }
                }

                Db::getInstance()->Execute('INSERT IGNORE INTO '._DB_PREFIX_.'tax_rules_group_shop (`id_tax_rules_group`, `id_shop`) 
												 SELECT "'.pSQL($tax_rules_group_id).'", id_shop FROM '._DB_PREFIX_.'shop');
                if (Db::getInstance()->getMsgError()) {
                    if (isset($this->_logger)) {
                        $this->_logger->logInfo('SQL ERROR  CLASS TAXES : INSERT TAX_RULES_GROUP_SHOP  ', Db::getInstance()->getMsgError());
                    }
                } else {
                    if (isset($this->_logger)) {
                        $this->_logger->logInfo('CLASS TAXES : INSERT TAX_RULES_GROUP_SHOP FOR ', $tax_rules_group_id);
                    }
                }
            } else {
                Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'tax_rule WHERE `id_tax_rules_group` = "'.pSQL($tax_rules_group_id).'" AND id_country = "'.pSQL(Country::getByIso($tax['TVAZone'])).'"');
                if (Db::getInstance()->getMsgError()) {
                    if (isset($this->_logger)) {
                        $this->_logger->logInfo('SQL ERROR  CLASS TAXES : DELETE TAX_RULES ', Db::getInstance()->getMsgError());
                    }
                } else {
                    if (isset($this->_logger)) {
                        $this->_logger->logInfo('CLASS TAXES : DELETE TAX_RULES FOR ', $tax['TVAZone']);
                    }
                }

                Db::getInstance()->Execute('INSERT IGNORE INTO '._DB_PREFIX_.'tax_rule (`id_tax_rules_group`, `id_country`, `id_state`, `id_tax`,`behavior`,`zipcode_from`, `zipcode_to`, `description`) 
													VALUES (
														"'.pSQL($tax_rules_group_id).'", 
														"'.pSQL(Country::getByIso($tax['TVAZone'])).'",
														"0",
														"'.pSQL($tax_id).'",
														"0",
														"0",
														"0",
														""
													)');

                if (Db::getInstance()->getMsgError()) {
                    if (isset($this->_logger)) {
                        $this->_logger->logInfo('SQL ERROR  CLASS TAXES : INSERT TAX_RULES  ', Db::getInstance()->getMsgError());
                    }
                } else {
                    if (isset($this->_logger)) {
                        $this->_logger->logInfo('CLASS TAXES : INSERT TAX_RULES FOR ', $tax['TVAZone']);
                    }
                }
                Db::getInstance()->Execute('INSERT IGNORE INTO '._DB_PREFIX_.'tax_rules_group_shop (`id_tax_rules_group`, `id_shop`) 
												 SELECT "'.pSQL($tax_rules_group_id).'", id_shop FROM '._DB_PREFIX_.'shop');
                if (Db::getInstance()->getMsgError()) {
                    if (isset($this->_logger)) {
                        $this->_logger->logInfo('SQL ERROR  CLASS TAXES : INSERT TAX_RULES_GROUP_SHOP  ', Db::getInstance()->getMsgError());
                    }
                } else {
                    if (isset($this->_logger)) {
                        $this->_logger->logInfo('CLASS TAXES : INSERT TAX_RULES_GROUP_SHOP FOR ', $tax_rules_group_id);
                    }
                }
            }
        }
    }

    private function _deleteTax($tax_id)
    {
        $tax_rules_group_id = !empty($this->_conversions['taxes_group'][$tax_id]) ? $this->_conversions['taxes_group'][$tax_id] : 0;
        $tax_id = !empty($this->_conversions['taxes'][$tax_id])       ? $this->_conversions['taxes'][$tax_id]       : 0;

        if (empty($tax_id)) {
            return false;
        }

        Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'tax_lang         WHERE id_tax = '.$tax_id);
        Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'tax              WHERE id_tax = '.$tax_id);
        Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'tax_rule         WHERE id_tax = '.$tax_id);
        Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'tax_rules_group  WHERE id_tax_rules_group = '.$tax_rules_group_id);

        // Delete conversion
        Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'artefact_conversions_rules 
									  WHERE imports_conversions_type = "taxes" AND imports_conversions_rules_original = '.$tax_id);
        Db::getInstance()->Execute('DELETE FROM '._DB_PREFIX_.'artefact_conversions_rules 
									  WHERE imports_conversions_type = "taxes_group" AND imports_conversions_rules_original = '.$tax_rules_group_id);
        unset($this->_conversions['taxes'][$tax_id]);
    }
}
